/*
 * AbstractElement.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamAttribute;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.StreamObjectVisitor;
import org.jabberstudio.jso.StreamText;
import org.jabberstudio.jso.format.LocaleFormat;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.util.Utilities;

import org.jaxen.JaxenException;
import org.jaxen.SimpleVariableContext;
import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import net.outer_planes.jso.io.XMLReader;
import net.outer_planes.jso.io.XMLWriter;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public abstract class AbstractElement extends AbstractNode implements StreamElement {
    //Inner-classes
    protected static class LocaleWrapper {
        //Fields
        private Locale          _Locale;
        private String          _Spec;
        
        //Constructors
        public LocaleWrapper(Locale l) {
            _Locale = l;
        }
        
        //Methods
        public final Locale getLocale() {
            return _Locale;
        }
        public final String toString() {
            if (_Spec == null)
                _Spec = LocaleFormat.getInstance().format(getLocale());
            return _Spec;
        }
        public final boolean equals(Object cmp) {
            return ((cmp != null) ? toString().equals(cmp) : false);
        }
        
        public static final LocaleWrapper fromString(String spec) {
            Locale  lang = null;

            try {
                lang = (Locale)LocaleFormat.getInstance().parseObject(spec);
            } catch (Exception e) {
                Logger.getLogger(AbstractElement.class).warn("Could not parse xml:lang into Locale", e);
            }
            
            return ((lang != null) ? new LocaleWrapper(lang) : null);
        }
    }
    protected static class ObjectAttribute extends AbstractAttribute {
        //Fields
        private NSI         _Name;
        private Object      _Val;

        //Constructors
        public ObjectAttribute(StreamElement p, NSI name, Object val) {
            super(p);
            setNSI(name);
            setObject(val);
        }

        //Methods
        public NSI getNSI() {
            return _Name;
        }
        private void setNSI(NSI name) {
            if (name == null)
                throw new IllegalArgumentException("Name cannot be null");
            if (!Utilities.isValidString(name.getLocalName()))
                throw new IllegalArgumentException("Local name cannot be null");
            if (name.getNamespaceURI() == null)
                name = new NSI(name.getLocalName(), "");
            _Name = name;
        }

        public Object getObject() {
            return _Val;
        }
        public void setObject(Object val) {
            _Val = val;
        }
        
        public String getValue() {
            return (getObject() != null) ? getObject().toString() : "";
        }
        public void setValue(String val) throws IllegalArgumentException {
            setObject(val);
        }
        
        public StreamObject copy(StreamElement p) {
            return new ObjectAttribute(p, getNSI(), getObject());
        }
        
        public String toString() {
            StringBuffer    buffer = new StringBuffer();
            
            buffer.append(getNSI()).append("=").append(getValue());
            
            return buffer.toString();
        }
    }
    
    //"Constants"
    public static final NSI         NON_NSI = new NSI(null, null);
    
    public static final NSI         ATTRNAME_ID = new NSI("id", null);
    public static final NSI         ATTRNAME_XMLLANG = new NSI("lang", Utilities.XML_NAMESPACE);
    
    //Fields
    private Map     _Nss = null;
    private Map     _Attrs = null;
    
    private XPath   _LocalePath;
    
    //Constructors
    public AbstractElement(StreamDataFactory sdf) {
        super(sdf);
    }
    public AbstractElement(StreamElement parent, AbstractElement base) {
        super(base.getDataFactory());
        setParent(parent);
        setNSI(base.getNSI());
        setDeclaredNamespaces(base.getDeclaredNamespaces());
        setAttributes(base.getAttributes());
    }

    //Methods
    public abstract StreamBuilder createBuilder();
    
    protected XPath getLocaleXPath() {
        if (_LocalePath == null) {
            try {
                _LocalePath = getDataFactory().createXPath("*[(not($ln) or local-name()=$ln) and (not($ns) or namespace-uri()=$ns) and locale-match($lang)]");
                _LocalePath.setVariableContext(new SimpleVariableContext());
                Utilities.setupNamespaces(_LocalePath, Collections.singletonMap("app", getNamespaceURI()));
            } catch (SAXPathException saxpe) {
                obtainLogger().error("Could not create Locale-based XPath object", saxpe);
            }
        }
        
        return _LocalePath;
    }
    
    public String getLocalName() {
        return getNSI().getLocalName();
    }
    public String getNamespacePrefix() {
        return getNamespacePrefix(getNamespaceURI());
    }
    public String getNamespaceURI() {
        StreamElement   parent = getParent();
        String          uri = getNSI().getNamespaceURI();

        if ((uri == null) && (parent != null)) {
            uri = (parent instanceof StreamContext) ? parent.getNamespaceURI("") : parent.getNamespaceURI();
        }

        return uri;
    }
    public String getQualifiedName() {
        StringBuffer        buffer = new StringBuffer();
        String              prefix = getNamespacePrefix();
        String              local = getLocalName();
        
        if (prefix != null)
            buffer.append(prefix).append(":");
        buffer.append(local);
        
        return buffer.toString();
    }
    
    public abstract void setNSI(NSI nsi) throws IllegalArgumentException;

    public Set getDeclaredNamespaces() {
        Set     vals = new java.util.TreeSet(new java.util.Comparator() {
            public int compare(Object cmp1, Object cmp2) {
                return cmp1.toString().compareTo(cmp2.toString());
            }
        });
        
        if(_Nss != null) {
            vals.addAll(_Nss.values());
        }
        
        return vals;
    }
    public void setDeclaredNamespaces(Set nss) {
        Iterator        itr = nss.iterator();
        StreamNamespace ns;
        
        if(_Nss == null && itr.hasNext()) {
            _Nss = new java.util.TreeMap();
        }
        
        while (itr.hasNext()) {
            ns = (StreamNamespace)itr.next();
            
            _Nss.put(ns.getPrefix(), ns.copy(this));
        }
    }
    
    public StreamNamespace getNamespaceByURI(String uri) {
        return getNamespaceByURI(uri, true);
    }
    protected StreamNamespace getNamespaceByURI(String uri, boolean deep) {
        
        StreamNamespace ns = null;

        //Validate URI
        if (uri == null)
            uri = "";
        
        if (Utilities.equateStrings(Utilities.XML_NAMESPACE, uri))
            ns = new NamespaceNode(this, Utilities.XML_PREFIX, Utilities.XML_NAMESPACE);
        
        if(_Nss != null) {
            Iterator        nsItr = _Nss.values().iterator();
            //Look for the namespace here
            while ((ns == null) && nsItr.hasNext()) {
                StreamNamespace     test = (StreamNamespace)nsItr.next();

                if (Utilities.equateStrings(test.getURI(), uri))
                    ns = test;
            }
        }

        //Check the parent if necessary
        if ((ns == null) && (getParent() != null) && deep)
            ns = getParent().getNamespaceByURI(uri);

        return ns;
    }
    public StreamNamespace getNamespaceByPrefix(String prefix) {
        return getNamespaceByPrefix(prefix, true);
    }
    protected StreamNamespace getNamespaceByPrefix(String prefix, boolean deep) {
        Set         nss = getDeclaredNamespaces();
        Iterator    nsItr = nss.iterator();
        StreamNamespace ns = null;

        //Validate Prefix
        if (prefix == null)
            prefix = "";
        
        if (Utilities.equateStrings(Utilities.XML_PREFIX, prefix))
            ns = new NamespaceNode(this, Utilities.XML_PREFIX, Utilities.XML_NAMESPACE);
        
        //Look for the namespace here
        while ((ns == null) && nsItr.hasNext()) {
            StreamNamespace     test = (StreamNamespace)nsItr.next();
            
            if (Utilities.equateStrings(test.getPrefix(), prefix))
                ns = test;
        }

        //Check the parent if necessary
        if ((ns == null) && (getParent() != null) && deep)
            ns = getParent().getNamespaceByPrefix(prefix);

        return ns;
    }

    public String getNamespacePrefix(String uri) throws IllegalArgumentException {
        StreamNamespace     ns = getNamespaceByURI(uri);

        return (ns != null) ? ns.getPrefix() : null;
    }
    public String getNamespaceURI(String prefix) throws IllegalArgumentException {
        StreamNamespace     ns = getNamespaceByPrefix(prefix);
        
        return (ns != null) ? ns.getURI() : null;
    }
    public void addNamespace(String prefix, String uri) throws UnsupportedOperationException {
        StreamNamespace     ns, currentNS;
        
        if (prefix == null)
            prefix = "";
        if (uri == null)
            uri = "";

        ns = new NamespaceNode(this, prefix, uri);
        
        //Remove the old before adding the new
        currentNS = getNamespaceByPrefix(ns.getPrefix(), false);
        //if (currentNS != null)
        //    removeNamespace(ns);
        addNamespace(ns);
    }
    public void addNamespace(StreamNamespace ns) {
        if(_Nss == null) {
            _Nss = new java.util.TreeMap();
        }
        _Nss.put(ns.getPrefix(), ns);
    }
    public void removeNamespace(String prefix, String uri) throws UnsupportedOperationException {
        StreamNamespace     ns;
        
        if (uri == null)
            uri = "";
        
        removeNamespace(getNamespaceByURI(uri, false));
    }
    public void removeNamespace(StreamNamespace ns) {
        if(_Nss != null) {
            _Nss.remove(ns.getPrefix());
        }
    }

    public Map getAttributes() {
        if(_Attrs == null) {
            return java.util.Collections.EMPTY_MAP;
        }
        return _Attrs;
    }
    public void setAttributes(Map attrs) {
        Iterator        itr = attrs.values().iterator();
        StreamAttribute attr;
        
        if(_Attrs == null && itr.hasNext()) {
            _Attrs = new java.util.TreeMap(NSI.NS_LOOSE_COMPARATOR);
        }
        while (itr.hasNext()) {
            attr = (StreamAttribute)itr.next();
            
            _Attrs.put(attr.getNSI(), attr.copy(this));
        }
    }
    public String getAttributeValue(String name) throws IllegalArgumentException {
        return getAttributeValue(new NSI(name, null));
    }
    public String getAttributeValue(String name, String ns) throws IllegalArgumentException {
        return getAttributeValue(new NSI(name, ns));
    }
    public String getAttributeValue(NSI name) throws IllegalArgumentException {
        Object  val = getAttributeObject(name);
        
        return (val != null) ? val.toString() : null;
    }

    protected Object getAttributeObject(String name) throws IllegalArgumentException {
        return getAttributeObject(new NSI(name, null));
    }
    protected Object getAttributeObject(String name, String ns) throws IllegalArgumentException {
        return getAttributeObject(new NSI(name, ns));
    }
    protected Object getAttributeObject(NSI name) throws IllegalArgumentException {
        ObjectAttribute     attr = null;

        //Validate name
        if (name == null)
            throw new IllegalArgumentException("Attribute name cannot be null");
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Attribute local name cannot be null or \"\"");

        if(_Attrs != null) {
            if (name.getNamespaceURI() == null)
                name = new NSI(name.getLocalName(), "");
            attr = (ObjectAttribute)_Attrs.get(name);
        }
        
        return (attr != null) ? attr.getObject() : null;
    }

    public void setAttributeValue(String name, String val) throws IllegalArgumentException, UnsupportedOperationException {
        setAttributeValue(new NSI(name, null), val);
    }
    public void setAttributeValue(String name, String ns, String val) throws IllegalArgumentException, UnsupportedOperationException {
        setAttributeValue(new NSI(name, ns), val);
    }
    public void setAttributeValue(NSI name, String val) throws IllegalArgumentException, UnsupportedOperationException {
        setAttributeObject(name, val);
    }
    protected void setAttributeObject(String name, Object val) throws IllegalArgumentException {
        setAttributeObject(new NSI(name, null), val);
    }
    protected void setAttributeObject(String name, String ns, Object val) throws IllegalArgumentException {
        setAttributeObject(new NSI(name, ns), val);
    }
    protected void setAttributeObject(NSI name, Object val) throws IllegalArgumentException {
        ObjectAttribute     attr;
        
        //Validate name
        if (name == null)
            throw new IllegalArgumentException("Attribute name cannot be null");
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Attribute local name cannot be null or \"\"");

        if(_Attrs == null) {
            _Attrs = new java.util.TreeMap(NSI.NS_LOOSE_COMPARATOR);
        }
        if (val == null) {
            if (_Attrs.containsKey(name))
                _Attrs.remove(name);
        } else {
            attr = (ObjectAttribute)_Attrs.get(name);

            if (attr != null)
                attr.setObject(val);
            else {
                if (name.getNamespaceURI() == null)
                    name = new NSI(name.getLocalName(), "");
                _Attrs.put(name, new ObjectAttribute(this, name, val));
            }
        }
    }
    
    public String getLanguage() {
        String      lang = getDeclaredLanguage();
        
        return (Utilities.isValidString(lang)) ? lang : super.getLanguage();
    }
    
    public String getID() {
        return getAttributeValue(ATTRNAME_ID);
    }
    public void setID(String id) {
        setAttributeValue(ATTRNAME_ID, id);
    }
    
    public String getDeclaredLanguage() {
        String      lang = getAttributeValue(ATTRNAME_XMLLANG);
        
        return (lang != null) ? lang : "";
    }
    public void setDeclaredLanguage(String lang) throws UnsupportedOperationException {
        if (!Utilities.isValidString(lang))
            lang = null;
        setAttributeValue(ATTRNAME_XMLLANG, lang);
    }
    
    public Locale getLocale() {
        Locale          lang = getDeclaredLocale();
        
        return ((lang != null) ? lang : super.getLocale());
    }
    public Locale getDeclaredLocale() {
        Object      val = getAttributeObject(ATTRNAME_XMLLANG);
        Locale      lang = null;
        
        if          (val instanceof LocaleWrapper) {
            lang = ((LocaleWrapper)val).getLocale();
        } else if   (val != null) {
            try {
                lang = (Locale)LocaleFormat.getInstance().parseObject(val.toString());
                setDeclaredLocale(lang);
            } catch (Exception e) {
                obtainLogger().warn("Could not parse xml:lang into locale", e);
            }
        }
        
        return lang;
    }
    public void setDeclaredLocale(Locale lang) {
        LocaleWrapper   lw = ((lang != null) ? new LocaleWrapper(lang) : null);
        
        setAttributeObject(ATTRNAME_XMLLANG, lw);
    }
    
    
    public List listText() {
        Iterator        itr = listNodes().iterator();
        List            text = new java.util.LinkedList();
        
        while (itr.hasNext()) {
            StreamNode  node = (StreamNode)itr.next();
            
            if (node instanceof StreamText) {
                text.add(node);
            }
        }
        
        return text;
    }
    public String normalizeText() {
        Iterator        itr = listText().iterator();
        StringBuffer    buffer = new StringBuffer();

        while (itr.hasNext()) {
            StreamText  text = (StreamText)itr.next();

            buffer.append(text.getValue());
        }

        return buffer.toString();
    }
    public String normalizeTrimText() {
        Iterator        itr = listText().iterator();
        StringBuffer    buffer = new StringBuffer();

        while (itr.hasNext()) {
            StreamText  text = (StreamText)itr.next();
            String      val = text.getValue();
            
            buffer.append(val.replaceAll("\\s+", " ").trim());
        }

        return buffer.toString().trim();
    }
    
    public StreamText addText(String txt) {
        StreamDataFactory   sdf;
        StreamText          node = null;
        
        if (Utilities.isValidString(txt)) {
            sdf = getDataFactory();
            node = sdf.createText(txt);

            add(node);
        }
        
        return node;
    }
    
    public void clearText() {
        Iterator        itr = listText().iterator();
        
        while (itr.hasNext()) {
            remove((StreamText)itr.next());
        }
    }

    public List listElements() {
        return listElements(NON_NSI, null);
    }
    public List listElements(Class iface) {
        return listElements(NON_NSI, iface);
    }
    public List listElements(String name) {
        NSI     nsi = (!Utilities.isValidString(name) ? NON_NSI : new NSI(name, null));
        
        return listElements(nsi, null);
    }
    public List listElements(String name, Class iface) {
        NSI     nsi = (!Utilities.isValidString(name) ? NON_NSI : new NSI(name, null));
        
        return listElements(nsi, iface);
    }
    public List listElements(String name, String ns) {
        return listElements(new NSI(name, ns), null);
    }
    public List listElements(String name, String ns, Class iface) {
        return listElements(new NSI(name, ns), iface);
    }
    public List listElements(NSI name) {
        return listElements(name, null);
    }
    public List listElements(NSI name, Class iface) {
        NSI.Comparator  cmp = NSI.LOOSE_COMPARATOR;
        List            elems = listNodes();
        Iterator        itr = elems.iterator();
        boolean         valid;
        
        if (name == null)
            name = NON_NSI;

        while (itr.hasNext()) {
            StreamNode  node = (StreamNode)itr.next();
            
            if (node instanceof StreamElement) {
                StreamElement   test = (StreamElement)node;
                NSI             nsi = new NSI(test.getLocalName(), test.getNamespaceURI());

                valid = (cmp.compare(name, nsi) == 0);
                valid = valid && ((iface == null) || iface.isInstance(test));
                
                if (!valid)
                    itr.remove();
            } else {
                itr.remove();
            }
        }

        return elems;
    }
    
    public StreamElement getFirstElement() {
        return getFirstElement(NON_NSI, null);
    }
    public StreamElement getFirstElement(Class iface) {
        return getFirstElement(NON_NSI, iface);
    }
    public StreamElement getFirstElement(String name) {
        NSI     nsi = (!Utilities.isValidString(name) ? NON_NSI : new NSI(name, null));
        
        return getFirstElement(nsi, null);
    }
    public StreamElement getFirstElement(String name, Class iface) {
        NSI     nsi = (!Utilities.isValidString(name) ? NON_NSI : new NSI(name, null));
        
        return getFirstElement(nsi, iface);
    }
    public StreamElement getFirstElement(String name, String ns) {
        return getFirstElement(new NSI(name, ns), null);
    }
    public StreamElement getFirstElement(String name, String ns, Class iface) {
        return getFirstElement(new NSI(name, ns), iface);
    }
    public StreamElement getFirstElement(NSI name) {
        return getFirstElement(name, null);
    }
    public StreamElement getFirstElement(NSI name, Class iface) {
        Iterator    itr = listElements(name, iface).iterator();
        
        return (StreamElement)(itr.hasNext() ? itr.next() : null);
    }

    public List findElementsForLocale(String ln, String ns, Locale l) {
        return findElementsForLocale(new NSI(ln, ns), l);
    }
    public List findElementsForLocale(NSI name, Locale l) {
        XPath                   xpath = getLocaleXPath();
        SimpleVariableContext   varCtx = (SimpleVariableContext)xpath.getVariableContext();
        String                  ln, ns;
        List                    found = Collections.EMPTY_LIST;
        
        //Validate input
        if (name != null) {
            //Use exactly what we're given
            ln = name.getLocalName();
            ns = name.getNamespaceURI();
        } else {
            //Use null to only check locale
            ln = null;
            ns = null;
        }
        
        if (l != null) {
            varCtx.setVariableValue("ln", ln);
            varCtx.setVariableValue("ns", ns);
            varCtx.setVariableValue("lang", l);
            xpath.setVariableContext(varCtx);        
            try {
                found = xpath.selectNodes(this);
            } catch (JaxenException je) {
                obtainLogger().error("XPath evaluation failed locating subjects for locale", je);
            }
        } else {
            l = getLocale();
            if (l != Locale.getDefault()) {
                //Use "element default" locale
                found = findElementsForLocale(name, l);
            }
            if (found.isEmpty()) {
                //Use language-only locale of platform
                found = findElementsForLocale(name, new Locale(Locale.getDefault().getLanguage()));
            }
        }
        
        return found;
    }
    
    public StreamElement addElement(String name) {
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Local name cannot be null or \"\"");
        return addElement(getDataFactory().createNSI(name, getNamespaceURI()), null);
    }
    public StreamElement addElement(String name, Class iface) {
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Local name cannot be null or \"\"");
        return addElement(getDataFactory().createNSI(name, getNamespaceURI()), iface);
    }
    public StreamElement addElement(String name, String uri) {
        if (!Utilities.isValidString(name) && (uri == null))
            throw new IllegalArgumentException("Local name and namespace URI cannot both be null or \"\"");
        if (uri == null)
            uri = getNamespaceURI();
            
        return addElement(getDataFactory().createNSI(name, uri), null);
    }
    public StreamElement addElement(String name, String uri, Class iface) {
        if (!Utilities.isValidString(name) && !Utilities.isValidString(uri))
            throw new IllegalArgumentException("Local name and namespace URI cannot both be null or \"\"");
        return addElement(getDataFactory().createNSI(name, uri), iface);
    }
    public StreamElement addElement(NSI name) {
        return addElement(name, null);
    }
    public StreamElement addElement(NSI name, Class iface) {
        StreamDataFactory   sdf = getDataFactory();
        StreamElement       elem;
        
        if (name.getNamespaceURI() == null)
            name = new NSI(name.getLocalName(), getNamespaceURI());
        
        elem = sdf.createElementNode(name, iface);
        add(elem);
        
        return elem;
    }
    
    public void clearElements() {
        clearElements(NON_NSI, null);
    }
    public void clearElements(Class iface) {
        clearElements(NON_NSI, iface);
    }
    public void clearElements(String name) {
        clearElements(new NSI(name, null), null);
    }
    public void clearElements(String name, Class iface) {
        clearElements(new NSI(name, null), iface);
    }
    public void clearElements(String name, String uri) {
        clearElements(new NSI(name, uri), null);
    }
    public void clearElements(String name, String uri, Class iface) {
        clearElements(new NSI(name, uri), iface);
    }
    public void clearElements(NSI name) {
        clearElements(name, null);
    }
    public void clearElements(NSI name, Class iface) {
        Iterator        itr = listElements(name, iface).iterator();
        StreamElement   elem;
        
        while (itr.hasNext()) {
            remove((StreamElement)itr.next());
        }
    }
    
    public int indexOf(StreamNode node) {
        return listNodes().indexOf(node);
    }

    protected abstract void checkNSI(NSI check) throws IllegalArgumentException;
    protected abstract void checkElement(StreamElement base) throws IllegalArgumentException;
    
    public void reset() {
        reset(null, null);
    }
    public void reset(NSI name) throws IllegalArgumentException {
        reset(name, null);
    }
    public void reset(StreamElement base) throws IllegalArgumentException {
        reset(null, base);
    }
    public void reset(NSI name, StreamElement base) throws IllegalArgumentException {
        Iterator        itr;

        //Initialize name
        if (name == null) {
            if (base != null)
                name = base.getNSI();
        }
        
        //Validate name and base element
        if (name != null)
            checkNSI(name);
        if (base != null)
            checkElement(base);
        
        //Clear out declared namespaces
        itr = new java.util.LinkedList(getDeclaredNamespaces()).iterator();
        while (itr.hasNext()) {
            ((StreamNamespace)itr.next()).detach();
        }
        
        //Clear out attributes
        itr = new java.util.LinkedList(getAttributes().values()).iterator();
        while (itr.hasNext()) {
            ((StreamAttribute)itr.next()).detach();
        }
        
        //Clear out children
        itr = new java.util.LinkedList(listNodes()).iterator();
        while (itr.hasNext()) {
            ((StreamNode)itr.next()).detach();
        }
        
        if (name != null)
            setNSI(name);
        if (base != null) {
            //Setup declared namespaces
            itr = base.getDeclaredNamespaces().iterator();
            while (itr.hasNext()) {
                StreamNamespace ns = (StreamNamespace)itr.next();
                
                addNamespace((StreamNamespace)ns.copy());
            }
            
            //Setup attributes
            itr = base.getAttributes().values().iterator();
            while (itr.hasNext()) {
                StreamAttribute     attr = (StreamAttribute)itr.next();
                
                setAttributeValue(attr.getNSI(), attr.getValue());
            }
            
            //Setup children
            itr = base.listNodes().iterator();
            while (itr.hasNext()) {
                StreamNode          node = (StreamNode)itr.next();
                
                if (node instanceof StreamElement) {
                    StreamElement   elem = (StreamElement)node;
                    
                    if (Utilities.equateStrings(getNamespaceURI(), elem.getNamespaceURI()))
                        elem.reset(new NSI(elem.getLocalName(), getNamespaceURI()));
                }
                
                add((StreamNode)node.copy());
            }
        }
    }
    public void reset(String val) throws IllegalArgumentException {
        StreamElement   in = new XMLReader(this).read(val);

        reset(in);
    }
    
    /**
     * <p>
     * Compares this <Code>AbstractElement</code> to the given object for
     * equality.  This comparison performs an object reference comparison.  If
     * <code>cmp</code> references the current instance of
     * <code>AbstractElement</code>, this method returns <code>true</code>.</p>
     *
     * <p>
     * The value of <code>cmp</code> can be <code>null</code>, in which case
     * <Code>false</code> is returned.</p>
     *
     * @param cmp The object to compare to.
     * @return <code>true</code> if the references are to the same object.
     */
    public boolean equals(Object cmp) {
        return (this == cmp);
    }
    
    public void accept(StreamObjectVisitor v) {
        v.visit(this);
    }

    public String toString() {
        return new XMLWriter(this).write(this);
    }
}