/*
 * AbstractPacketBuilder.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class AbstractPacketBuilder extends AbstractBuilder {
    //Fields
    private AbstractPacket  _Packet;
    private StreamBuilder   _Extended;

    //Constructors
    public AbstractPacketBuilder(AbstractPacket packet) {
        setPacket(packet);
    }
    
    //Methods
    protected AbstractPacket getPacket() {
        return _Packet;
    }
    private void setPacket(AbstractPacket pkt) throws IllegalArgumentException {
        if (pkt == null)
            throw new IllegalArgumentException("Packet cannot be null");
        _Packet = pkt;
    }

    public NSI getName() {
        return getPacket().getNSI();
    }
    
    /** <p>
     * Resets this <tt>StreamBuilder</tt>.  By resetting, the state is
     * set to <tt>{@link #READY}</tt>, and all internal data for building
     * an object is either deleted or set to default values.</p>
     */
    public void reset() {
        Packet      p = getPacket();
        Iterator    itr = p.listExtensions().iterator();
        
        setStatus(READY);
        p.setFrom(null);
        p.setTo(null);
        p.setID(null);
        
        while (itr.hasNext()) {
            p.removeExtension((Extension)itr.next());
        }
    }

    /** <p>
     * Retrieves the <tt>StreamNode</tt> this <tt>StreamBuilder</tt> has
     * been constructing.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The new <tt>StreamNode</tt> that was built.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #COMPLETE}</tt>.
     */
    public StreamNode getNode() throws IllegalStateException {
        if (getStatus() != COMPLETE)
            throw new IllegalStateException("Not ready to get the node!");
        return getPacket();
    }

    public Status processElementStart(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        NSI                 name = s.getName();
        NSI.Comparator      cmpr = NSI.STRICT_COMPARATOR;
        StreamDataFactory   sdf = ctx.getDataFactory();
        Packet              p = getPacket();
        Set                 attrs = s.getAttributeNames();
        Set                 nss = s.getNamespacePrefixes(s.getDepth());
        Status              status = INVALID;

        try {
            if (NSI.NS_LOOSE_COMPARATOR.compare(name, getName()) == 0) {
                Iterator    attrItr = attrs.iterator();
                Iterator    nssItr = nss.iterator();
                
                //Process all attributes
                while (attrItr.hasNext()) {
                    NSI         attrName = (NSI)attrItr.next();
                    String      val = s.getAttributeValue(attrName);

                    p.setAttributeValue(attrName, val);
                }
                //Process all namespaces
                while (nssItr.hasNext()) {
                    String      nsPre = (String)nssItr.next();
                    String      nsURI = s.getNamespaceURI(nsPre);
                    
                    p.addNamespace(nsPre, nsURI);
                }

                status = PROCESSING;
            } else {
                //Process 
                status = processNestedElementStart(s);
            }
        } finally {
            setStatus(status);
        }
        
        return getStatus();
    }    
    public Status processElementEnd(StreamParseState s) {
        NSI     name = s.getName();
        Packet  p = getPacket();
        Status  status = INVALID;
        
        try {
            if (NSI.NS_LOOSE_COMPARATOR.compare(name, getName()) == 0) {
                status = COMPLETE;
            }
        } finally {
            setStatus(status);
        }
        
        return getStatus();
    }
    public Status processText(StreamParseState s) {
        StreamElement   ctx = s.getContext();
        String          txt = s.getText();
        Packet          elem = getPacket();
        Status          status = INVALID;

        try {
            if (elem != null) {
                try { elem.add(ctx.getDataFactory().createText(txt)); } catch (UnsupportedOperationException uoe) {}

                status = PROCESSING;
            }
        } finally {
            setStatus(status);
        }
        
        return getStatus();
    }
    
    protected Status processNestedElementStart(StreamParseState s) {
        StreamElement       ctx = s.getContext();
        NSI.Comparator      cmpr = NSI.NS_LOOSE_COMPARATOR;
        NSI                 name = s.getName();
        String              uri = name.getNamespaceURI();
        String              ln = name.getLocalName();
        StreamDataFactory   sdf = ctx.getDataFactory();
        StreamBuilder       b;
        Status              status = INVALID;


        if          (Utilities.isValidString(uri) && !Utilities.equateStrings(getName().getNamespaceURI(), uri)) {
            setExtendedData(sdf.createExtensionBuilder(name));
            status = EXTENDED_DATA;
        } else {
            //Setup "extended data"
            setExtendedData(sdf.createElementBuilder(name));
            status = EXTENDED_DATA;
        }
        
        return status;
    }
    
    /** <p>
     * Retrieves the <tt>StreamBuilder</tt> from which to create extended
     * data.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The <tt>StreamBuilder</tt> for the extended data.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #EXTENDED_DATA}</tt>.
     */
    public StreamBuilder buildExtendedData() throws IllegalStateException {
        return _Extended;
    }
    protected void setExtendedData(StreamBuilder b) {
        _Extended = b;
    }
    /** <p>
     * The the given <tt>StreamNode</tt> as extended data.</p>
     *
     *
     * @param node The extended data to add.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #EXTENDED_DATA}</tt>.
     */
    public void addExtendedData(StreamNode node) throws IllegalStateException {
        if          (node instanceof Extension) {
            getPacket().addExtension((Extension)node);
        } else {
            getPacket().add(node);
        }
        setExtendedData(null);
    }
}