/*
 * BasicStreamContext.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.util.Utilities;


/**
 * <p>
 * Basic implementation of <tt>StreamContext</tt>.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$   
 * @since   JSO-0.2
 */
public class BasicStreamContext extends ElementNode implements StreamContext {
    //"Constants"
    public static final NSI NAME = new NSI("stream", Utilities.STREAM_NAMESPACE);

    public static final NSI ATTRIBUTE_ID        = new NSI("id", null);
    public static final NSI ATTRIBUTE_FROM      = new NSI("from", null);
    public static final NSI ATTRIBUTE_TO        = new NSI("to", null);
    
    public static final NSI ATTRNAME_VERSION    = new NSI("version", null);

    //Fields
    private StreamDataFactory   _SDF;
    private Charset             _CS;
    
    //Constructors
    public BasicStreamContext() {
        this(NAME, Utilities.STREAM_PREFIX);
    }
    public BasicStreamContext(NSI name) {
        this(name, "");
    }
    public BasicStreamContext(NSI name, String prefix) {
        super(null, name);
        addNamespace(prefix, getNamespaceURI());
    }
    protected BasicStreamContext(BasicStreamContext base) {
        super(null, base);
    }
    
    //Methods
    public StreamBuilder createBuilder() {
        return new ContextBuilder(this);
    }
    /** <p>
     * Sets the parent <tt>StreamElement</tt> for this
     * <tt>StreamNode</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> may be <tt>null</tt>, in which case
     * the current parent (if any)for this <tt>StreamElement</tt> is
     * detached.  If the parent cannot be changed, this method throws an
     * <tt>IllegalStateException</tt>.</p>
     *
     * @param p the <tt>StreamElement</tt> parent.
     * @throws IllegalStateException If the parent cannot be changed.
     */
    public void setParent(StreamElement p) throws IllegalStateException {
        if (p != null)
            throw new IllegalStateException("Parent cannot be set for StreamContext");
    }

    public StreamContext getContext() {
        return this;
    }
    
    /** <p>
     * Retrieves the "from" <tt>JID</tt> for this
     * <tt>StreamContext</tt>.  The usage of this method depends on
     * the direction of this <tt>StreamContext</tt>, as this may represent
     * this end of the <tt>Stream</tt>, or the remote end.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The "from" <tt>JID</tt> for this <tt>StreamContext</tt>.
     */
    public JID getFrom() {
        Object  val = getAttributeObject(ATTRIBUTE_FROM);
        JID     jid = null;
        
        if (val instanceof JID)
            jid = (JID)val;
        else if (val != null) {
            try {
                jid = JID.valueOf(val);
                setAttributeObject(ATTRIBUTE_FROM, jid);
            } catch(Exception e) {
            }
        }
        
        return jid;
    }
    /** <p>
     * Sets the "from" <tt>JID</tt> for this <tt>StreamContext</tt>.
     * Calling this method after its attached <tt>Stream</tt> is connected
     * or accepted may have deterimental effects on the connection.  Depending
     * on how this instance of <tt>StreamContext</tt> is obtained, this
     * method may effectively do nothing.</p>
     *
     * <p>
     * The value of <tt>from</tt> can be <tt>null</tt>.</p>
     *
     * @param from The new "from" <tt>JID</tt>.
     */
    public void setFrom(JID from) {
        setAttributeObject(ATTRIBUTE_FROM, from);
    }

    /** <p>
     * Retrieves the "to" <tt>JID</tt> for this <tt>StreamContext</tt>.
     * The usage of this method depends on the direction of this
     * <tt>StreamContext</tt>, as this may represent the remote end of the
     * <tt>Stream</tt>, or this end.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The "to" <tt>JID</tt> for this <tt>StreamContext</tt>.
     */
    public JID getTo() {
        Object  val = getAttributeObject(ATTRIBUTE_TO);
        JID     jid = null;
        
        if (val instanceof JID)
            jid = (JID)val;
        else if (val != null) {
            try {
                jid = JID.valueOf(val);
                setAttributeObject(ATTRIBUTE_TO, jid);
            } catch(Exception e) {
            }
        }
        
        return jid;
    }
    /** <p>
     * Sets the "to" <tt>JID</tt> for this <tt>StreamContext</tt>.
     * Calling this method after its attached <tt>Stream</tt> is connected
     * or accepted may have detrimental effects on the connection.  Depending
     * on how this instance of <tt>StreamContext</tt> is obtained, this
     * method may effectively do nothing.</p>
     *
     * <p>
     * The value of <tt>to</tt> can be <tt>null</tt>.</p>
     *
     * @param to The new "to" <tt>JID</tt>.
     */
    public void setTo(JID to) {
        setAttributeObject(ATTRIBUTE_TO, to);
    }
    
    public String getVersion() {
        String      ver = getAttributeValue(ATTRNAME_VERSION);
        
        return ((ver != null) ? ver : "");
    }
    public void setVersion(String ver) {
        setAttributeValue(ATTRNAME_VERSION, Utilities.isValidString(ver) ? ver : null);
    }
    
    /** <p>
     * Determines if the direction for this <tt>StreamContext</tt> is
     * "inbound".  "Inbound" means the data travels from the remote end of the
     * connection to this local end of the connection.</p>
     *
     * <p>
     * This method returns <tt>true</tt> if this <tt>StreamContext</tt>
     * is for the inbound connection, regardless of the current state of the
     * connection.</p>
     *
     * @return <tt>true</tt> if this is for the inbound connection.
     */
    public boolean isInbound() {
        return false;
    }
    /** <p>
     * Determines if the direction for this <tt>StreamContext</tt> is
     * "outbound".  "Outbound" means the data travels from this local end of
     * the connection to the remote end of the connection.</p>
     *
     * <p>
     * This method returns <tt>true</tt> if this <tt>StreamContext</tt>
     * is for the outbound connection, regardless of the current state of the
     * connection.</p>
     *
     * @return <tt>true</tt> if this is for the outbound connection.
     */
    public boolean isOutbound() {
        return false;
    }
    
    /** <p>
     * Creates an "almost-exact" duplicate of this <tt>StreamNode</tt>,
     * using the given parent <tt>StreamElement</tt>.  By "almost-exact",
     * all information for this <tt>StreamNode</tt>, and its contained
     * objects (obtained via <tt>{@link #listNodes}</tt>) are duplicated.
     * The parent for this <tt>StreamNode</tt> is one specified by
     * <tt>p</tt>, rather than thatreferenced by this
     * <tt>StreamNode</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> may be <tt>null</tt>, in which case no
     * parent is attached to the newly created <tt>StreamNode</tt>.</p>
     *
     * @return the "almost-exact" duplicate of this <tt>StreamNode</tt>.
     */
    public StreamObject copy(StreamElement p) {
        return new BasicStreamContext(this);
    }
    
    public PacketRouter getRouter() {
        return null;
    }
    
    public Stream.Status getCurrentStatus() {
        return Stream.DISCONNECTED;
    }
    
    public String getCharsetName() {
        return (_CS != null) ? _CS.name() : "";
    }
    public void setCharsetName(String name) throws IllegalArgumentException {
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("name cannot be null or \"\"");
        
        try {
            _CS = Charset.forName(name);
        } catch (IllegalCharsetNameException icne) {
            throw new IllegalArgumentException("charset named " + name + " is not valid");
        } catch (UnsupportedCharsetException uce) {
            throw new IllegalArgumentException("charset named " + name + " is not supported");
        }
    }
    public Charset obtainCharset() {
        if (_CS == null) {
            _CS = Charset.forName("UTF-8");
        }
        
        return _CS;
    }
    
    protected void checkReset(StreamElement check) throws IllegalArgumentException {
        if (NSI.STRICT_COMPARATOR.compare(getNSI(), check.getNSI()) != 0)
            throw new IllegalArgumentException("NSI must be equal");
    }
}