/*
 * ContextBuilder.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;

/**
 *
 * @author  linuxwolf
 */
public class ContextBuilder extends AbstractBuilder {
    //Fields
    private StreamContext   _Ctx;
    private StreamBuilder   _Extended;

    //Constructors
    public ContextBuilder(StreamContext ctx) {
        setContext(ctx);
    }
    
    //Methods
    public StreamContext getContext() {
        return _Ctx;
    }
    private void setContext(StreamContext ctx) throws IllegalArgumentException {
        if (ctx == null)
            throw new IllegalArgumentException("StreamContext cannot be null");
        _Ctx = ctx;
    }
    
    public NSI getName() {
        return getContext().getNSI();
    }
    
    /** <p>
     * Retrieves the <tt>StreamNode</tt> this <tt>StreamBuilder</tt> has
     * been constructing.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The new <tt>StreamNode</tt> that was built.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #COMPLETE}</tt>.
     */
    public StreamNode getNode() throws IllegalStateException {
        if (getStatus() != COMPLETE)
            throw new IllegalStateException("StreamBuilder not complete");
        return getContext();
    }
    /** <p>
     * Resets this <tt>StreamBuilder</tt>.  By resetting, the state is
     * set to <tt>{@link #READY}</tt>, and all internal data for building
     * an object is either deleted or set to default values.</p>
     */
    public void reset() {
        setStatus(READY);
    }

    /** <p>
     * The the given <tt>StreamNode</tt> as extended data.</p>
     *
     *
     * @param node The extended data to add.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #EXTENDED_DATA}</tt>.
     */
    public void addExtendedData(StreamNode node) throws IllegalStateException {
        getContext().add(node);
    }
    /** <p>
     * Retrieves the <tt>StreamBuilder</tt> from which to create extended
     * data.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The <tt>StreamBuilder</tt> for the extended data.
     * @throws IllegalStateException If the current state is not <tt>{@link
     * #EXTENDED_DATA}</tt>.
     */
    public StreamBuilder buildExtendedData() throws IllegalStateException {
        return _Extended;
    }
    private void setExtendedDataBuilder(StreamBuilder b) {
        _Extended = b;
    }

    /** <p>
     * Processes "start-tag" data.</p>
     *
     * <p>
     * The value of <tt>s</tt> is never <tt>null</tt>.</p>
     *
     * @param s The current <tt>StreamParseState</tt>.
     * @return The new state for this <tt>StreamBuilder</tt>.
     */
    public Status processElementStart(StreamParseState s) {
        StreamDataFactory   sdf = s.getContext().getDataFactory();
        StreamContext       ctx = getContext();
        NSI                 name = s.getName();
        Collection          nss;
        Set                 attrs;
        StreamBuilder       extended;
        Status              status = StreamBuilder.INVALID;
        Iterator            itr;

        try {
            if (name.equals(getName())) {
                //Process StreamContext
                nss = s.getNamespacePrefixes(s.getDepth());
                attrs = s.getAttributeNames();

                //Process namespaces
                itr = nss.iterator();
                while (itr.hasNext()) {
                    String      prefix = (String)itr.next();
                    String      uri = s.getNamespaceURI(prefix);

                    ctx.addNamespace(prefix, uri);
                }

                //Process attributes
                itr = attrs.iterator();
                while (itr.hasNext()) {
                    NSI         attrName = (NSI)itr.next();
                    String      attrVal = s.getAttributeValue(attrName);

                    ctx.setAttributeValue(attrName, attrVal);
                }

                status = PROCESSING;
            } else {
                //Process Extended data (Packets)
                extended = sdf.createPacketBuilder(name);

                if (extended == null) {
                    throw new IllegalStateException("Invalid element \"" + name + "\" encountered");
                }
                setExtendedDataBuilder(extended);
                status = EXTENDED_DATA;
            }
        } finally {
            setStatus(status);
        }

        return getStatus();
    }
    /** <p>
     * Process "end-tag" data.</p>
     *
     * <p>
     * The value of <tt>s</tt> is never <tt>null</tt>.</p>
     *
     * @param s The current <tt>StreamParseState</tt>.
     * @return The new state for thsi <tt>StreamBuilder</tt>.
     */
    public Status processElementEnd(StreamParseState s) {
        if (s.getName().equals(getName()))
            setStatus(COMPLETE);
        else {
            setStatus(INVALID);
            throw new IllegalStateException("Unknown Element end encountered");
        }

        return getStatus();
    }
    /** <p>
     * Processes text data.</p>
     *
     * <p>
     * The value of <tt>s</tt> is never <tt>null</tt>.</p>
     *
     * @param s The current <tt>StreamParseState</tt>.
     * @return The new state for this <tt>StreamBuilder</tt>.
     */
    public Status processText(StreamParseState s) {
        if (!isWhiteSpace(s.getText())) {
            setStatus(INVALID);
            throw new IllegalStateException("Text should not be encountered here (text='" + s.getText() + "')");
        }
        
        return getStatus();
    }
    
    private boolean isWhiteSpace(String test) {
        int     idx;
        boolean result = true;
        
        for (idx = 0; (idx < test.length()) && result; idx++) {
            result = Character.isWhitespace(test.charAt(idx));
        }
        
        return result;
    }
}
