/*
 * ClientPLAINMechanism.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl.mechanisms;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;

import org.jabberstudio.jso.sasl.SASLAuthPacket;
import org.jabberstudio.jso.sasl.SASLClientInfo;
import org.jabberstudio.jso.sasl.SASLMechanism;
import org.jabberstudio.jso.sasl.SASLMechanismManager;
import org.jabberstudio.jso.sasl.SASLPacket;
import org.jabberstudio.jso.sasl.callback.AuthorizationIDCallback;

import net.outer_planes.jso.sasl.AbstractMechanismProvider;
import net.outer_planes.jso.sasl.ClientMechanism;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20051010073318
 * @since   JSO-0.12
 */
public class ClientPLAINMechanism extends ClientMechanism {
    //Inner-classes
    public static class Entry extends AbstractMechanismProvider.ClientEntry {
        //Constructors
        public Entry() {
            super(NAME, POLICY);
        }
        
        //Methods
        public SASLMechanism createMechanism(SASLClientInfo info) {
            return new ClientPLAINMechanism(info);
        }
    }
    
    //"Constants"
    public static final String  NAME        = "PLAIN";
    public static final Map     POLICY;

    static {
        Map     temp = new java.util.TreeMap();

        temp.put(SASLMechanismManager.PROPERTY_POLICY_FORWARD_SECRECY, "false");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NOANONYMOUS, "true");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NOACTIVE, "false");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NODICTIONARY, "false");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NOPLAINTEXT, "false");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_PASS_CREDENTIALS, "false");

        POLICY = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Constructors
    public ClientPLAINMechanism(SASLClientInfo ci) {
        super(ci);
    }

    //Methods
    public String getMechanismName() {
        return NAME;
    }
    
    protected SASLPacket generateAuth() {
        SASLAuthPacket  sasl = (SASLAuthPacket)getDataFactory().createPacketNode(SASLAuthPacket.NAME, SASLAuthPacket.class);

        sasl.setMechanismName(getMechanismName());
        setCurrentStatus(AUTHENTICATING);
        
        return generateResponse(sasl);
    }
    protected SASLPacket evaluateChallenge(SASLPacket challenge) {
        return generateResponse((SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_RESPONSE, SASLPacket.class));
    }
    
    private final SASLPacket generateResponse(SASLPacket input) {
        CallbackHandler cbh = getCallbackHandler();
        List                    calls = new java.util.ArrayList();
        Object                  azid, user, pass;
        AuthorizationIDCallback cbAzid = null;
        NameCallback            cbUser = null;
        PasswordCallback        cbPass = null;
        SASLPacket              output = null;

        azid = getAuthorizationID();
        user = getProperty("javax.security.sasl.username");
        pass = getProperty("javax.security.sasl.password");

        if (cbh != null) {
            if (user == null)
                calls.add(user = new NameCallback("username"));
            if (pass == null)
                calls.add(pass = new PasswordCallback("password", false));
            if (azid == null)
                calls.add(cbAzid = new AuthorizationIDCallback("authorization id"));

            if (!calls.isEmpty()) {
                try {
                    Callback    cb;
                    Callback[]  cbs = new Callback[calls.size()];
                    Iterator    itr;

                    cbs = (Callback[])calls.toArray(cbs);
                    cbh.handle(cbs);

                    itr = calls.iterator();
                    while (itr.hasNext()) {
                        cb = (Callback)itr.next();

                        if      (cb instanceof NameCallback)
                            user = ((NameCallback)cb).getName();
                        if      (cb instanceof PasswordCallback)
                            pass = new String(((PasswordCallback)cb).getPassword());
                        if      (cb instanceof AuthorizationIDCallback)
                            azid = ((AuthorizationIDCallback)cb).getAuthorizationID();
                    }
                } catch (Exception e) {
                    obtainLogger().warn("Could not process all calbacks", e);
                    output = (SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_ABORT, SASLPacket.class);
                }
            }
        }
        
        if (output == null) {
            ByteArrayOutputStream   bout = new ByteArrayOutputStream();
            
            output = input;
            try {
                if (azid != null) {
                    writeParameter(bout, azid);
                }
                bout.write(0);
                if (user != null) {
                    writeParameter(bout, user);
                }
                bout.write(0);
                if (pass != null) {
                    writeParameter(bout, pass);
                }
            } catch (IOException ioe) {
                obtainLogger().fatal("Could not convert string to UTF-8", ioe);
            }
            
            output.setData(bout.toByteArray());
        }
        
        setCurrentStatus(AUTHENTICATING);
        return output;
    }
    private final void writeParameter(ByteArrayOutputStream bout, Object param) throws IOException {
        byte[]      data;

        data = param.toString().getBytes("UTF-8");
        bout.write(data, 0, data.length);
    }
}