/*
 * DiscoInfoNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.disco;


import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jaxen.JaxenException;
import org.jaxen.SimpleVariableContext;
import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.disco.DiscoIdentity;
import org.jabberstudio.jso.x.disco.DiscoInfoQuery;
import org.jabberstudio.jso.x.disco.DiscoUtilities;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class InfoNode extends ExtensionNode implements DiscoInfoQuery {
    //"Constants"
    public static final NSI     ATTRNAME_VAR        = new NSI("var", null);
    
    public static final NSI     ELEMNAME_FEATURE    = new NSI("feature", NAMESPACE);
    
    public static final Map     NSS;
    
    static {
        NSS = Collections.unmodifiableMap(Collections.singletonMap("dinfo", NAMESPACE));
    }
    
    //Fields
    private String          _Name = null;
    
    private XPath           _IdentPath;
    private XPath           _FeatPath;
    
    //Constructors
    public InfoNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected InfoNode(StreamElement parent, InfoNode base) {
        super(parent, base);
    }
    
    //Methods
    protected XPath getIdentityXPath() {
        SimpleVariableContext   varCtx;
        
        if (_IdentPath == null) {
            try {
                _IdentPath  = getDataFactory().createXPath("dinfo:identity[($category='' or @category=$category and ($type='' or @type=$type)]", NSS);
            } catch (SAXPathException saxpe) {
            }
        }
        
        return _IdentPath;
    }
    
    public String getNode() {
        String  node = getAttributeValue(DiscoUtilities.ATTRNAME_NODE);
        
        return ((node != null) ? node : "");
    }
    public void setNode(String node) {
        setAttributeValue(DiscoUtilities.ATTRNAME_NODE, Utilities.isValidString(node) ? node : null);
    }
    
    public String getName() {
        DiscoIdentity   ident;
        
        if (!Utilities.isValidString(_Name)) {
            ident = (DiscoIdentity)getFirstElement(DiscoIdentity.class);
            
            if (ident != null)
                _Name = ident.getName();
        }
        
        return ((_Name != null) ? _Name : "");
    }
    public void setName(String name) {
        Iterator    itr = listElements(DiscoIdentity.class).iterator();
        
        _Name = (Utilities.isValidString(name) ? name : null);
        if (_Name != null) {
            while (itr.hasNext()) {
                DiscoIdentity   ident = (DiscoIdentity)itr.next();
                
                ident.setName(name);
            }
        }
    }
    
    public List listIdentities() {
        return listIdentities(null, null);
    }
    public List listIdentities(String cat, String type) {
        XPath                   xpath;
        SimpleVariableContext   varCtx;
        List                    idents;
        
        try {
            xpath   = getIdentityXPath();
            varCtx  = new SimpleVariableContext();
            
            varCtx.setVariableValue("category", (cat != null) ? cat : "");
            varCtx.setVariableValue("type", (type != null) ? type : "");
            xpath.setVariableContext(varCtx);
            idents  = xpath.selectNodes(this);
        } catch (JaxenException je) {
            idents = Collections.EMPTY_LIST;
        }
        
        return idents;
    }
    public DiscoIdentity getIdentity(String cat, String type) throws IllegalArgumentException {
        Iterator                itr;
        DiscoIdentity           ident = null;
        
        if (!Utilities.isValidString(cat))
            throw new IllegalArgumentException("Category cannot be null or \"\"");
        if (!Utilities.isValidString(type))
            throw new IllegalArgumentException("Type cannot be null or \"\"");

        itr = listIdentities(cat, type).iterator();
        
        ident = (DiscoIdentity)(itr.hasNext() ? itr.next() : null); 
        
        return ident;
    }
    public DiscoIdentity addIdentity(String cat, String type) throws IllegalArgumentException {
        return addIdentity(cat, type, getName());
    }
    public DiscoIdentity addIdentity(String cat, String type, String name) throws IllegalArgumentException {
        DiscoIdentity   ident = getIdentity(cat, type);
        
        if (ident == null) {
            IdentityNode    temp = (IdentityNode)addElement(IdentityNode.NAME, IdentityNode.class);
            
            temp.setCategory(cat);
            temp.setType(type);
            temp.setName(name);
            
            ident = temp;
        }
        
        return ident;
    }
    public void removeIdentity(String cat, String type) throws IllegalArgumentException {
        DiscoIdentity   ident = getIdentity(cat, type);
        
        if (ident != null)
            ident.detach();
    }
    public void clearIdentities() {
        clearElements(DiscoIdentity.class);
    }
    
    public Set getFeatures() {
        List        elems = listElements(ELEMNAME_FEATURE);
        Set         feats = new java.util.HashSet();
        Iterator    itr = elems.iterator();
        
        while (itr.hasNext()) {
            StreamElement   elem = (StreamElement)itr.next();
            
            feats.add(elem.getAttributeValue(ATTRNAME_VAR));
        }
        
        return feats;
    }
    public void addFeature(String feat) {
        if (!Utilities.isValidString(feat))
            return;
        
        if (!getFeatures().contains(feat))
            addElement(ELEMNAME_FEATURE).setAttributeValue(ATTRNAME_VAR, feat);
    }
    public void removeFeature(String feat) {
        Iterator        itr;
        StreamElement   elem;
        
        if (!Utilities.isValidString(feat))
            return;
        
        itr = listElements(ELEMNAME_FEATURE).iterator();
        while (itr.hasNext()) {
            elem = (StreamElement)itr.next();
            
            if (Utilities.equateStrings(elem.getAttributeValue(ATTRNAME_VAR), feat))
                elem.detach();
        }
    }
    public void clearFeatures() {
        clearElements(ELEMNAME_FEATURE);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new InfoNode(parent, this);
    }
}