/*
 * NSI.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


/**
 * <p>
 * A Namespace Identifier is used to encapsulate the
 * "local name" and namespace URI.  This object is used to
 * identify stream data (such as <code>Packet</code> objects).
 * Itr is also used as a "key" for looking up generation
 * objects for stream data.</p>
 *
 * <p>
 * There are two primary uses for this class:  as a qualified identifier, and
 * as a name filter.  Each usage has certain requirements, but are not enforced
 * by this class' methods:
 * <ul>
 * <li><b>Qualified Identifier.</b>  This usage has the general requirement
 * that <code>{@link #getLocalName}</code> never return <code>null</code> or
 * "".  The requirements for <code>{@link #getNamespaceURI}</code> depend on
 * the object it is identifying (note that "" is always interpreted to mean
 * "no namespace"):
 * <ul>
 * <li><b>Element Name.</b>  The result can be <code>null</code>, which means
 * the namespace is determined to be that of its parent (if present), the
 * <code>Stream</code> default namespace (if known), or "" otherwise.</li>
 * <li><b>Attribute Name.</b>  The result cannot be <code>null</code>.  This
 * is because the namespaces of attributes and their containing elements are
 * not related.</li></ul>
 *
 * <li><b>Name Filter.</b>  This usage allows for flexibility in matching
 * elements and attributes.  Depending on the searching algorithm (the
 * particular "get" or "list" method), <code>{@link #getLocalName}</code>
 * and/or <code>{@link #getNamespaceURI}</code> can be <code>null</code>.
 * the local name cannot be "" in almost all cases.</li>
 * </ul>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class NSI implements Comparable, java.io.Serializable {
    //Inner-classes
    /**
     * <p>
     * Compares two <code>NSI</code> objects.  This comparison may use the
     * namespace and/or local name to determine ordering.</p>
     *
     * @author linuxwolf
     * @version $VERSION$
     * @since   JSO-0.2
     */
    public static final class Comparator implements java.util.Comparator, java.io.Serializable {
        //Fields
        private boolean     _IgnoreNS;
        private boolean     _IgnoreLN;
        
        //Constructors
        /**
         * <p>
         * Creates a new <code>Comparator</code> object.  the comparisons made
         * with this object follow the "ignorance rules" established by this
         * constructor.</p>
         *
         * <p>
         * If the value of <code>ns</code> is <code>true</code>, the namespace
         * is always used as part of the comparison, otherwise <code>null</code>
         * values cause it to be ignored.  If the value of <code>ln</code> is
         * <code>true</code>, the local name is always used as part of the
         * comparison, otherwise <code>null</code> values cause it to be
         * ignored.</p>
         *
         * <p>
         * If both <code>ns</code> and <code>ln</code> are <code>false</code>,
         * an <code>IllegalArgumentException</code> is thrown.</p>
         *
         * @param ns Flag to consider namespaces for comparisons
         * @param ln Flag to consider local names for comparisons
         * @since   JSO-0.2
         */
        public Comparator(boolean ns, boolean ln) throws IllegalArgumentException {
            _IgnoreLN = !ln;
            _IgnoreNS = !ns;
        }
        
        //Methods
        /**
         * <p>
         * Retrieves the "ignorance" flag for namespace comparisons.</p>
         *
         * <p>
         * If the value returned by this method is <code>true</code>, then
         * any two <code>NSI</code> in which one or the other has a
         * <code>null</code> namespace, it is considered identical (and
         * <code>{#compare}</code> returns 0).  Otherwise, the following
         * algorithm is followed:
         * <ul>
         *   <li>If the first namespace is <code>null</code> and the second is
         * also, 0 is returned.</li>
         *   <li>If the first namespace is <code>null</code> and the second is
         * not, -1 is returned.</li>
         *   <li>If the first namespace is not <code>null</code> and the second
         * is, 1 is returned.</li>
         *   <li>If the first namespace is not <code>null</code> and the second
         * is also not, the result of a string comparison is returned.</li>
         * </ul></p>
         *
         * @return The "ignorance" flag for namespace comparisons.
         * @since   JSO-0.2
         */
        public final boolean isIgnoringNamespaces() {
            return _IgnoreNS;
        }
        /**
         * <p>
         * Retrieves the "ignorance" flag for local name comparisons.</p>
         *
         * <p>
         * If the value returned by this method is <code>true</code>, then
         * any two <code>NSI</code> in which one or the other has a
         * <code>null</code> local name, it is considered identical (and
         * <code>{#compare}</code> returns 0).  Otherwise, the following
         * algorithm is followed:
         * <ul>
         *   <li>If the first local name is <code>null</code> and the second is
         * also, 0 is returned.</li>
         *   <li>If the first local name is <code>null</code> and the second is
         * not, -1 is returned.</li>
         *   <li>If the first local name is not <code>null</code> and the second
         * is, 1 is returned.</li>
         *   <li>If the first local name is not <code>null</code> and the second
         * is also not, the result of a string comparison is returned.</li>
         * </ul></p>
         *
         * @return The "ignorance" flag for local name comparisons.
         * @since   JSO-0.2
         */
        public final boolean isIgnoringLocalNames() {
            return _IgnoreLN;
        }

        /**
         * <p>
         * Compares two objects using <code>NSI</code> comparison rules.  The
         * order of comparison is as follows:
         * <ol>
         *   <li>Compare namespaces (as described for
         * <code>{@link #isIgnoringNamespaces}</code>).</li>
         *   <li>Compare local names (as described for
         * <code>{@link #isIgnoringLocalNames}</code>).</li>
         * </ol>
         * The second "sub" comparison is only performed if the first results
         * in 0.</p>
         *
         * @param obj1 The first object to compare.
         * @param obj2 the second object to compare;
         * @return The results of the comparison.
         * @throws ClassCastException If the either of the parameters are not
         * <code>NSI</code> objects.
         * @throws NullPointerException If either of the parameters are
         * <code>null</code>.
         * @since   JSO-0.2
         */
        public final int compare(Object obj1, Object obj2) throws ClassCastException, NullPointerException {
            NSI     cmp1 = (NSI)obj1;
            NSI     cmp2 = (NSI)obj2;
            int     results = 0;
            
            if ((cmp1 == null) || (cmp2 == null))
                throw new NullPointerException("Compared objects cannot be null");
            
            if (results == 0) {
                results = compareNamespaces(cmp1.getNamespaceURI(), cmp2.getNamespaceURI());
            }
            if (results == 0) {
                results = compareLocalNames(cmp1.getLocalName(), cmp2.getLocalName());
            }
            
            return results;
        }
        
        private final int compareNamespaces(String ns1, String ns2) {
            if (ns1 == ns2)                             //"null" or "same object"
                return 0;
            else if ((ns1 == null) && (ns2 != null))    //first is "null"
                return _IgnoreNS ? 0 : -1;
            else if ((ns1 != null) && (ns2 == null))    //second is "null"
                return _IgnoreNS ? 0 : 1;
            return ns1.compareTo(ns2);
        }
        private final int compareLocalNames(String ln1, String ln2) {
            if (ln1 == ln2)
                return 0;
            else if ((ln1 == null) && (ln2 != null))
                return _IgnoreLN ? 0 : -1;
            else if ((ln1 != null) && (ln2 == null))
                return _IgnoreLN ? 0 : 1;
            return ln1.compareTo(ln2);
        }
    }

    //"Constants"
    /** <code>Comparator</code> for "strict" <code>NSI</code> comparisons.
     * @since   JSO-0.2
     */
    public static final Comparator STRICT_COMPARATOR = new Comparator(true, true);
    /** <code>Comparator</code> for "Namespace loose" <code>NSI</code>
     * comparisons.
     * @since   JSO-0.2
     */
    public static final Comparator NS_LOOSE_COMPARATOR = new Comparator(false, true);
    /** <code>Comparator</code> for "Localname loose" <code>NSI</code>
     * comparisons.
     * @since   JSO-0.10
     */
    public static final Comparator LN_LOOSE_COMPARATOR = new Comparator(true, false);
    /** <code>Comparator</code> for "loose" <code>NSI</code> comparisons.
     * @since   JSO-0.2
     */
    public static final Comparator LOOSE_COMPARATOR = new Comparator(false, false);

    //Fields
    private String      _LocalName;
    private String      _URI;
    
    private String      _Formal;

    //Constructors
    /**
     * <p>
     * Creates a new <code>NSI</code> object with the given "local name"
     * and namespace URI.</p>
     *
     * @param name The "local name" for this <code>NSI</code>
     * @param uri The namespace URI for this <code>NSI</code>.
     * @since   JSO-0.2
     */    
    public NSI(String name, String uri) {
        setLocalName(name);
        setNamespaceURI(uri);
    }

    //Methods -- Property getter/setters
    /**
     * <p>
     * Retrieves the "local name" for this <code>NSI</code>.  The
     * local name is name of the XML element, minus prefix.</p>
     *
     * <p>
     * This method may return <code>null</code> or "".</p>
     *
     * @return The "local name" of this <code>NSI</code>.
     * @since   JSO-0.2
     */    
    public final String getLocalName() {
        return _LocalName;
    }
    private final void setLocalName(String name) {
        _LocalName = name;
    }
    
    /**
     * <p>
     * Retrieves the namespace URI for this <code>NSI</code>.</p>
     *
     * <p>
     * This method may return <code>null</code> or "".</p>
     *
     * @return The namespace URI for this <code>NSI</code>.
     * @since   JSO-0.2
     */    
    public final String getNamespaceURI() {
        return _URI;
    }
    private final void setNamespaceURI(String uri) {
        _URI = uri;
    }
    
    /**
     * <p>
     * Generates the hascode for this <tt>NSI</tt>.</p>
     *
     * @return The hashcode for this <tt>NSI</tt>.
     * @since   JSO-0.8.5
     */
    public final int hashCode() {
        String  val;
        int     results = 23;
        
        if ((val = getLocalName()) != null)
            results = 41 * results + val.hashCode();
        if ((val = getNamespaceURI()) != null)
            results = 41 * results + val.hashCode();
        
        return results;
    }
    
    //Methods -- Equality
    /**
     * <p>
     * Compares this <code>NSI</code> to another object for equality.
     * The two objects are equal only if both are <code>NSI</code> objects,
     * and contains the same "local name" and namespace URI.</p>
     *
     * <p>
     * If the value of <code>cmp</code> is <code>null</code> or not an
     * instance of <code>NSI</code>, this method returns <code>false</code>.</p>
     *
     * @param cmp The object to compare to.
     * @return <code>true</code> if both objects are equivalent.
     * @since   JSO-0.2
     */    
    public final boolean equals(Object cmp) {
        if (cmp instanceof NSI)
            return equals((NSI)cmp);
        return false;
    }
    /**
     * <p>
     * Compares this <code>NSI</code> to the given <code>NSI</code> for
     * equality.  The two <code>NSI</code> are equivalent if both are not
     * <code>null</code> and contain equivalent local names and namespace
     * URI's.</p>
     *
     * <p>
     * If the value of <code>cmp</code> is <code>null</code>, this method
     * returns <code>false</code>.</p>
     *
     * @param cmp The <code>NSI</code> to compare to.
     * @return <code>true</code> if both <code>NSI</code> are equivalent.
     * @since   JSO-0.2
     */    
    public final boolean equals(NSI cmp) {
        String      test1, test2;
        boolean     results = (cmp != null);

        //Test using "strict" comparison rules
        if (results) {
            results = (STRICT_COMPARATOR.compare(this, cmp) == 0);
        }
        
        return results;
    }
    
    /**
     * <p>
     * Compares this <tt>NSI</tt> to the given <tt>NSI</tt> for natural
     * ordering. This method uses the same algorithm as that for {@link
     * #STRICT_COMPARATOR}.</p>
     *
     * <p>
     * The value of <tt>cmp</tt> must be a non-<tt>null</tt> instance of a
     * <tt>NSI</tt> type, or a <tt>NullPointerException</tt> or
     * <tt>ClassCastException</tt> may be thrown.</p>
     *
     * @param cmp The object to compare to.
     * @return less than 0, 0, or greater than 0 if this <tt>NSI</tt> is less
     * than, equal to, or greater than <tt>cmp</tt>.
     * @since   JSO-0.12
     */
    public final int compareTo(Object cmp) {
        return STRICT_COMPARATOR.compare(this, cmp);
    }
    
    /**
     * <p>
     * Generates the string representation of this <code>NSI</code>.  The
     * generated string is in the form "{URI}local-name".  The values of
     * "URI" and "local-name" are determined using the following algorithm:
     * <ul>
     * <li>If <code>{@link #getNamespaceURI}</code> returns <code>null</code>,
     * "URI" == "[uknown]".</li>
     * <li>If <code>{@link #getLocalName}</code> returns <code>null</code>,
     * "local-name" == "[unknown]".</li>
     * </ul>
     * </p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return The string representing this <code>NSI</code>.
     * @since   JSO-0.2
     */
    public final String toString() {
        return toString(getLocalName(), getNamespaceURI());
    }
    /**
     * <p> 
     * Generates the "formal" notation string for the given local-name and
     * namespace URI.  The generated string is in the form "{URI}local-name".
     * The values of "URI" and "local-name" are determined using the following
     * algorithm:
     * <ul>
     * <li>If <tt>uri</tt> returns <tt>null</tt>, "URI" == "[uknown]".</li>
     * <li>If <tt>ln</tt> returns <tt>null</tt>, "local-name" ==
     * "[unknown]".</li>
     * </ul>
     * </p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The string representing the given local-name and namespace URI.
     * @since   JSO-0.12
     */
    public static final String toString(String ln, String uri) {
        StringBuffer        buffer = new StringBuffer();
        String              temp;
        
        temp = ((uri == null) ? "[unknown]" : uri);
        buffer.append("{").append(temp).append("}");
        
        temp = ((ln == null) ? "[unknown]" : ln);
        buffer.append(temp);
        
        return buffer.toString();
    }
}