/*
 * Stream.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.StreamFeaturesDispatcher;
import org.jabberstudio.jso.event.StreamStatusDispatcher;
import org.jabberstudio.jso.io.StreamSource;
import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Common interface defining all connection types.  Through this interface,
 * data is sent and received, the various <tt>Packet</tt> and
 * <tt>Extension</tt> objects are created, and connection
 * status and detailed information is maintained.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface Stream extends StreamElementRegistry, StreamStatusDispatcher, StreamFeaturesDispatcher, PacketDispatcher, PacketRouter {
    //Inner-classes
    /**
     * <p>
     * Class to enumerate the possible status types for a <tt>Stream</tt>.
     * Instances of this class only exist as "constant" field members of the
     * parent <tt>Stream</tt> interface.</p>
     *
     * @author  linuxwolf
     * @version $VERSION$
     * @since   JSO-0.2
     */
    public static final class Status extends Enumerator {
        //Fields
        private boolean _Connected;
        private boolean _Opened;
        
        //Constructors
        private Status(String name, boolean conn, boolean opn) {
            super(name);
            _Connected = conn;
            _Opened = opn;
        }
        
        //Methods
        /**
         * <p>
         * Determines if this <tt>Status</tt> represents a disconnected
         * state.</p>
         *
         * @return <tt>true</tt> if this <tt>Status</tt> represents
         * a disconnected status type.
         * @since   JSO-0.2
         */
        public boolean isDisconnected() {
            return !isConnected();
        }
        /**
         * <p>
         * Determines if this <tt>Status</tt> represents a connected
         * state.  The state is considered connected if the
         * <tt>StreamSource</tt> is connected, and I/O is ready.</p>
         *
         * @return <tt>true</tt> if this <tt>Status</tt> represents
         * a connected status type.
         * @since   JSO-0.2
         */
        public boolean isConnected() {
            return _Connected;
        }
        /**
         * <p>
         * Determines if this <tt>Status</tt> represents an opened state.
         * The state is considered opened if the
         * <tt>StreamSource</tt> is connected, all I/O is ready, and the
         * "stream document" is started.</p>
         * 
         * <p>
         * Note that this method returning <tt>ture</tt> implies {@link
         * #isConnected} returns <tt>true</tt>.</p>
         *
         * @return <tt>true</tt> if this <tt>Status</tt> represents
         * an opened status type.
         * @since   JSO-0.2
         */
        public boolean isOpened() {
            return _Opened;
        }
        
        /**
         * <p>
         * Determines the <tt>Stream.Status</tt> which best represents the
         * given inbound and outbound <tt>Stream.Status</tt> objects.</p>
         *
         * <p>
         * The value of <tt>in</tt> cannot be <tt>null</tt>, or an
         * <tt>IllegalArgumentException</tt> is thrown.</p>
         *
         * <p>
         * The value of <tt>out</tt> cannot be <tt>null</tt> or an
         * <tt>IllegalArgumentException</tt> is thrown.</p>
         *
         * @param in The inbound <tt>Stream.Status</tt>.
         * @param out The outbound <tt>Stream.Status</tt>.
         * @return The aggregate <tt>Stream.Status</tt>.
         * @throws IllegalArgumentException If any parameters are invalid.
         * @since   JSO-0.10
         */
        public static Status getAggregateStatus(Status in, Status out) throws IllegalArgumentException {
            Status  curr;
            if (in == null)
                throw new IllegalArgumentException("Inbound status cannot be null");
            if (out == null)
                throw new IllegalArgumentException("Outbound status cannot be null");
            
            if (in.isOpened() && out.isOpened()) {
                curr = Stream.OPENED;
            } else if (in.isConnected() && out.isConnected()) {
                curr = Stream.CONNECTED;
            } else {
                curr = Stream.DISCONNECTED;
            }

            return curr;
        }
    }

    //"Constants"
    /** <tt>Status</tt> for a disconnected <tt>Stream</tt>.
     * @since   JSO-0.2
     */
    public static final Status DISCONNECTED = new Status("disconnected", false, false);
    /** <tt>Status</tt> for a connected <tt>Stream</tt>.
     * @since   JSO-0.2
     */
    public static final Status CONNECTED = new Status("connected", true, false);
    /** <tt>Status</tt> for an opened <tt>Stream</tt>.
     * @since   JSO-0.2
     */
    public static final Status OPENED = new Status("opened", true, true);
    /** <tt>Status</tt> for a closed <tt>Stream</tt>.
     * @since   JSO-0.2
     */
    public static final Status CLOSED = new Status("closed", true, false);
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>Status</tt> for this <tt>Stream</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The current <tt>Status</tt>.
     * @since   JSO-0.2
     */
    public Status getCurrentStatus();
    
    /**
     * <p>
     * Retrieves the URI for the default namespace (if any).  This will
     * typically be one of the "jabber:" namespaces ("jabber:client",
     * "jabber:component:accept", etc).</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * @return The URI for the default namespace.
     * @since   JSO-0.2
     */
    public String getDefaultNamespace();
    
    /**
     * <p>
     * Retrieves the <tt>JSOImplementation</tt> that created this
     * <tt>Stream</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>JSOImplementation</tt> which created this
     * <tt>Stream</tt>.
     * @since   JSO-0.2
     */
    public JSOImplementation getJSO();
    
    /**
     * <p>
     * Retrieves the <tt>StreamDataFactory</tt> for this
     * <tt>Stream</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamDataFactory</tt> for this <tt>Stream</tt>.
     * @since   JSO-0.2
     */
    public StreamDataFactory getDataFactory();
    
    /**
     * <p>
     * Retrieves the <tt>StreamContext</tt> for the inbound direction.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return the inbound <tt>StreamContext</tt>.
     * @since   JSO-0.2
     */
    public StreamContext getInboundContext();
    /**
     * <p>
     * Retrieves the <tt>Stream.Status</tt> for the inbound direction.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The inbound <tt>Stream.Status</tt>.
     * @since   JSO-0.2
     */
    public Status getInboundStatus();
    
    /**
     * <p>
     * Retrieves the <tt>StreamContext</tt> for the outbound direction.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return the outbound <tt>StreamContext</tt>.
     * @since   JSO-0.2
     */
    public StreamContext getOutboundContext();
    /**
     * <p>
     * Retrieves the <tt>Stream.Status</tt> for the outbound direction.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The outbound <tt>Stream.Status</tt>.
     * @since   JSO-0.2
     */
    public Status getOutboundStatus();

    /**
     * <p>
     * Connects this <tt>Stream</tt> to the <tt>StreamSource</tt>. This
     * method does not return until {@link #getCurrentStatus} returns {@link
     * #CONNECTED} or an exception occurs.  If this <tt>Stream</tt> is
     * already in this state (or "higher"), this method does nothing.</p>
     *
     * <p>
     * Any exception (other than <tt>IllegalArgumentException</tt> and
     * <tt>StreamException</tt>) thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * <p>
     * The value of <tt>src</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param src The <tt>StreamSource</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws StreamException If any other errors occur.
     * @since   JSO-0.2
     */
    public void connect(StreamSource src) throws IllegalArgumentException, StreamException;
    /**
     * <p>
     * Disconnects this <tt>Stream</tt>.  This method does not return until
     * {@link #getCurrentStatus} returns {@link #DISCONNECTED} or an exception
     * occurs.  If this <tt>Stream</tt> is already is in this state, this
     * method does nothing.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * @throws StreamException If any errors occur.
     * @since   JSO-0.2
     */
    public void disconnect() throws StreamException;
    
    /**
     * <p>
     * Opens this <tt>Stream</tt>. This method sends the outbound
     * "&lt;stream:stream&gt; and waits to receive the inbound
     * &lt;stream:stream&gt; (if not already received). This method does not
     * return until both opening &lt;stream:stream&gt; opening tags have been
     * transferred.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.2
     */
    public void open() throws StreamException;
    /**
     * <p>
     * Opens this <tt>Stream</tt>. This method sends the outbound
     * "&lt;stream:stream&gt; and waits to receive the inbound
     * &lt;stream:stream&gt; (if not already received). This method does not
     * return until either both opening &lt;stream:stream&gt; opening tags have
     * been transferred, or the specified time has elapsed.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * <p>
     * The value of <tt>timeout</tt> may be <tt>0</tt> or less, in which case
     * this method never times out.</p>
     *
     * @param timeout The maximum time to wait in milliseconds.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.12
     */
    public void open(long timeout) throws StreamException;
    
    /**
     * <p>
     * Drops the XML information for this <tt>Stream</tt>. This method resets
     * the XML parser for incoming data, then resets the inbound- and outboud
     * <tt>Stream.Status</tt> to {@link #CONNECTED}, firing the relevant
     * events.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.<?p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.12
     */
    public void drop() throws StreamException;
    
    /**
     * <p>
     * Drops the XML information for this <tt>Stream</tt>'s inbound context. 
     * This method resets optionally
     * the XML parser for incoming data, then resets the inbound
     * <tt>Stream.Status</tt> to {@link #CONNECTED}, firing the relevant
     * events.  Resetting the XML parser is necessary if the 
     * stream source has been reset.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.<?p>
     *
     * @param reset whether to reset the XML parser.  
     *        Resetting the XML parser is necessary if the 
     *        stream source has been reset.
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.12
     */
    public void dropInbound(boolean reset) throws StreamException;
    
    /**
     * <p>
     * Drops the XML information for this <tt>Stream</tt>'s outbound context.
     * This method optionally resets
     * the XML exporter for data, then resets the outbound
     * <tt>Stream.Status</tt> to {@link #CONNECTED}, firing the relevant
     * events.  Resetting the XML exporter is necessary if the 
     * stream source has been reset.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.<?p>
     *
     * @param reset whether to reset the XML exporter.
     *        Resetting the XML parser is necessary if the 
     *        stream source has been reset.
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.12
     */
    public void dropOutbound(boolean reset) throws StreamException;
    
    /**
     * <p>
     * Closes this <tt>Stream</tt>.  This method sends the outbound
     * &lt;/stream:stream&gt; and waits to receive the inbound
     * &lt;/stream:stream&gt; (if not already received).  This method does
     * not return until both closing &lt;/stream:stream&gt; tags have been
     * transferred.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.2
     */
    public void close() throws StreamException;
    /**
     * <p>
     * Closes this <tt>Stream</tt>.  This method sends the outbound
     * &lt;/stream:stream&gt; and waits to receive the inbound
     * &lt;/stream:stream&gt; (if not already received).  This method does
     * not return until either both closing &lt;/stream:stream&gt; tags have
     * been transferred, or the specified time has elapsed.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * <p>
     * The value of <tt>timeout</tt> may be <tt>0</tt> or less, in which case
     * this method never times out.</p>
     *
     * @param timeout The maximum time to wait in milliseconds.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.12
     */
    public void close(long timeout) throws StreamException;
    /**
     * <p>
     * Closes this <tt>Stream</tt>, sending the given
     * <tt>StreamException</tt>.  This method sends the outbound
     * &lt;/stream:stream&gt; and waits to receive the inbound
     * &lt;/stream:stream&gt; (if not already received).  This method does
     * not return until both closing &lt;/stream:stream&gt; tags have been
     * transferred.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * <p>
     * The value of <tt>se</tt> may be <tt>null</tt>. If not, this method
     * first sends a &lt;stream:error/&gt; using <tt>se</tt> before sending
     * the closing tag.</p>
     *
     * @param se The <tt>StreamException</tt> closing this <tt>Stream</tt>.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.8.4
     */
    public void close(StreamException se) throws StreamException;
    /**
     * <p>
     * Closes this <tt>Stream</tt>, sending the given
     * <tt>StreamException</tt>.  This method sends the outbound
     * &lt;/stream:stream&gt; and waits to receive the inbound
     * &lt;/stream:stream&gt; (if not already received).  This method does
     * not return until both closing &lt;/stream:stream&gt; tags have been
     * transferred.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * <p>
     * The value of <tt>se</tt> may be <tt>null</tt>. If not, this method
     * first sends a &lt;stream:error/&gt; using <tt>se</tt> before sending
     * the closing tag.</p>
     *
     * <p>
     * The value of <tt>timeout</tt> may be <tt>0</tt> or less, in which case
     * this method never times out.</p>
     *
     * @param se The <tt>StreamException</tt> closing this <tt>Stream</tt>.
     * @param timeout The maximum time to wait in milliseconds.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.12
     */
    public void close(StreamException se, long timeout) throws StreamException;
    /**
     * <p>
     * Closes this outbound context for this <tt>Stream</tt>.
     * This method returns immediately after sending the closing
     * &lt;/stream:stream&gt;</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * <p>
     * The value of <tt>se</tt> may be <tt>null</tt>. If not, this method
     * first sends a &lt;stream:error/&gt; using <tt>se</tt> before sending
     * the closing tag.</p>
     *
     * @param se The <tt>StreamException</tt> closing this <tt>Stream</tt>.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.13
     */
    public void closeOutbound(StreamException se) throws StreamException;
    /**
     * <p>
     * Closes this outbound context for this <tt>Stream</tt>.
     * This method returns immediately after sending the closing
     * &lt;/stream:stream&gt;</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.13
     */
    public void closeOutbound() throws StreamException;

    /**
     * <p>
     * Send a <tt>Packet</tt> across this <tt>Stream</tt>.</p>
     *
     * <p>
     * If the value of <tt>p</tt> is <tt>null</tt>, this method sends
     * an empty string.</p>
     *
     * <p>
     * Any exception thrown (for reasons other than the above) is caught and
     * re-thrown as a <tt>StreamException</tt>.  Also, this
     * <tt>Stream</tt> will progress through {@link #close} and {@link
     * #disconnect}.</p>
     *
     * @param p The <tt>Packet</tt> to send.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.2
     */
    public void send(Packet p) throws StreamException;
    /**
     * <p>
     * Processes <tt>Packet</tt> data coming into this
     * <tt>Stream</tt>.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.  Also, this <tt>Stream</tt> will
     * progress through {@link #close} and {@link #disconnect}.</p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.2
     */
    public void process() throws StreamException;
    /**
     * <p>
     * Sends the opening <tt>&lt;stream:stream&gt;</tt> tag.
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>.</p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.13
     */
     public void openOutbound() throws StreamException;
}
