/*
 * StreamDataFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;

import java.util.Set;

import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamNodeFactory;
import org.jabberstudio.jso.xpath.XPathSupport;

/**
 * <p>
 * Interface for creating stream data.  Through this interface, instances of
 * <code>JID</code>, <code>Packet</code>, and <code>Extension</code> can be
 * created (as well as those from <code>StreamNodeFactory</code>.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamDataFactory extends StreamNodeFactory, StreamElementRegistry, XPathSupport {
    //Methods
    /**
     * <p>
     * Creates a new <code>NSI</code> object with the given local name and
     * namespace URI.  The factory may return a cached value, rather than
     * create a new object.</p>
     *
     * <p>
     * The value of <code>name</code> can be <code>null</code>.  Its value
     * cannot be "", or an <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <code>uri</code> can be <code>null</code> or "".   The
     * "non-namspace" is considered "".</p>
     *
     * @param name The local name for the <code>NSI</code>.
     * @param uri The namespace URI for the <code>NSI</code>.
     * @return The "new" <code>NSI</code> object.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public NSI createNSI(String name, String uri) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the set of supported <tt>StreamElement</tt> names, as
     * <tt>NSI</tt> objects.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The set of supported <tt>StreamElement</tt> names.
     * @since   JSO-0.10
     */
    public Set getSupportedElements();
    /**
     * <p>
     * Determines if the given <tt>NSI</tt> and class are supported by this
     * <tt>StreamDataFactory</tt>. This method returns <tt>true</tt> only
     * if both a <tt>StreamElement</tt> and <tt>StreamBuilder</tt> can be
     * created with the given <tt>NSI</tt> and class.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt>, in which case an
     * empty <tt>NSI</tt> is used.</p>
     *
     * <p>
     * The vallue of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * @param name The <tt>NSI</tt> to check against.
     * @param iface The <tt>Class</tt> to check against.
     * @return <tt>true</tt> if <tt>StreamElement</tt> nodes and builders can
     * be created with the given arguments
     * @since   JSO-0.10
     */
    public boolean isSupportedElement(NSI name, Class iface);

    /**
     * <p>
     * Creates a new <tt>StreamElement</tt> with the given <tt>NSI</tt>.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, the value of {@link
     * NSI#getLocalName} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The <tt>NSI</tt> to create with.
     * @return The <tt>StreamElement</tt> for the given name and class.
     * @throws IllegalArgumentException If the parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement createElementNode(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <tt>StreamElement</tt> with the given <tt>NSI</tt> and
     * class.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, the value of {@link
     * NSI#getLocalName} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored. a <tt>StreamElement</tt> cannot be created with the given
     * class, an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The <tt>NSI</tt> to create with.
     * @param iface The <tt>Class</tt> to create with.
     * @return The <tt>StreamElement</tt> for the given name and class.
     * @throws IllegalArgumentException If the parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement createElementNode(NSI name, Class iface) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <tt>StreamBuilder</tt> for a <tt>StreamElement</tt> with
     * the given <tt>NSI</tt>.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, the value of {@link
     * NSI#getLocalName} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The <tt>NSI</tt> to create with.
     * @return The <tt>StreamBuilder</tt> for the given name and class.
     * @throws IllegalArgumentException If the parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamBuilder createElementBuilder(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <tt>StreamBuilder</tt> for a <tt>StreamElement</tt> with
     * the given <tt>NSI</tt> and class.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, the value of {@link
     * NSI#getLocalName} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored. a <tt>StreamBuilder</tt> cannot be created with the given
     * class, an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The <tt>NSI</tt> to create with.
     * @param iface The <tt>Class</tt> to create with.
     * @return The <tt>StreamBuilder</tt> for the given name and class.
     * @throws IllegalArgumentException If the parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamBuilder createElementBuilder(NSI name, Class iface) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates a new <code>StreamElement</code> with the given local name, and
     * no namespace (or the "system default" namespace).  If a "non-namespace"
     * is desired, called <code>{@link #createElement(String, String)}</code>
     * with <code>null</code> for the last argument.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param name The local name for this <code>StreamElement</code>.
     * @return the new <code>StreamElement</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     * @deprecated As of JSO-0.12, this method is replaced with
     * {@link #createElementNode}. Using a <tt>NSI</tt> with a namespace
     * URI of <tt>null</tt> provides equivalent behavior.
     */
    public StreamElement createElement(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>StreamElement</code> with the given local name and
     * namespace.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <code>uri</code> can be <code>null</code> or "".</p>
     *
     * @param name The local name for this <code>StreamElement</code>.
     * @param uri The namespace URI for this <code>StreamElement</code>.
     * @return The new <code>StreamElement</code>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     * @deprecated As of JSO-0.12, this method is replaced with
     * {@link #createElementNode}.
     */
    public StreamElement createElement(String name, String uri) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>StreamElement</code> with the given name.  The name
     * is specified as an <code>NSI</code>, which contains the local name and
     * the governing namespace URI.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.  Also, if the value
     * returned by <code>{@link NSI#getLocalName}</code> is <code>null</code>
     * or "", this method throws an <code>IllegalArgumentException</code>.</p>
     *
     * @param name The name for this <code>StreamElement</code>.
     * @return The new <code>StreamElement</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     * @deprecated As of JSO-0.12, this method is replaced with
     * {@link #createElementNode}. Using a <tt>NSI</tt> with a namespace
     * URI of <tt>null</tt> provides equivalent behavior.
     */
    public StreamElement createElement(NSI name) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates a new <code>StreamText</code> object with the given text value.
     * This new <code>StreamText</code> does not have any parent reference.</p>
     *
     * <p>
     * The value of <code>txt</code> can be <code>null</code> or "".  If it is
     * <code>null</code>, it is reconsidered to be "".</p>
     *
     * @param txt  The text value string for the <code>StreamText</code>.
     * @return the new <code>StreamText</code>.
     * @since   JSO-0.12
     */
    public StreamText createText(String txt);
    
    /**
     * <p>
     * Creates a new <code>SerializedNodes</code> object with the given text 
     * value. This new <code>SerializedNodes</code> does not have 
     * any parent reference.</p>
     *
     * <p>
     * The value of <code>txt</code> can be <code>null</code> or "".  If it is
     * <code>null</code>, it is reconsidered to be "".</p>
     *
     * @param txt  The text value string for the <code>SerializedNodes</code>.
     * @return the new <code>SerializedNodes</code>.
     * @since   JSO-0.12
     */
    public SerializedNodes createSerializedNodes(String txt);
    
    /**
     * <p>
     * Creates a new <code>JID</code> using the given node, domain, and
     * resource values.  Because instances of <code>JID</code> cannot be
     * modified after creation, implementations of this interface may return
     * a cached value.</p>
     *
     * <p>
     * The value of <code>node</code> can be <code>null</code> or "".  Such a
     * value is considered <code>null</code>.</p>
     *
     * <p>
     * The value of <code>domain</code> cannot be <code>null</code> or "", or
     * an <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <code>resource </code> can be <code>null</code> or "".  Such
     * a value is considered <code>null</code>.</p>
     *
     * @param node The node for the <code>JID</code>.
     * @param domain The domain for the <code>JID</code>.
     * @param resource The resource for the <code>JID</code>.
     * @return The new <code>JID</code>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.2
     */
    public JID createJID(String node, String domain, String resource) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>JID</code> by parsing the given specification.
     * The general format of a JID string is "[node@]domain[/resource]", where
     * <code>node</code> corresponds to <code>{@link JID#getNode}</code>,
     * <code>domain</code> corresponds to <code>{@link JID#getDomain}</code>,
     * and <code>resource</code> corresponds to <code>{@link
     * JID#getResource}</code>).</p>
     *
     * <p>
     * The value of <code>spec</code> cannot be <code>null</code> or "", or
     * an <code>IllegalArgumentException</code> is thrown.  Also, if
     * <code>spec</code> cannot be parsed into a <code>JID</code> an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param spec The string specifcation for the <code>JID</code>.
     * @return The new <code>JID</code>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.8
     */
    public JID createJID(Object spec) throws IllegalArgumentException;

    /**
     * <p>
     * Creates a new <tt>PacketError</tt> with the given type and defined
     * condition.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>cond</tt> may be <tt>null</tt> or "", in which case
     * the condition is set to "undefined-condition". If the value of
     * <tt>cond</tt> otherwise does not match a defined condition, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param type The type for the error.
     * @param cond The defined condition for the error.
     * @return The new <tt>PacketError</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public PacketError createPacketError(PacketError.Type type, String cond) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <tt>StreamError</tt> with the given defined condition.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>cond</tt> may be <tt>null</tt> or "", in which case
     * the condition is set to "undefined-condition". If the value of
     * <tt>cond</tt> otherwise does not match a defined condition, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param cond The defined condition for the error.
     * @return The new <tt>StreamError</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamError createStreamError(String cond) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates a new <code>PacketError</code> with the given error code.</p>
     *
     * <p>
     * The value returned by this <code>PacketError</code> is never
     * <code>null</code>.  Also, the "text message" for any
     * <code>PacketError</code> object created with this method follows a set
     * of "pre-defined" messages, as alluded to in the IETF submission.</p>
     *
     * <p>
     * The value of <code>code</code> cannot be less than 0, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param code The error code for the <code>PacketError</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @return The new <code>PacketError</code>.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, this method is remove. Use {@link
     * #createPacketError(PacketError.Type, String)} instead.
     */
    public PacketError createError(int code) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>PacketError</code> with the given error code and
     * text message.</p>
     *
     * <p>
     * The value returned by this <code>PacketError</code> is never
     * <code>null</code>.</p>
     *
     * <p>
     * The value of <code>code</code> cannot be less than 0, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value of <code>msg</code> can be <code>null</code> or "", in which
     * case it is considered to be "".</p>
     *
     * @param code The error code for the <code>PacketError</code>.
     * @param msg The text message for the <code>PacketError</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @return The new <code>PacketError</code>.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, this method is remove. Use {@link
     * #createPacketError(PacketError.Type, String)} instead.
     */
    public PacketError createError(int code, String msg) throws IllegalArgumentException;

    /**
     * <p>
     * Creates a new <code>Packet</code> node with the given name.  This method
     * uses the registered <code>PacketFactory</code> which most closely
     * matches <code>name</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown. Also, this exception
     * is thrown if the "local name" for <code>name</code> is <code>null</code>
     * or "".</p>
     *
     * @param name The name of the <code>Packet</code> to create.
     * @return The created <code>Packet</code>
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.2
     */
    public Packet createPacketNode(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>Packet</code> node with the given name and interface.
     * This method uses the registered <code>PacketFactory</code> which most
     * closely matches <code>name</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown. Also, this exception
     * is thrown if the "local name" for <code>name</code> is <code>null</code>
     * or "".</p>
     *
     * <p>
     * If <code>iface</code> is not supported by the new <code>Packet</code>,
     * this method throws an <code>IllegalArgumentException</code>.</p>
     *
     * @param name The name of the <code>Packet</code> to create
     * @param iface The interface <code>Packet</code> needs to support
     * @return The created <code>Packet</code>
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.8
     */
    public Packet createPacketNode(NSI name, Class iface) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates a new <code>Packet</code> builder for the given name.  This
     * method uses the registered <code>PacketFactory</code> which most closely
     * matches <code>name</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown.Also, this exception
     * is thrown if the "local name" for <code>name</code> is <code>null</code>
     * or "".</p>
     *
     * @param name The name of the packet <code>StreamBuilder</code> to create.
     * @return The named packet <code>StreamBuilder</code>
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.2
     */
    public StreamBuilder createPacketBuilder(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>Packet</code> builder for the given name and
     * interface. This method uses the registered <code>PacketFactory</code>
     * which most closely matches <code>name</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown.Also, this exception
     * is thrown if the "local name" for <code>name</code> is <code>null</code>
     * or "".</p>
     *
     * <p>
     * If <code>iface</code> is not supported by the <code>Packet</code> built
     * by the new <code>StreamBuilder</code>, this method throws an
     * <code>IllegalArgumentException</code>.</p>
     *
     * @param name The name of the packet <code>StreamBuilder</code> to create.
     * @param iface The interface <code>Packet</code> needs to support
     * @return The named packet <code>StreamBuilder</code>
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.8
     */
    public StreamBuilder createPacketBuilder(NSI name, Class iface) throws IllegalArgumentException;

    /**
     * <p>
     * Creates a new <code>Extension</code> node with the given name.  This
     * method uses the registered <code>ExtensionFactory</code> which most
     * closely matches <code>name</code>.  If none can be found, then
     * <code>null</code> is returned.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown.  Also, this exception
     * is thrown if the "namespace URI" for <code>name</code> is
     * <code>null</code> or "".  The exception may be thrown if the "local name"
     * for <code>name</code> is <code>null</code> or "".</p>
     *
     * @param name The name of the <code>Extension</code> to create.
     * @return The named <code>Extension</code>.
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.2
     */
    public Extension createExtensionNode(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>Extension</code> node with the given name and
     * interface. This method uses the registered <code>ExtensionFactory</code>
     * which most closely matches <code>name</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown.  Also, this exception
     * is thrown if the "namespace URI" for <code>name</code> is
     * <code>null</code> or "".  The exception may be thrown if the "local name"
     * for <code>name</code> is <code>null</code> or "".</p>
     *
     * <p>
     * If <code>iface</code> is not supported by the <code>Extension</code>
     * built by the new <code>StreamBuilder</code>, this method throws an
     * <code>IllegalArgumentException</code>.</p>
     *
     * <p>
     * If <code>iface</code> is not supported by the new <code>Extension</code>,
     * this method throws an <code>IllegalArgumentException</code>.</p>
     *
     * @param name The name of the <code>Extension</code> to create
     * @param iface The interface <code>Extension</code> needs to support
     * @return The created <code>Extension</code>
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.8
     */
    public Extension createExtensionNode(NSI name, Class iface) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>Extension</code> builder node with the given name.
     * This method uses the registered <code>ExtensionFactory</code> which most
     * closely matches <code>name</code>.  If none can be found, then
     * <code>null</code> is returned.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown.  Also, this exception
     * is thrown if the "namespace URI" for <code>name</code> is
     * <code>null</code> or "".  The exception may be thrown if the "local name"
     * for <code>name</code> is <code>null</code> or "".</p>
     *
     * @param name The name of the extension <code>StreamBuilder</code> to
     * create.
     * @return The named extension <code>StreamBuilder</code>.
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.2
     */
    public StreamBuilder createExtensionBuilder(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <code>Extension</code> builder node with the given name
     * and interface. This method uses the registered
     * <code>ExtensionFactory</code> which most closely matches
     * <code>name</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or an
     * <code>IllegalArgumentException</code> is thrown.  Also, this exception
     * is thrown if the "namespace URI" for <code>name</code> is
     * <code>null</code> or "".  The exception may be thrown if the "local name"
     * for <code>name</code> is <code>null</code> or "".</p>
     *
     * <p>
     * If <code>iface</code> is not supported by the <code>Extension</code>
     * built by the new <code>StreamBuilder</code>, this method throws an
     * <code>IllegalArgumentException</code>.</p>
     *
     * @param name The name of the extension <code>StreamBuilder</code> to
     * create.
     * @param iface The interface <code>Extension</code> needs to support
     * @return The named extension <code>StreamBuilder</code>.
     * @throws IllegalArgumentException If any parameters are invalid, or would
     * result in returning <code>null</code>.
     * @since   JSO-0.8
     */
    public StreamBuilder createExtensionBuilder(NSI name, Class iface) throws IllegalArgumentException;
}