/*
 * StreamElement.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.io.StreamBuilder;

/**
 * <p>
 * Basic "element" interface for stream-based information.  This has a similar
 * purpose as the "Element" interface for XML DOM.  This interface many
 * (possibly inconsistent) assumptions:
 *
 * <ul>
 *   <li>Each Namespace URI has one and only one prefix, at the current
 *   "node-depth"</li>
 *   <li>Attributes are considered an "integral" information-set in the
 *   <tt>StreamElement</tt>, rather than being a specialized type of
 *   node.</li>
 * </ul></p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamElement extends StreamNode {
    //Methods
    /**
     * <p>
     * Retrieves the set of namespaces declared for this <tt>StreamElement</tt>.
     * Each element of this set is a <tt>StreamNamespace</tt> object. This set
     * may not be modifiable.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The Set of declared <tt>StreamNamespace</tt> objects.
     * @since   JSO-0.4.1
     */
    public Set getDeclaredNamespaces();
    /**
     * <p>
     * Retrieves the <tt>StreamNamespace</tt> for the given URI.  This
     * method attempts to find a matching <tt>StreamNamespace</tt> within
     * this <tt>StreamElement</tt> or its parent.</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt> or "", in which
     * case it is considered "".</p>
     *
     * <p>
     * This method may return <tt>null</tt>.</p>
     *
     * @return The <tt>StreamNamespace</tt> for the the given URI.
     * @since   JSO-0.4.1
     */
    public StreamNamespace getNamespaceByURI(String uri);
    /**
     * <p>
     * Retrieves the <tt>StreamNamespace</tt> for the given prefix.  This
     * method attempts to find a matching <tt>StreamNamespace</tt> within
     * this <tt>StreamElement</tt> or its parent.</p>
     *
     * <p>
     * The value of <tt>prefix</tt> may be <tt>null</tt> or "", in which
     * case it is considered "".</p>
     *
     * <p>
     * This method may return <tt>null</tt>.</p>
     *
     * @return The <tt>StreamNamespace</tt> for the the given prefix.
     * @since   JSO-0.4.1
     */
    public StreamNamespace getNamespaceByPrefix(String prefix);
    /**
     * <p>
     * Retrieves the XML namespace prefix associated to the given namespace
     * URI.  This retrieval is not retricted to the current
     * <tt>StreamElement</tt>, and will search the parent (if defined).</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the namespace is not defined.
     * The string "" is returned if the namespace is the "default" namespace for
     * this <tt>StreamElement</tt> (or its ancesctor).</p>
     *
     * <p>
     * The value of <tt>uri</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param uri The Namespace URI to look for.
     * @return The Namespace Prefix for <tt>uri</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public String getNamespacePrefix(String uri) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the XML namespace URI associated to the given namespace
     * prefix.  This retrieval is not restricted to the current
     * <tt>StreamElement</tt>, and will search the parent (if defined).</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the namespace is not defined.
     * The string "" is returned if the prefix is for a "non-namespace".</p>
     *
     * <p>
     * The value of <tt>prefix</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.  The value can be "",
     * in which case it is interpretted to be a "non-prefix".</p>
     *
     * @param prefix The Namespace prefix to look for.
     * @return The Namespace URI for <tt>prefix</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public String getNamespaceURI(String prefix) throws IllegalArgumentException;
    /**
     * <p>
     * Adds the given namespace (prefix and URI) to this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * If the value of <tt>uri</tt> is <tt>null</tt>, it is interpretted
     * to be "".  If the value of <tt>prefix</tt> is <tt>null</tt>, it
     * is interpretted to be "".  If <tt>uri</tt> is already defined for
     * this namespace, its prefix is replaced.</p>
     *
     * <p>
     * If this namespace cannot be added, for whatever reason other than
     * invalid parameters, this method may throw an
     * <tt>UnsupportedOperationException</tt>.</p>
     *
     * @param prefix The namespace prefix.
     * @param uri The namespace URI.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void addNamespace(String prefix, String uri) throws UnsupportedOperationException;
    /**
     * <p>
     * Removes the given namespace (prefix and URI) from this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * If the value of <tt>uri</tt> is <tt>null</tt>, it is interpretted
     * to be "".  If the value of <tt>prefix</tt> is <tt>null</tt>, it
     * is interpretted to be "".</p>
     *
     * <p>
     * Typically, since only one prefix can be mapped to a URI, the prefix is
     * not strictly necessary, although some implementations may require
     * <tt>prefix</tt> to match the currently defined mapping to function
     * properly.</p>
     *
     * <p>
     * If this namespace cannot be removed, for whatever reason other than
     * invalid parameters, this method may throw an
     * <tt>UnsupportedOperationException</tt>.</p>
     *
     * @param prefix The namespace prefix.
     * @param uri The namespace URI.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void removeNamespace(String prefix, String uri) throws UnsupportedOperationException;

    /**
     * <p>
     * Retrieves the attributes for this <tt>StreamElement</tt>.
     * The returned map uses the attribute name (as an <tt>NSI</tt>) for
     * the key, and the <tt>StreamAttribute</tt> as the value.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.  For
     * (memory and computational) efficiency concerns, try to retain this set,
     * as it may be generated each time it is requested.</p>
     *
     * @return the set of <tt>StreamElement</tt> attributes.
     * @since   JSO-0.2
     */
    public Map getAttributes();
    /**
     * <p>
     * Retrieves the (string) value for the named attribute.  This method will
     * locate the first attribute with the given "local name", regardless of
     * the namespace URI.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the
     * attribute is not defined.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The "local name" of the attribute.
     * @return The (string) value of the attribute.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public String getAttributeValue(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the (string) value for the named attribute.  This method will
     * attempt to locate the attribute best matching the "local name" and
     * namespace URI.  If the URI is valid, it will not attempt to locate a
     * "default" attribute.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the
     * attribute is not defined.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.  The value of
     * <tt>uri</tt> can be <tt>null</tt> (in which case the first
     * attribute with <tt>name</tt> is retrieved), or "".</p>
     *
     * @param name The "local name" of the attribute.
     * @param ns The namespace URI of the attribute.
     * @return The string value of the attribute.
     * @throws IllegalArgumentException If the "local name" is invalid.
     * @since   JSO-0.2
     */
    public String getAttributeValue(String name, String ns) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the (string) value for the named attribute.  This method will
     * attempt to locate the attribute best matching the "local name" and
     * namespace URI.  If the URI is valid, it will not attempt to locate a
     * "default" attribute.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the
     * attribute is not defined.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.  Also, the value
     * returned by <tt>{@link NSI#getLocalName}</tt> cannot be
     * <tt>null</tt> or "", or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param name The <tt>NSI</tt> of the attribute.
     * @return The (string) value of the attribute.
     * @throws IllegalArgumentException If the "local name" is invalid.
     * @since   JSO-0.2
     */
    public String getAttributeValue(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Sets (or removes) the (string) value of the named attribute.  This
     * method will to locate the first attribute with the given "local name",
     * regardless of its namespace URI.  If an existing attribute cannot be
     * located, then a new attribute may be added, using the
     * "non-namespace".</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.  The value of
     * <tt>val</tt> can be <tt>null</tt> (which may result in the
     * attribute being removed from this <tt>StreamElement</tt>).</p>
     *
     * <p>
     * If this attribute cannot be added/modified/removed, for whatever reason
     * other than invalid parameters, this method may throw an
     * <tt>UnsupportedOperationException</tt>.</p>
     *
     * @param name The "local name" of the attribute.
     * @param val The new value for the attribute.
     * @throws IllegalArgumentException If the "local name" is invalid.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void setAttributeValue(String name, String val) throws IllegalArgumentException, UnsupportedOperationException;
    /**
     * <p>
     * Sets (or removes) the (string) value of the named attribute.  This
     * method will attempt to locate the attribute best matching the "local
     * name" and namespace URI.  If the URI is invalid, this method will find
     * the first attribute with the given "local name" (regardless of its
     * namespace).  If an existing attribute cannot be located, then a new
     * attribute may be added, using the given namespace URI (or the
     * "non-namespace" if the URI is not valid).</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.  The value of
     * <tt>uri</tt> can be <tt>null</tt> (in which case the first
     * attribute with the given "local name" is used), or "".  The value of
     * <tt>val</tt> can be <tt>null</tt> (which may result in the
     * attribute being removed from this <tt>StreamElement</tt>).</p>
     *
     * <p>
     * If this attribute cannot be added/modified/removed, for whatever reason
     * other than invalid parameters, this method may throw an
     * <tt>UnsupportedOperationException</tt>.</p>
     *
     * @param name The "local name" of the attribute.
     * @param ns The namespace URI of the attribute.
     * @param val The new value for the attribute.
     * @throws IllegalArgumentException If the "local name" is invalid.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void setAttributeValue(String name, String ns, String val) throws IllegalArgumentException, UnsupportedOperationException;
    /**
     * <p>
     * Sets (or removes) the (string) value of the named attribute.  This
     * method will attempt to locate the attribute best matching the "local
     * name" and namespace URI.  If the URI is invalid, this method will find
     * the first attribute with the given "local name" (regardless of its
     * namespace).  If an existing attribute cannot be located, then a new
     * attribute may be added, using the given namespace URI (or the
     * "non-namespace" if the URI is not valid).</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.  Also, the value
     * returned by <tt>{@link NSI#getLocalName}</tt> cannot be
     * <tt>null</tt> or "", or an <tt>IllegalArgumentException</tt> is
     * thrown.  The value of <tt>val</tt> can be <tt>null</tt> (which
     * may result in the attribute being removed from this
     * <tt>StreamElement</tt>).</p>
     *
     * <p>
     * If this attribute cannot be added/modified/removed, for whatever reason
     * other than invalid parameters, this method may throw an
     * <tt>UnsupportedOperationException</tt>.</p>
     *
     * @param name The <tt>NSI</tt> of the attribute.
     * @param val The new value for the attribute.
     * @throws IllegalArgumentException If the "local name" is invalid.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void setAttributeValue(NSI name, String val) throws IllegalArgumentException, UnsupportedOperationException;
    
    /**
     * <p>
     * Retrieves the identifier value for this <tt>StreamElement</tt>. This
     * value corresponds with the "id" attribute.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * yet been set.</p>
     *
     * @return The ID attribute value for this <tt>StreamElement</tt>.
     * @since   JSO-0.10
     */
    public String getID();
    /**
     * <p>
     * Sets the ID attribute value for this <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>id</tt> may be <tt>null</tt>, in which case the
     * attribute is removed.</p>
     *
     * @param id The new ID attribute value.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.10
     */
    public void setID(String id) throws UnsupportedOperationException;
    
    /**
     * <p>
     * Retrieves the language attribute value for this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If a value
     * is not set, this method returns "".</p>
     *
     * @return The language directive for this <tt>StreamElement</tt>.
     * @since   JSO-0.10
     * @deprecated  As of JSO-0.12, this method is replaced with {@link
     * #getDeclaredLocale}.
     */
    public String getDeclaredLanguage();
    /**
     * <p>
     * Sets the language attribute value for this <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>id</tt> may be <tt>null</tt>, in which case the
     * attribute is removed.</p>
     *
     * @param lang The new language attribute value.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.10
     * @deprecated  As of JSO-0.12, this method is replaced with {@link
     * #setDeclaredLocale}.
     */
    public void setDeclaredLanguage(String lang) throws UnsupportedOperationException;
    
    /**
     * <p>
     * Retrieves the locale for this <tt>StreamElement</tt>. This value
     * corresponds to the "xml:lang" attribute, and expects the syntax defined
     * in RFC-1766.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * previously been set.</p>
     *
     * @return The locale for this <tt>StreamElement</tt>.
     * @since   JSO-0.12
     * @see <a href='http://www.ietf.org/rfc/rfc1766.txt'>RFC-1766 - Tags for
     * the Identification of Languages</a>
     */
    public Locale getDeclaredLocale();
    /**
     * <p>
     * Sets the locale for this <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value of <tt>lang</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param lang The new locale.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.12
     * @see <a href='http://www.ietf.org/rfc/rfc1766.txt'>RFC-1766 - Tags for
     * the Identification of Languages</a>
     */
    public void setDeclaredLocale(Locale lang) throws UnsupportedOperationException;
    
    /**
     * <p>
     * Retrieves the list of <tt>StreamText</tt> objects. This list
     * consists of all the <tt>StreamText</tt> children of this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The list of directly-contained <tt>StreamText</tt> objects.
     * @since   JSO-0.8
     */
    public List listText();
    /**
     * <p>
     * Retrieves a "normalization" of all the <tt>StreamText</tt> nodes
     * within this <tt>StreamElement</tt>, as a single string.  The String
     * returned by this method does not trim whitespace from the
     * <tt>StreamText</tt> values before concatenating.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, although
     * it may be "".</p>
     * 
     * @return The "normalized" text within this <tt>StreamElement</tt>.
     * @since   JSO-0.2
     */
    public String normalizeText();
    /**
     * <p>
     * Retrieves a "normalization" of all the <tt>StreamText</tt> nodes
     * within this <tt>StreamElement</tt>, as a string string.  The String
     * returned by this method trims whitespace from the <tt>StreamText</tt>
     * values before concatenating, and places a single space (' ') between
     * the values for each found <tt>StreamText</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, although
     * it may be "".</p>
     *
     * @return The "normalized" text within this <tt>StreamElement</tt>.
     * @since   JSO-0.2
     */
    public String normalizeTrimText();
    
    /**
     * <p>
     * Adds a new <tt>StreamText</tt> child to this <tt>StreamElement</tt>.
     * This method uses the <tt>StreamDataFactory</tt> from {@link
     * #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>txt</tt> may be <tt>null</tt> or "", in which case
     * no actions are performed.</p>
     *
     * @param txt The string to create from.
     * @return The newly created and added <tt>StreamText</tt>.
     * @since   JSO-0.12
     */
    public StreamText addText(String txt);
    
    /**
     * <p>
     * Removes all child <tt>StreamText</tt> objects from this
     * <tt>StreamElement</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearText();
    
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects.  This list
     * consists of all <tt>StreamElement</tt> children, regardless of
     * name.  The ordering in this list follows the order in which they are
     * located.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The list of directly-contained <tt>StreamElement</tt>
     * objects.
     * @since   JSO-0.2
     */
    public List listElements();
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects that are
     * assignable to the given class. The ordering in this list follows the
     * order in which they are located.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param iface The class each <tt>StreamElement</tt> is assignable to.
     * @return The list of directly-contained <tt>StreamElement</tt>
     * objects that are instances of <tt>iface</tt>.
     * @since   JSO-0.8
     */
    public List listElements(Class iface);
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects with the given
     * local name.  This list consists of all <tt>StreamElement</tt>
     * children with the given local name, regardless of namespace URIs.  The
     * ordering in this list follows the order in which they are located.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of <tt>{@link #listElements()}</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param name  The <tt>StreamElement</tt> local name to locate by.
     * @return The list of directly-contained <tt>StreamElement</codee>
     * objects.
     * @since   JSO-0.2
     */
    public List listElements(String name);
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects with the given
     * local name and assignable to the given class.  The ordering in this list
     * follows the order in which they are located.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of <tt>{@link #listElements()}</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param name  The <tt>StreamElement</tt> local name to locate by.
     * @param iface The class each <tt>StreamElement</tt> is assignable to.
     * @return The list of directly-contained <tt>StreamElement</codee>
     * objects.
     * @since   JSO-0.8
     */
    public List listElements(String name, Class iface);
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects with the given
     * local name and/or namespace URI.  The ordering in this list follows the
     * order in which they are located.</p>
     *
     * <p>
     * If both <tt>name</tt> and <tt>ns</tt> are <tt>null</tt> or
     * "", this method behaves identically to <tt>{@link
     * #listElements()}</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case it locates all <tt>StreamElement</tt> children with a namespace
     * URI of <tt>ns</tt>, regardless of local name../p>
     *
     * <p>
     * The value of <tt>ns</tt> can be <tt>null</tt> or "", in which
     * case it locates all <tt>StreamElement</tt> children with the local
     * name <tt>name</tt>, regardless of namespace.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param name The <tt>StreamElement</tt> local name to locate by.
     * @param ns The <tt>StreamElement</tt> namespace URI to locate by.
     * @return The list of directly-contained <tt>StreamElement</codee>
     * objects.
     * @since   JSO-0.2
     */
    public List listElements(String name, String ns);
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects with the given
     * local name and/or namespace URI, and assignable to the given class.  The
     * ordering in this list follows the order in which they are located.</p>
     *
     * <p>
     * If both <tt>name</tt> and <tt>ns</tt> are <tt>null</tt> or
     * "", this method behaves identically to <tt>{@link
     * #listElements()}</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case it locates all <tt>StreamElement</tt> children with a namespace
     * URI of <tt>ns</tt>.</p>
     *
     * <p>
     * The value of <tt>ns</tt> can be <tt>null</tt> or "", in which
     * case it locates all <tt>StreamElement</tt> children with the local
     * name <tt>name</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param name The <tt>StreamElement</tt> local name to locate by.
     * @param ns The <tt>StreamElement</tt> namespace URI to locate by.
     * @param iface The class each <tt>StreamElement</tt> is assignable to.
     * @return The list of directly-contained <tt>StreamElement</codee>
     * objects.
     * @since   JSO-0.8
     */
    public List listElements(String name, String ns, Class iface);
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects with the given
     * name (in <tt>NSI</tt> form).  The ordering in this list follows the
     * order in which they are located.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt>, in which case
     * this method behaves identically to <tt>{@link #listElements()}</tt>.
     * Otherwise, the behavior depends upon the local name and namespace URI
     * reported by <tt>name</tt>.</p>
     *
     * <p>
     * If the value returned by <tt>{@link NSI#getLocalName}</tt> is
     * <tt>null</tt> or "", all <tt>StreamElement</tt> children with
     * the reported namespace URI are returned.  If the value returned by
     * <tt>{@link NSI#getNamespaceURI}</tt> is <tt>null</tt> or "", all
     * <tt>StreamElement</tt> children with the reported local name are
     * returned.  If both methods return <tt>null</tt> or "", this method
     * behaves identically to <tt>{@link #listElements()}</tt>.</p>
     *
     * @param name The "full" <tt>StreamElement</tt> name to locate by.
     * @return The list of directly-contained <tt>StreamElement</codee>
     * objects.
     * @since   JSO-0.2
     */
    public List listElements(NSI name);
    /**
     * <p>
     * Retrieves the list of <tt>StreamElement</tt> objects with the given
     * name (in <tt>NSI</tt> form), and assignable to the given class. The
     * ordering in this list follows the order in which they are located.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt>, in which case
     * this method behaves identically to <tt>{@link #listElements()}</tt>.
     * Otherwise, the behavior depends upon the local name and namespace URI
     * reported by <tt>name</tt>.</p>
     *
     * <p>
     * If the value returned by <tt>{@link NSI#getLocalName}</tt> is
     * <tt>null</tt> or "", all <tt>StreamElement</tt> children with
     * the reported namespace URI are returned.  If the value returned by
     * <tt>{@link NSI#getNamespaceURI}</tt> is <tt>null</tt> or "", all
     * <tt>StreamElement</tt> children with the reported local name are
     * returned.  If both methods return <tt>null</tt> or "", this method
     * behaves identically to <tt>{@link #listElements()}</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param name The "full" <tt>StreamElement</tt> name to locate by.
     * @param iface The class each <tt>StreamElement</tt> is assignable to.
     * @return The list of directly-contained <tt>StreamElement</codee>
     * objects.
     * @since   JSO-0.8
     */
    public List listElements(NSI name, Class iface);
    
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child in this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>, if there are
     * no <tt>StreamElement</tt> children.</p>
     *
     * @return First <tt>StreamElement</tt> child.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement();
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child assignable to the given
     * class in this <tt>StreamElement</tt>. This search is performed
     * regardless of name.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case this
     * method behaves exactly like {@link #getFirstElement()}.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no matching
     * <tt>StreamElement</tt> could be found.</p>
     *
     * @param iface The class the <tt>StreamElement</tt> is assignable to.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(Class iface);
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child with the given
     * local name.  This search matches the first <tt>StreamElement</tt>
     * child with the given local name, regardless of namespace URIs.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of <tt>{@link #getFirstElement()}</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no matching
     * <tt>StreamElement</tt> could be found.</p>
     *
     * @param name  The <tt>StreamElement</tt> local name to locate by.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(String name);
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child with the given
     * local name and assignable to the given class.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of {@link #getFirstElement()}.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no matching
     * <tt>StreamElement</tt> could be found.</p>
     *
     * @param name  The <tt>StreamElement</tt> local name to locate by.
     * @param iface The class the <tt>StreamElement</tt> is assignable to.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(String name, Class iface);
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child with the given
     * local name and/or namespace URI.</p>
     *
     * <p>
     * If both <tt>name</tt> and <tt>ns</tt> are <tt>null</tt> or
     * "", this method behaves identically to <tt>{@link
     * #getFirstElement()}</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case it locates all <tt>StreamElement</tt> children with a namespace
     * URI of <tt>ns</tt>, regardless of local name../p>
     *
     * <p>
     * The value of <tt>ns</tt> can be <tt>null</tt> or "", in which
     * case it locates all <tt>StreamElement</tt> children with the local
     * name <tt>name</tt>, regardless of namespace.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no matching
     * <tt>StreamElement</tt> could be found.</p>
     *
     * @param name The <tt>StreamElement</tt> local name to locate by.
     * @param ns The <tt>StreamElement</tt> namespace URI to locate by.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(String name, String ns);
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child with the given
     * local name and/or namespace URI, and assignable to the given class.</p>
     *
     * <p>
     * If both <tt>name</tt> and <tt>ns</tt> are <tt>null</tt> or "", this
     * method behaves identically to {@link #getFirstElement()}.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt> or "", in which
     * case it locates the first <tt>StreamElement</tt> child with a namespace
     * URI of <tt>ns</tt>.</p>
     *
     * <p>
     * The value of <tt>ns</tt> can be <tt>null</tt> or "", in which case it
     * locates the first <tt>StreamElement</tt> child with the local name
     * <tt>name</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no matching
     * <tt>StreamElement</tt> could be found.</p>
     *
     * @param name The <tt>StreamElement</tt> local name to locate by.
     * @param ns The <tt>StreamElement</tt> namespace URI to locate by.
     * @param iface The class the <tt>StreamElement</tt> is assignable to.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(String name, String ns, Class iface);
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child with the given
     * name (in <tt>NSI</tt> form).</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt>, in which case
     * this method behaves identically to {@link #getFirstElement()}. Otherwise, the
     * behavior depends upon the local name and namespace URI
     * reported by <tt>name</tt>.</p>
     *
     * <p>
     * If the value returned by {@link NSI#getLocalName} is <tt>null</tt> or "",
     * the first <tt>StreamElement</tt> child with the reported namespace URI
     * is returned.  If the value returned by {@link NSI#getNamespaceURI} is
     * <tt>null</tt> or "", the first <tt>StreamElement</tt> child with the
     * reported local name is returned.  If both methods return <tt>null</tt>
     * or "", this method behaves identically to {@link #getFirstElement()}.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no matching
     * <tt>StreamElement</tt> could be found.</p>
     *
     * @param name The "full" <tt>StreamElement</tt> name to locate by.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(NSI name);
    /**
     * <p>
     * Retrieves the first <tt>StreamElement</tt> child with the given
     * name (in <tt>NSI</tt> form), and assignable to the given class.</p>
     *
     * <p>
     * The value of <tt>name</tt> can be <tt>null</tt>, in which case
     * this method behaves identically to {@link #getFirstElement()}. Otherwise, the
     * behavior depends upon the local name and namespace URI
     * reported by <tt>name</tt>.</p>
     *
     * <p>
     * If the value returned by {@link NSI#getLocalName} is <tt>null</tt> or
     * "", the first <tt>StreamElement</tt> child with the reported namespace
     * URI is returned.  If the value returned by {@link NSI#getNamespaceURI} is
     * <tt>null</tt> or "", the first <tt>StreamElement</tt> child with the
     * reported local name is returned.  If both methods return <tt>null</tt>
     * or "", this method behaves identically to {@link #getFirstElement()}</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param name The "full" <tt>StreamElement</tt> name to locate by.
     * @param iface The class the <tt>StreamElement</tt> is assignable to.
     * @return The first matching <tt>StreamElement</tt>.
     * @since   JSO-0.12
     */
    public StreamElement getFirstElement(NSI name, Class iface);
    
    /**
     * <p>
     * Adds a new <tt>StreamElement</tt> child to this <tt>StreamElement</tt>
     * with the given local name. This method uses the
     * <tt>StreamDataFactory</tt> from {@link #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the values of <tt>name</tt> does not match a registered
     * <tt>StreamElementFactory</tt>, an  <tt>IllegalArguementException</tt> is
     * thrown.</p>
     *
     * @param name The local name to create from.
     * @return The newly created and added <tt>StreamElement</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement addElement(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Adds a new <tt>StreamElement</tt> child to this <tt>StreamElement</tt>
     * with the given local name and interface. This method uses the
     * <tt>StreamDataFactory</tt> from {@link #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * <p>
     * If the values of <tt>name</tt> and <tt>iface</tt> do not match a
     * registered <tt>StreamElementFactory</tt>, an
     * <tt>IllegalArguementException</tt> is thrown.</p>
     *
     * @param name The local name to create from.
     * @param iface The class to be assignable to.
     * @return The newly created and added <tt>StreamElement</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement addElement(String name, Class iface) throws IllegalArgumentException;
    /**
     * <p>
     * Adds a new <tt>StreamElement</tt> child to this <tt>StreamElement</tt>
     * with the given local name and namespace URI. This method uses the
     * <tt>StreamDataFactory</tt> from {@link #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case a
     * default value (selected by the matching <tt>StreamElementFactory</tt>)
     * will be used if possible.</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt> or "", in which case a
     * default value (selected by the matching <tt>StreamElementFactory</tt>)
     * will be used if possible.</p>
     *
     * <p>
     * If both <tt>name</tt> and <tt>uri</tt> are <tt>null</tt> or "", an
     * <tt>IllegalArgumentException</tt> is thrown. If the values of
     * <tt>name</tt> and <tt>uri</tt> do not match a registered
     * <tt>StreamElementFactory</tt>, an <tt>IllegalArguementException</tt> is
     * thrown.</p>
     *
     * @param name The local name to create from.
     * @param uri The namespace URI to create from.
     * @return The newly created and added <tt>StreamElement</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement addElement(String name, String uri) throws IllegalArgumentException;
    /**
     * <p>
     * Adds a new <tt>StreamElement</tt> child to this <tt>StreamElement</tt>
     * with the given local name, namespace URI, and interface. This method
     * uses the <tt>StreamDataFactory</tt> from {@link #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case a
     * default value (selected by the matching <tt>StreamElementFactory</tt>)
     * will be used if possible.</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt> or "", in which case a
     * default value (selected by the matching <tt>StreamElementFactory</tt>)
     * will be used if possible.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * <p>
     * If both <tt>name</tt> and <tt>uri</tt> are <tt>null</tt> or "", an
     * <tt>IllegalArgumentException</tt> is thrown. If the values of
     * <tt>name</tt>, <tt>uri</tt>, and <tt>iface</tt> do not match a
     * registered <tt>StreamElementFactory</tt>, an
     * <tt>IllegalArguementException</tt> is thrown.</p>
     *
     * @param name The local name to create from.
     * @param uri The namespace URI to create from.
     * @param iface The class to be assignable to.
     * @return The newly created and added <tt>StreamElement</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement addElement(String name, String uri, Class iface) throws IllegalArgumentException;
    /**
     * <p>
     * Adds a new <tt>StreamElement</tt> child to this <tt>StreamElement</tt>
     * with the given <tt>NSI</tt>. This method uses the
     * <tt>StreamDataFactory</tt> from {@link #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, this <tt>NSI</tt>
     * must at least have a non-empty string value for either the local name
     * or namespace URI, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the values of <tt>name</tt> does not match a registered
     * <tt>StreamElementFactory</tt>, an <tt>IllegalArguementException</tt> is
     * thrown.</p>
     *
     * @param name The <tt>NSI</tt> to create from.
     * @return The newly created and added <tt>StreamElement</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement addElement(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Adds a new <tt>StreamElement</tt> child to this <tt>StreamElement</tt>
     * with the given <tt>NSI</tt> and interface. This method uses the
     * <tt>StreamDataFactory</tt> from {@link #getDataFactory}.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, this <tt>NSI</tt>
     * must at least have a non-empty string value for either the local name
     * or namespace URI, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * <p>
     * If the values of <tt>name</tt> and <tt>iface</tt> do not match a
     * registered <tt>StreamElementFactory</tt>, an
     * <tt>IllegalArguementException</tt> is thrown.</p>
     *
     * @param name The <tt>NSI</tt> to create from.
     * @param iface The class to be assignable to.
     * @return The newly created and added <tt>StreamElement</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public StreamElement addElement(NSI name, Class iface) throws IllegalArgumentException;
    
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearElements();
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given class.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param iface The class each <tt>StreamElement</tt> implements.
     * @since   JSO-0.12
     */
    public void clearElements(Class iface);
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given local name. The namespace URI is
     * ignored.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * it is ignored.</p>
     *
     * @param name The local name to remove by.
     * @since   JSO-0.12
     */
    public void clearElements(String name);
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given local name and class. The
     * namespace URI is ignored.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * it is ignored.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param name The local name to remove by.
     * @param iface The class each <tt>StreamElement</tt> implements.
     * @since   JSO-0.12
     */
    public void clearElements(String name, Class iface);
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given local name and namespace URI.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * it is ignored.</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt> or "", in which case
     * it is ignored.</p>
     *
     * @param name The local name to remove by.
     * @param uri The namespace URI to remove by.
     * @since   JSO-0.12
     */
    public void clearElements(String name, String uri);
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given local name, namespace URI, and
     * class.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * it is ignored.</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt> or "", in which case
     * it is ignored.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param name The local name to remove by.
     * @param uri The namespace URI to remove by.
     * @param iface The class each <tt>StreamElement</tt> implements.
     * @since   JSO-0.12
     */
    public void clearElements(String name, String uri, Class iface);
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given name.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt>, in which case
     * it is ignored. The value of {@link NSI#getLocalName}
     * may be <tt>null</tt> or "", in which case it is ignored and the
     * namespace URI is used. The value of {@link NSI#getNamespaceURI} may
     * be <tt>null</tt> or "", in which case it is ignored and the local name
     * is used. If both the "local name" and "namespace URI" are
     * <tt>null</tt> or "", this value is ignored.</p>
     *
     * @param name The <tt>NSI</tt> to remove by.
     * @since   JSO-0.12
     */
    public void clearElements(NSI name);
    /**
     * <p>
     * Removes all child <tt>StreamElement</tt> objects from this
     * <tt>StreamElement</tt> with the given name and class.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt>, in which case
     * it is ignored. The value of {@link NSI#getLocalName}
     * may be <tt>null</tt> or "", in which case it is ignored and the
     * namespace URI is used. The value of {@link NSI#getNamespaceURI} may
     * be <tt>null</tt> or "", in which case it is ignored and the local name
     * is used. If both the "local name" and "namespace URI" are
     * <tt>null</tt> or "", this value is ignored.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case
     * it is ignored.</p>
     *
     * @param name The <tt>NSI</tt> to remove by.
     * @param iface The class each <tt>StreamElement</tt> implements.
     * @since   JSO-0.12
     */
    public void clearElements(NSI name, Class iface);

    /**
     * <p>
     * Finds the index of the given <tt>StreamNode</tt> within this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * This method returns -1 if <tt>node</tt> is not present, or if the
     * value of <tt>node</tt> is <tt>null</tt>.</p>
     *
     * @param node The <tt>StreamNode</tt> to find.
     * @return The index of <tt>node</tt>, or -1 if not present.
     * @since   JSO-0.8
     */
    public int indexOf(StreamNode node);
    /**
     * <p>
     * Adds the given <tt>StreamNode</tt> as a child to this
     * <tt>StremElement</tt>.</p>
     *
     * @param node The <tt>StreamNode</tt> to add.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void add(StreamNode node) throws UnsupportedOperationException;
    /**
     * <p>
     * Inserts the given <tt>StreamNode</tt> at the given index, as a
     * child to this <tt>StreamElement</tt>.</p>
     *
     * <p>
     * If the value of <tt>idx</tt> is less than 0, either it will be
     * considered 0 or an <tt>IllegalArgumentException</tt> may be thrown.
     * If the value of <tt>idx</idx> is greater than or equal to the number
     * of child nodes already contained, this method performs the same
     * operation that <tt>{@link #add}</tt> would.</p>
     *
     * <p>
     * If the value of <tt>node</tt> is <tt>null</tt>, no actions are
     * performed.  If <tt>node</tt> already has another parent, and that
     * parent is not this <tt>StreamElement</tt>, this method throws an
     * <tt>IllegalArgumentException</tt>.</p>
     *
     * @param idx  The index to insert the <tt>StreamElement</tt>.
     * @param node The <tt>StreamNode</tt> to insert.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void insert(int idx, StreamNode node) throws IllegalArgumentException, UnsupportedOperationException;
    /**
     * <p>
     * Remove the given <tt>StreamNode</tt> from this
     * <tt>StremElement</tt>.</p>
     *
     * @param node The <tt>StreamNode</tt> to remove.
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.2
     */
    public void remove(StreamNode node) throws UnsupportedOperationException;
    /**
     * <p>
     * Removes all <tt>StreamNode</tt> objects from this
     * <tt>StreamElement</tt>.</p>
     *
     * @throws UnsupportedOperationException If the change is not permitted.
     * @since   JSO-0.12
     */
    public void clear() throws UnsupportedOperationException;

    
    /**
     * <p>
     * Creates or retrieves a <tt>StreamBuilder</tt> for this
     * <tt>StreamElement</tt>. The method {@link StreamBuilder#getNode}
     * will always return this instance of <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamBuilder</tt> to build this
     * <tt>StreamElement</tt>
     * @since   JSO-0.8
     */
    public StreamBuilder createBuilder();
    
    
    /**
     * <p>
     * Resets this <tt>StreamElement</tt>. This method operates as if
     * calling {@link #reset(StreamElement)} with a value of
     * <tt>null</tt>.</p>
     *
     * @since   JSO-0.6
     */
    public void reset();
    /**
     * <p>Resets this <tt>StreamElement</tt>, using the given
     * <tt>NSI</tt>. This method operates as if calling {@link
     * #reset(StreamElement)} with a value of <tt>null</tt>, then sets
     * the <tt>NSI</tt> to <tt>name</tt>.</p>
     *
     * <p>
     * If the value of <tt>name</tt> is <tt>null</tt>, the current
     * <tt>NSI</tt> is not changed. The value returned by {@link
     * #getLocalName} cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown. An
     * <tt>IllegalArgumentException</tt> may also be thrown if any part of
     * <tt>name</tt> is not valid for the implementation.</p>
     *
     * @param name The new <tt>NSI</tt>.
     * @throws IllegalArgumentException If the parameter is not valid.
     * @since   JSO-0.8
     */
    public void reset(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Resets this <tt>StreamElement</tt> to be "identical" to the given
     * <tt>StreamElement</tt>. This method first clears this
     * <tt>streamElement</tt> of all its namespace declarations, attributes,
     * and child nodes. Then all of the namespace declarations, attributes, and
     * child nodes from <tt>base</tt> are copied to this
     * <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The exact behavior of this method is implementation-dependent. Many
     * implementations may impose requirements over <tt>base</tt>, such as
     * having the same {@link #getLocalName} and/or {@link #getNamespaceURI}.
     * If <tt>base</tt> does not meet the requirements, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>base</tt> may be <tt>null</tt>, in which case
     * this <tt>StreamElement</tt> is cleared.</p>
     *
     * @param base The <tt>StreamElement</tt> to reset from.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.6
     */
    public void reset(StreamElement base) throws IllegalArgumentException;
    /**
     * <p>
     * Resets this <tt>StreamElement</tt> to be "identical" to the given
     * <tt>StreamElement</tt>, and using the given name. This method first
     * clears this <tt>streamElement</tt> of all its namespace
     * declarations, attributes, and child nodes. Then all of the namespace
     * declarations, attributes, and child nodes from <tt>base</tt> are
     * copied to this <tt>StreamElement</tt>.</p>
     *
     * <p>
     * The exact behavior of this method is implementation-dependent. Many
     * implementations may impose requirements over <tt>base</tt> and/or
     * <tt>name</tt>, such as having the same {@link #getLocalName} and/or
     * {@link #getNamespaceURI}. If <tt>base</tt> does not meet the
     * requirements, an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt>, in which case
     * {@link #getNSI} from <tt>base</tt> is used (if not <tt>null</tt>)
     * or is ignored.</p>
     *
     * <p>
     * The value of <tt>base</tt> may be <tt>null</tt>, in which case
     * this <tt>StreamElement</tt> is cleared.</p>
     *
     * @param name The <tt>NSI</tt> to reset to.
     * @param base The <tt>StreamElement</tt> to reset from.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.8
     */
    public void reset(NSI name, StreamElement base) throws IllegalArgumentException;
    /**
     * <p>
     * Resets this <tt>StreamElement</tt> to be "identical" to the given
     * XML string. This method first parses <tt>xml</tt>, then performs the
     * same operations as {@link #reset(StreamElement)}.</p>
     *
     * <p>
     * The value of <tt>xml</tt> must be a valid XML document fragment, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param xml The XML string to reset to.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void reset(String xml) throws IllegalArgumentException;
}