/*
 * StreamElementFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.Set;

import org.jabberstudio.jso.io.StreamBuilder;

/**
 * <p>
 * Factory interface for creating <tt>StreamElement</tt> nodes and builders.
 * This interface is implemented both by providers of <tt>StreamElement</tt>
 * implementations and by centralized factory objects.</p>
 *
 * <p>
 * This interface represents a "service". Implementations of this interface
 * can be automatically registered at class-loading time via the SPI
 * process, using the service name
 * "org.jabberstudio.jso.io.StreamElementFactory".</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.10
 * @see <a href='http://java.sun.com/j2se/1.4.2/docs/guide/jar/jar.html#Service%20Provider'>JAR File Specification :: Service Provider</a>
 */
public interface StreamElementFactory {
    //Methods
    /**
     * <p>
     * Retrieves the set of supported <tt>StreamElement</tt> names, as
     * <tt>NSI</tt> objects.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The set of supported <tt>StreamElement</tt> names.
     * @since   JSO-0.10
     */
    public Set getSupportedElements();
    /**
     * <p>
     * Determines if the given <tt>NSI</tt> and class are supported by this
     * <tt>StreamElementFactory</tt>. This method returns <tt>true</tt> only
     * if both a <tt>StreamElement</tt> and <tt>StreamBuilder</tt> can be
     * created with the given <tt>NSI</tt> and class.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt>, in which case an
     * empty <tt>NSI</tt> is used.</p>
     *
     * <p>
     * The vallue of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * @param name The <tt>NSI</tt> to check against.
     * @param iface The <tt>Class</tt> to check against.
     * @return <tt>true</tt> if <tt>StreamElement</tt> nodes and builders can
     * be created with the given arguments
     * @since   JSO-0.10
     */
    public boolean isSupportedElement(NSI name, Class iface);
    
    /**
     * <p>
     * Creates a new <tt>StreamElement</tt> with the given <tt>NSI</tt>,class,
     * and <tt>StreamDataFactory</tt>.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, the value of {@link
     * NSI#getLocalName} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored. a <tt>StreamElement</tt> cannot be created with the given
     * class, an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>sdf</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thown.</p>
     *
     * @param name The <tt>NSI</tt> to create with.
     * @param iface The <tt>Class</tt> to create with.
     * @param sdf The creating <tt>StreamDataFactory</tt>.
     * @return The <tt>StreamElement</tt> for the given name and class.
     * @throws IllegalArgumentException If the parameters are invalid.
     * @since   JSO-0.10
     */
    public StreamElement createElementNode(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException;
    /**
     * <p>
     * Creates a new <tt>StreamBuilder</tt> with the given <tt>NSI</tt> and
     * class.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, the value of {@link
     * NSI#getLocalName} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> may be <tt>null</tt>, in which case it is
     * ignored. a <tt>StreamBuilder</tt> cannot be created with the given
     * class, an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>sdf</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thown.</p>
     *
     * @param name The <tt>NSI</tt> to create with.
     * @param iface The <tt>Class</tt> to create with.
     * @param sdf The creating <tt>StreamDataFactory</tt>.
     * @return The <tt>StreamBuilder</tt> for the given name and class.
     * @throws IllegalArgumentException If the parameters are invalid.
     * @since   JSO-0.10
     */
    public StreamBuilder createElementBuilder(NSI name, Class iface, StreamDataFactory sdf) throws IllegalArgumentException;
}