/*
 * StreamException.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Root for all "serious" Stream-related errors within JSO.  This is used to
 * generate the &lt;stream:error&gt; element.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class StreamException extends java.lang.Exception {
    //"Constants"
    /**
     * <p>NSI for <tt>StreamException</tt>.
     *
     * @deprecated As of JSO-0.12, this field is removed without replacement.
     */
    public static final NSI NSI = new NSI("error", null);
    
    //Fields
    private StreamError     _SE;
    
    //Constructors
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> without a detail
     * message or nested exception.</p>
     *
     * <p>
     * This constructor initializes the contained <tt>StreamError</tt> with one
     * with the defined condition {@link StreamError#UNDEFINED_CONDITION} and
     * without an error message. The <tt>StreamError</tt> is created using the
     * <tt>StreamDataFactory</tt> from the global
     * <tt>JSOImplementation</tt>.</p>
     *
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this constructor is replaced with
     * {@link #StreamException(StreamError)}.
     */
    public StreamException() {
        this(createStreamError(StreamError.UNDEFINED_CONDITION, null), (String)null);
    }
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given nested
     * exception.</p>
     *
     * <p>
     * This constructor initializes the contained <tt>StreamError</tt> with one
     * with the defined condition {@link StreamError#UNDEFINED_CONDITION} and
     * an error message based on <tt>t</tt>. The <tt>StreamError</tt> is
     * created using the <tt>StreamDataFactory</tt> from the global
     * <tt>JSOImplementation</tt>.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case
     * this constructor behaves like {@link #StreamException()}.</p>
     *
     * @param t The nested exception.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this constructor is replaced with
     * {@link #StreamException(StreamError, Throwable)}.
     */
    protected StreamException(Throwable t) {
        this(createStreamError(StreamError.UNDEFINED_CONDITION, (t != null ? t.getMessage() : null)), t);
    }
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given detail
     * message.</p>
     *
     * <p>
     * This constructor initializes the contained <tt>StreamError</tt> with one
     * with the defined condition {@link StreamError#UNDEFINED_CONDITION} and
     * the given error message. The <tt>StreamError</tt> is created using the
     * <tt>StreamDataFactory</tt> from the global
     * <tt>JSOImplementation</tt>.</p>
     *
     * <p>
     * The value of <tt>msg</tt> can be <tt>null</tt>, in which case
     * this constructor behaves like {@link #StreamException()}.</p>
     *
     * @param msg The detail message.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this constructor is replaced with
     * {@link #StreamException(StreamError)}.
     */
    public StreamException(String msg) {
        this(createStreamError(StreamError.UNDEFINED_CONDITION, msg), msg);
    }
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given detail
     * message and nested exception.</p>
     *
     * <p>
     * This constructor initializes the contained <tt>StreamError</tt> with one
     * with the defined condition {@link StreamError#UNDEFINED_CONDITION} and
     * the given error message. The <tt>StreamError</tt> is created using the
     * <tt>StreamDataFactory</tt> from the global
     * <tt>JSOImplementation</tt>.</p>
     *
     * <p>
     * The value of <tt>msg</tt> can be <tt>null</tt>, in which case
     * this constructor behaves like {@link #StreamException(StreamError,
     * Throwable)}.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case
     * this constructors behaves like {@link #StreamException()}.</p>
     *
     * @param msg The detail message.
     * @param t The nested exception.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this constructor is replaced with
     * {@link #StreamException(StreamError, Throwable)}.
     */
    protected StreamException(String msg, Throwable t) {
        this(createStreamError(StreamError.UNDEFINED_CONDITION, msg), t);
    }
    
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given
     * <tt>StreamError</tt>.</p>
     *
     * <p>
     * The value of <tt>err</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param err The <tt>StreamError</tt> being thrown.
     * @since   JSO-0.12
     */
    public StreamException(StreamError err) throws NullPointerException {
        this(err, err.getText());
    }
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given
     * <tt>StreamError</tt> and nested exception.</p>
     *
     * <p>
     * The value of <tt>err</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case this
     * constructors behaves like {@link #StreamException(StreamError)}.</p>
     *
     * @param err The <tt>StreamError</tt> being thrown.
     * @param t The nested exception.
     * @since   JSO-0.12
     */
    public StreamException(StreamError err, Throwable t) {
        this(err, (Utilities.isValidString(err.getText()) ? err.getText() : (t != null) ? t.getMessage() : null), t);
    }
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given
     * <tt>StreamError</tt> and error message.</p>
     *
     * <p>
     * The value of <tt>err</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>msg</tt> can be <tt>null</tt> or "".</p>
     *
     * @param err The <tt>StreamError</tt> being thrown.
     * @param msg The detail message.
     * @since   JSO-0.12
     */
    public StreamException(StreamError err, String msg) {
        super(msg);
        
        _SE = err;
    }
    /**
     * <p>
     * Constructs a new <tt>StreamException</tt> with the given
     * <tt>StreamError</tt> and nested exception.</p>
     *
     * <p>
     * The value of <tt>err</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>msg</tt> can be <tt>null</tt> or "".</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case this
     * constructors behaves like {@link #StreamException(StreamError,
     * String)}.</p>
     *
     * @param err The <tt>StreamError</tt> being thrown.
     * @param msg The detail message.
     * @param t The nested exception.
     * @since   JSO-0.12
     */
    public StreamException(StreamError err, String msg, Throwable t) {
        super(msg, t);
        
        _SE = err;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>StreamEror</tt> for this <tt>PacketException</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamError</tt> being thrown.
     * @since   JSO-0.12
     */
    public StreamError getStreamError() {
        return _SE;
    }
    
    /**
     * <p>
     * Generates a <tt>StreamElement</tt> representing this
     * <tt>StreamException</tt>.  This method is used by the
     * <tt>Stream</tt> to transmit the &lt;stream:error&gt; element.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> cannot be <tt>null</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param ctx The owning <tt>StreamContext</tt>.
     * @return The <tt>StreamElement</tt> representation.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, this method is removed. Use the result from
     * {@link #getStreamError} instead.
     */
    public StreamElement asNode(StreamContext ctx) {
        return getStreamError();
    }
    
    private static StreamError createStreamError(String cond, String msg) {
        StreamError     err = JSOImplementation.getInstance().getDataFactory().createStreamError(cond);
        
        err.setText(msg);
        
        return err;
    }
}