/**
 * StreamObject.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.List;
import java.util.Map;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;

/**
 * <p>
 * Basic data unit for any stream-based information.  This interface and
 * its immediate derivations can be used to construct the XML-formatted
 * stream data.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamObject extends Cloneable, java.io.Serializable {
    //Methods
    /**
     * <p>
     * Retrieves the <code>StreamContext</code> for this
     * <code>StreamObject</code>. The <code>StreamContext</code> has a similar
     * role to the DOM Document.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code> if there is
     * not owning <code>StreamContext</code> defined.</p>
     *
     * @return The owning <code>StreamContext</code>. 
     * @since JSO-0.4
     */
    public StreamContext getContext();
    
    /**
     * <p>
     * Retrieves the parent <code>StreamElement</code> for this
     * <code>StreamObject</code>.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if the
     * parent has not yet been defined.</p>
     *
     * @return The <code>StreamElement</code> parent.
     * @since   JSO-0.2
     */
    public StreamElement getParent();
    /**
     * <p>
     * Sets the parent <code>StreamElement</code> for this
     * <code>StreamObject</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case
     * the current parent (if any)for this <code>StreamElement</code> is
     * detached.  If the parent cannot be changed, this method throws an
     * <code>IllegalStateException</code>.</p>
     *
     * @param p the <code>StreamElement</code> parent.
     * @throws IllegalStateException If the parent cannot be changed.
     * @since   JSO-0.2
     */
    public void setParent(StreamElement p) throws IllegalStateException;

    /**
     * <p>
     * Retrieves the <tt>StreamDataFactory</tt> for this <tt>StreamObject</tt>.
     * This is the <tt>StreamDataFactory</tt> that created this
     * <tt>StreamObject</tt>.</p>
     *
     * <p>
     * The value returned by this method should never be <tt>null</tt>.</p>
     *
     * @return The creating <tt>StreamDataFactory</tt>
     * @since   JSO-0.12
     */
    public StreamDataFactory getDataFactory();

    /**
     * <p>
     * Retrieves the <code>NSI</code> for this <code>StreamObject</code>. This
     * value is used for naming information specific to this
     * <code>StreamObject</code>.</p>
     *
     * <p>
     * The value returned by this method is <code>null</code> if an
     * implementation of <code>StreamObject</code> does not support or require
     * it.</p>
     *
     * @return the name for this <code>StreamObject</code>.
     * @since   JSO-0.6
     */
    public NSI getNSI();
    /**
     * <p>
     * Retrieves the "local" name for this <Code>StreamObject</code>.</p>
     *
     * <p>
     * The value returned by this method is <code>null</code> if an
     * implementation of <code>StreamObject</code> does not support or require
     * it.</p>
     *
     * @return The local-name for this <Code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public String getLocalName();
    /**
     * <p>
     * Retrieves the XML namespace prefix for this <code>StreamObject</code>.
     * This method will recursively search for the namespace prefix through
     * all parents.</p>
     *
     * <p>
     * The value returned by this method is <code>null</code> if an
     * implementation of <code>StreamObject</code> does not support or require
     * it.</p>
     *
     * @return The namespace prefix for this <code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public String getNamespacePrefix();
    /**
     * <p>
     * Retrieves the XML namespace URI for this <code>StreamObject</code>.
     * This method will recursively search for the namespace URI through
     * all parents.</p>
     *
     * <p>
     * This method returns <code>null</code> if a namespace URI cannot be
     * determined for this <code>StreamObject</code>.</p>
     *
     * @return The namespace URI for this <code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public String getNamespaceURI();
    /**
     * <p>
     * Retrieves the "qualified" name for this <Code>StreamObject</code>.
     * The qualified name is the namespace-prefix (if any) and the local
     * name, separated by ":".</p>
     *
     * <p>
     * This method returns <code>null</code> if this
     * <code>StreamObject</code> does not support it.</p>
     *
     * @return The qualified name for this <Code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public String getQualifiedName();
    
    /**
     * <p>
     * Accepts a <code>StreamObjectVisitor</code> for processing.  This is the
     * "accepting" method implementing the <b><i>Visitor</i></b> design
     * pattern.</p>
     *
     * @param v The <code>StreamObjectVisitor</code> to accept.
     * @since   JSO-0.2
     */
    public void accept(StreamObjectVisitor v);
    
    /**
     * <p>
     * Evaluates the given XPath expression, using this <tt>StreamObject</tt>
     * as the context. This evaluation makes no assumptions about namespace
     * mappings.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt>. The actual type of object
     * returned depends on the expression, but is one of the following:</p>
     *
     * <ul>
     * <li><b>java.util.List</b>: For lists of <tt>StreamObject</tt>s</li>
     * <li><b>java.lang.Boolean</b>: For boolean values</li>
     * <li><b>java.lang.Number</b>: For number values</li>
     * <li><b>java.lang.String</b>: For string values</li>
     * </ul>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @return The results of the evaluation.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public Object evaluateXPath(String expr) throws IllegalArgumentException;
    /**
     * <p>
     * Evaluates the given XPath expression, using this <tt>StreamObject</tt>
     * as the context. This evaluation uses the namespace mappings provided
     * by the given Map.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt>. The actual type of object
     * returned depends on the expression, but is one of the following:</p>
     *
     * <ul>
     * <li><b>java.util.List</b>: For node-set (as <tt>StreamObject</tt>)
     * values</li>
     * <li><b>java.lang.Boolean</b>: For boolean values</li>
     * <li><b>java.lang.Number</b>: For number values</li>
     * <li><b>java.lang.String</b>: For string values</li>
     * </ul>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. Each entry in <tt>nss</tt> is assumed to
     * be a prefix/URI namespace pair, where the namespace prefix is the key
     * and the namespace URI is the value. If either is <tt>null</tt>, the pair
     * is ignored.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @param nss The namespace mappings.
     * @return The results of the evaluation.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public Object evaluateXPath(String expr, Map nss) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the first <tt>StreamObject</tt> that matches the given XPath
     * expression, using this <tt>StreamObject</tt> as the context. This
     * evaluation makes no assumptions about namespace mappings.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if no matching nodes were
     * found.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression that
     * evaluates to a node-set, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @return The first matching <tt>StreamObject</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public StreamObject select(String expr) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the first <tt>StreamObject</tt> that matches the given XPath
     * expression, using this <tt>StreamObject</tt> as the context. This
     * evaluation uses the namespace mappings provided by the given Map.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if no matching nodes were
     * found.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression that
     * evaluates to a node-set, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. Each entry in <tt>nss</tt> is assumed to
     * be a prefix/URI namespace pair, where the namespace prefix is the key
     * and the namespace URI is the value. If either is <tt>null</tt>, the pair
     * is ignored.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @param nss The namespace mappings.
     * @return The first matching <tt>StreamObject</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public StreamObject select(String expr, Map nss) throws IllegalArgumentException;
    
    /**
     * <p>
     * Evaluates the given XPath expression into a number, using this
     * <tt>StreamObject</tt> as the context. This evaluation makes no
     * assumptions about the namespace mappings.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if the expression does not
     * evaluate to a Number value.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or an
     * <tt>IllegalArgumentException</tt> is thrown. This value may be modified
     * to perform a numeric evaluation.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @return The numeric results of the evaluation.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.12
     */
    public Number selectNumber(String expr) throws IllegalArgumentException;
    /**
     * <p>
     * Evaluates the given XPath expression into a number, using this
     * <tt>StreamObject</tt> as the context. This evaluation makes no
     * assumptions about the namespace mappings.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if the expression does not
     * evaluate to a Number value.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or an
     * <tt>IllegalArgumentException</tt> is thrown. This value may be modified
     * to perform a numeric evaluation.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. Each entry in <tt>nss</tt> is assumed to
     * be a prefix/URI namespace pair, where the namespace prefix is the key
     * and the namespace URI is the value. If either is <tt>null</tt>, the pair
     * is ignored.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @param nss The namespace mappings.
     * @return The numeric results of the evaluation.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.12
     */
    public Number selectNumber(String expr, Map nss) throws IllegalArgumentException;
    
    /**
     * <p>
     * Evaluates the given XPath expression into a string, using this
     * <tt>StreamObject</tt> as the context. This evaluation makes no
     * assumptions about the namespace mappings.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if the expression does not
     * evaluate to a String value.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or an
     * <tt>IllegalArgumentException</tt> is thrown. This value may be modified
     * to perform a string evalutation.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @return The string results of the evaluation.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.12
     */
    public String selectString(String expr) throws IllegalArgumentException;
    /**
     * <p>
     * Evaluates the given XPath expression into a string, using this
     * <tt>StreamObject</tt> as the context. This evaluation makes no
     * assumptions about the namespace mappings.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if the expression does not
     * evaluate to a String value.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression, or an
     * <tt>IllegalArgumentException</tt> is thrown. This value may be modified
     * to perform a string evaluation.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. Each entry in <tt>nss</tt> is assumed to
     * be a prefix/URI namespace pair, where the namespace prefix is the key
     * and the namespace URI is the value. If either is <tt>null</tt>, the pair
     * is ignored.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @param nss The namespace mappings.
     * @return The string results of the evaluation.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.12
     */
    public String selectString(String expr, Map nss) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the list of <tt>StreamObject</tt> values that match the given
     * XPath expression, using this <tt>StreamObject</tt> as the context. This
     * evaluation makes no assumptions about namespace mappings.</p>
     *
     * <p>
     * The value returned is never <tt>null</tt>. The returned list contains
     * all the <tt>StreamObject</tt>s that match <tt>expr</tt>.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression that
     * evaluates to a node-set, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @return The resulting node-set.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public List selectAll(String expr) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the list of <tt>StreamObject</tt> values that match the given
     * XPath expression, using this <tt>StreamObject</tt> as the context. This
     * evaluation uses the namespace mappings provided by the given Map.</p>
     *
     * <p>
     * The value returned is never <tt>null</tt>. The returned list contains
     * all the <tt>StreamObject</tt>s that match <tt>expr</tt>.</p>
     *
     * <p>
     * The value of <tt>expr</tt> must be a valid XPath expression that
     * evaluates to a node-set, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value of <tt>nss</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty Map. Each entry in <tt>nss</tt> is assumed to
     * be a prefix/URI namespace pair, where the namespace prefix is the key
     * and the namespace URI is the value. If either is <tt>null</tt>, the pair
     * is ignored.</p>
     *
     * @param expr The XPath expression to evaluate.
     * @param nss The namespace mappings.
     * @return The resulting node-set.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public List selectAll(String expr, Map nss) throws IllegalArgumentException;
    
    /**
     * <p>
     * Creates an "almost-exact" duplicate of this <code>StreamObject</code>.
     * This method is expected to behave identically to the method
     * <code>{@link #copy()}</code>.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public Object clone();
    /**
     * <p>
     * Creates an "almost-exact" duplicate of this <code>StreamObject</code>.
     * The parent for this <code>StreamObject</code> is not referenced by the
     * newly created object.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public StreamObject copy();
    /**
     * <p>
     * Creates an "almost-exact" duplicate of this <code>StreamObject</code>,
     * using the given parent <code>StreamElement</code>.  The parent for this
     * <code>StreamObject</code> is one specified by <code>p</code>, rather than
     * that referenced by this <code>StreamObject</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case no
     * parent is attached to the newly created <code>StreamObject</code>.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamObject</code>.
     * @since   JSO-0.2
     */
    public StreamObject copy(StreamElement p);


    /**
     * <p>
     * Detaches this <code>StreamObject</code> from its parent.  Once this
     * method is called, {@link #getParent} returns <code>null</code>.</p>
     *
     * @since   JSO-0.6
     */
    public void detach();
}