/*
 * IOStreamSource.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io.src;


import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.io.StreamSource;

/**
 * <p>
 * Implementation of <code>StreamSource</code> for using arbitrary
 * I/O streams.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class IOStreamSource implements StreamSource {
    //Fields
    private InputStreamQueue    _Input;
    private OutputStream        _Output;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>IOStreamSource</tt> without any I/O.</p>
     *
     * @since   JSO-0.10
     */
    protected IOStreamSource() {
    }
    /**
     * <p>
     * Creates a new <tt>IOStreamSource</tt> connected to the given
     * <tt>InputStream</tt> and <tt>OutputStream</tt>.</p>
     *
     * <p>
     * The value of <tt>in</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>out</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param in The <tt>InputStream</tt> to connect to.
     * @param out The <tt>OutputStream</tt> to connect to.
     * @throws IOException If any I/O error occur.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.2
     */
    public IOStreamSource(InputStream in, OutputStream out) throws IOException, IllegalArgumentException {
        setInput(in);
        setOutput(out);
    }

    //Methods
    public void connect(Stream s) throws IOException, Exception {
        _Input.start();
    }
    public void disconnect(Stream s) throws IOException, Exception {
        _Input.stop();
        _Output.close();
    }
    
    public String getHostname() {
        return "localhost";
    }

    /**
     * <p>
     * Retrieves the <tt>InputStreamQueue</tt> for this <tt>IOStreamSource</tt>.
     * This object is used to read data from a (blocking) <tt>InputStream</tt>
     * within a separate thread.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if this
     * <tt>IOStreamSource</tt> has not been connected.</p>
     *
     * @return The <tt>InputStreamQueue</tt>.
     * @since   JSO-0.12
     */
    protected InputStreamQueue getInputQueue() {
        return _Input;
    }
    /**
     * <p>
     * Sets the <tt>InputStream</tt> for this <tt>IOStreamSource</tt>. This
     * method is intended for subclasses to initialize the input.</p>
     *
     * <p>
     * The value of <tt>in</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param in The new <tt>InputStream</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.10
     */
    protected void setInput(InputStream in) throws IllegalArgumentException {
        if (in == null)
            throw new IllegalArgumentException("InputStream cannot be null");
        _Input = new InputStreamQueue("jso-ioss:" + getHostname(), in, Short.MAX_VALUE / 4);
    }
    /**
     * <p>
     * Sets the <tt>OutputStream</tt> for this <tt>IOStreamSource</tt>. This
     * method is intended for subclasses to initialize the output.</p>
     *
     * <p>
     * The value of <tt>out</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param out The new <tt>OutputStream</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.10
     */
    protected void setOutput(OutputStream out) throws IllegalArgumentException {
        if (out == null)
            throw new IllegalArgumentException("OutputStream cannot be null");
        _Output = out;
    }
    
    public boolean ready() throws IOException {
        return _Input.isValid();
    }
    
    public int read(byte[] buffer, int offset, int length) throws IllegalArgumentException, IOException {
        return _Input.read(buffer, offset, length);
    }
    public int write(byte[] buffer, int offset, int length) throws IllegalArgumentException, IOException {
        _Output.write(buffer, offset, length);
        _Output.flush();
        
        return length;
    }
}