/*
 * SocketChannelSource.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io.src;


import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.ByteChannel;
import java.nio.channels.Selector;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.nio.channels.SelectableChannel;
import java.nio.charset.Charset;

import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.io.StreamSource;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Implementation of <code>StreamSource</code> using <code>ByteChannel</code>
 * instances for the source. The <code>ByteChannel</code> should be a
 * <code>SelectableChannel</code>, and configured for non-blocking operation
 * before {@link #connect} is called.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.8
 */
public class SelectableChannelStreamSource extends ChannelStreamSource {

    //Fields

    //Constructors
    /**
     * <p>
     * Creates a new <code>ChannelStreamSource</code> with the given
     * <code>ByteChannel</code>.  Once created, the underlying
     * <code>ByteChannel</code> is assumed to be ready for {@link #connect} to
     * to be called.</p>
     *
     * <p>
     * The value of <code>channel</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param sc The <code>ByteChannel</code> to connect with.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.8
     */
    public SelectableChannelStreamSource(ByteChannel sc) throws IllegalArgumentException {
        super(sc);
    }
        
    public int write(byte[] buffer, int offset, int length) 
	throws IllegalArgumentException, IOException {
        ByteChannel     channel = getChannel();
        int             amt = 0, idx;
        
        //Validate parameters
        if (buffer == null)
            throw new IllegalArgumentException("buffer cannot be null");
        if ((offset < 0) || (buffer.length < (offset + length)))
            throw new IllegalArgumentException("offset not in valid range");
        if ((length < 0) || (length > (buffer.length - offset)))
            throw new IllegalArgumentException("length not in valid range");
        
	ByteBuffer out = ByteBuffer.wrap(buffer, offset, length);
	return channel.write(out);
    }

}
