/*
 * SASLFailedException.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import org.jabberstudio.jso.JSOImplementation;

/**
 * <p>
 * Exception thrown for most SASL-related failures and errors.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class SASLFailedException extends RuntimeException {
    //Fields
    private SASLFailurePacket   _Result;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>SASLFailedException</tt> with the given
     * <tt>SASLFailurePacket</tt>.</p>
     *
     * <p>
     * The value of <tt>result</tt> should not be <tt>null</tt>, but no effort
     * is made to validate this.</p>
     * 
     * @param result The resulting <tt>SASLFailurePacket</tt>.
     * @since   JSO-0.12
     */
    public SASLFailedException(SASLFailurePacket result) {
        this(null, result);
    }
    /**
     * <p>
     * Creates a new <tt>SASLFailedException</tt> with the given message and
     * <tt>SASLFailurePacket</tt>.</p>
     *
     * <p>
     * The value of <tt>msg</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * <p>
     * The value of <tt>result</tt> may be <tt>null</tt>, in which case a
     * new <tt>SASLFailurePacket</tt> is created with the condition
     * {@link SASLFailurePacket#TEMPORARY_AUTH_FAILURE_CONDITION}.</p>
     * 
     * @param msg The text message for this <tt>SASLException</tt>.
     * @param result The resulting <tt>SASLFailurePacket</tt>.
     * @since   JSO-0.12
     */
    public SASLFailedException(String msg, SASLFailurePacket result) {
        super(msg);
        
        if (result == null) {
            result = (SASLFailurePacket)JSOImplementation.getInstance().getDataFactory().createElementNode(SASLFailurePacket.NAME, SASLFailurePacket.class);
            
            result.setCondition(SASLFailurePacket.TEMPORARY_AUTH_FAILURE_CONDITION);
        }
        _Result = result;
    }

    //Methods
    /**
     * <p>
     * Retrieves the <tt>SASLFailurePacket</tt> that results from the failure.
     * The exact origin of this packet is different for clients and servers.
     * For clients, this is the <tt>SASLFailurePacket</tt> received from the
     * server. For servers, this is generated in response to some error in
     * authentication.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The resulting <tt>SASLFailurePacket</tt>.
     * @since   JSO-0.12
     */
    public SASLPacket getResultPacket() {
        return _Result;
    }
}