/*
 * SASLMechanism.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import java.util.Map;

import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Interface for evaluating SASL challenges/responses for a particular
 * mechanism.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public interface SASLMechanism {
    //Inner-classes
    /**
     * <p>
     * Class to enumerate the possible <tt>SASLMechanism</tt> statuses.</p>
     *
     * @author  linuxwolf
     * @version $VERSION$
     * @since   JSO-0.12
     */
    public static class Status extends Enumerator {
        //Fields
        private final boolean   _Authing;
        private final boolean   _Complete;
        
        //Constructors
        /**
         * <p>
         * Creates a new <tt>Status</tt> with the given name, in-process flag,
         * and complete flag. This is meant for use by <tt>SASLMechanism</tt>
         * implementations for extending the possible status types.</p>
         *
         * @param name The name for this <tt>Status</tt>.
         * @param auth The in-process flag value.
         * @param complete The complete flag value.
         * @since   JSO-0.12
         */
        protected Status(String name, boolean auth, boolean complete) {
            super(name);
            _Authing = auth;
            _Complete = complete;
        }
        
        //Methods
        /**
         * <p>
         * Determines if this <tt>Status</tt> corresponds to an in-process
         * authentication attempt.</p>
         * 
         * @return <tt>true</tt> if this <tt>Status</tt> signals that
         * authentication is still in progress.
         * @since   JSO-0.12
         */
        public final boolean isAuthenticating() {
            return _Authing;
        }
        
        /**
         * <p>
         * Determines if this <tt>Status</tt> corresponds to a completed
         * authentication attempt. It does not indicate whether or not that
         * attempt was successful.</p>
         * 
         * @return <tt>true</tt> if this <tt>Status</tt> signals that
         * authentication is complete.
         * @since   JSO-0.12
         */
        public final boolean isComplete() {
            return _Complete;
        }
    }
    
    //"Constants"
    /** Status to indicate readiness to start authentication.
     *
     * @since   JSO-0.12
     */
    public static final Status      READY           = new Status("ready", false, false);
    /** Status to indicate authentication is in process.
     *
     * @since   JSO-0.12
     */
    public static final Status      AUTHENTICATING  = new Status("in process", true, false);
    /** Status to indicate authentication succeeded.
     *
     * @since   JSO-0.12
     */
    public static final Status      SUCCEEDED       = new Status("succeeded", false, true);
    /** Status to indicate authentication failed.
     *
     * @since   JSO-0.12
     */
    public static final Status      FAILED          = new Status("failed", false, true);
    
    //Methods
    /**
     * <p>
     * Retreives the mechanism name for this <tt>SASLMechanism</tt>. This value
     * matches the well-known mechanism identifier.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * @return The mechanism name.
     * @since   JSO-0.12
     */
    public String getMechanismName();
    /**
     * <p>
     * Retrieves the current status of this <tt>SASLMechanism</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, and is always
     * one of the static final fields for this interface.</p>
     *
     * @return The current status.
     * @since   JSO-0.12
     */
    public Status getCurrentStatus();
    
    /**
     * <p>
     * Retrieves the current properties for this <tt>SASLMechanism</tt>.
     * The returned map should not be modified, and any changes to the map (if
     * not explicitly denied) are not reflected into this
     * <tt>SASLMechanism</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each entry
     * in the returned map is a mapping from a string key to a string value.</p>
     *
     * @return The authentication properties.
     * @since   JSO-0.12
     */
    public Map getProperties();

    /**
     * <p>
     * Evaluates the given challenge/response.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if <tt>input</tt>
     * successfully completes authentication.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt>. If this is a client
     * <tt>SASLMechanism</tt>, a value of <tt>null</tt> means the return value
     * should be the <tt>SASLAuthPacket</tt>. Otherwise, if <tt>input</tt> is
     * not intended as input for this particular <tt>SASLMechanism</tt> (such
     * as providing a response <tt>SASLPacket</tt> to a client
     * <tt>SASLMechanism</tt>), an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param input The input <tt>SASLPacket</tt>.
     * @return The output <tt>SASLPacket</tt>.
     * @throws IllegalArgumentException If <tt>input</tt> is not valid for this
     * <tt>SASLMechanism</tt>.
     * @throws IllegalStateException If <tt>input</tt> is not valid for this
     * <tt>SASLMechanism</tt>'s current status.
     * @throws SASLFailedException If any error occured evaluating the SASL
     * challenge/response.
     */
    public SASLPacket evaluate(SASLPacket input) throws IllegalStateException, SASLFailedException;
    /**
     * <p>
     * Resets this <tt>SASLMechanism</tt> for later reuse. When complete,
     * this <tt>SASLMechanism</tt>'s current status should be {@link #READY}
     * (if this <tt>SASLMechanism</tt> can be reused) or {@link #FAILED}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The current status for this <tt>SASLMechanism</tt>.
     * @since   JSO-0.12
     */
    public Status reset();

    /**
     * <p>
     * Wraps the given bytes using the mechanism's security layer (if any).</p>
     *
     * <p>
     * The value returned by this method is never </tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt>, in which case is is
     * considered to be an array of zero length.</p>
     *
     * @param input The bytes to wrap.
     * @return The wrapped bytes.
     * @throws IllegalStateException If this <tt>SASLMechanism</tt> is not in
     * a state it can wrap.
     * @since   JSO-0.12
     */
    public byte[] wrap(byte[] input) throws IllegalStateException;
    /**
     * <p>
     * Unwraps the given bytes using the mechanism's security layer
     * (if any).</p>
     *
     * <p>
     * The value returned by this method is never </tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt>, in which case is is
     * considered to be an array of zero length.</p>
     *
     * @param input The bytes to unwrap.
     * @return The unwrapped bytes.
     * @throws IllegalStateException If this <tt>SASLMechanism</tt> is not in
     * a state it can unwrap.
     * @since   JSO-0.12
     */
    public byte[] unwrap(byte[] input) throws IllegalStateException;
}