/*
 * SASLServerInfo.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import java.util.Map;
import javax.security.auth.callback.CallbackHandler;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.JSOImplementation;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.util.Utilities;


/**
 * <p>
 * Class that manages the parameters for creating a server
 * <tt>SASLMechanism</tt>.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class SASLServerInfo {
    //Fields
    private String              _Mech;
    private String              _Service;

    private Map                 _Props;
    private CallbackHandler     _CBH;
    private StreamDataFactory   _SDF;

    //Constructors
    /**
     * <p>
     * Creates a new <tt>SASLServerInfo</tt> with default field values.</p>
     *
     * @since   JSO-0.12
     */
    public SASLServerInfo() {
        this(null);
    }
    /**
     * <p>
     * Creates a new <tt>SASLServerInfo</tt> based on the given
     * <tt>Stream</tt>. This construction uses <tt>conn</tt> to determine
     * the <tt>StreamDataFactory</tt> and the server name (using {@link
     * org.jabberstudio.jso.StreamContext#getFrom} from the "outbound"
     * <tt>StreamContext</tt> of <tt>conn</tt>).</p>
     *
     * <p>
     * The value of <tt>conn</tt> may be <tt>null</tt>, in which case
     * all fields are initialized to their default values.</p>
     *
     * @param conn The <tt>Stream</tt> to be based on.
     * @since   JSO-0.12
     */
    public SASLServerInfo(Stream conn) {
        StreamDataFactory   sdf = null;
        JID                 host = null;

        if (conn != null) {
            sdf = conn.getDataFactory();
            host = conn.getOutboundContext().getFrom();
        }

        _Mech = "";
        _Props = new java.util.TreeMap();

        setDataFactory(sdf);
        setServer((host != null) ? host.getDomain() : "");
    }

    //Methods
    /**
     * <p>
     * Retrieves the mechanism name to use for authentication. By default,
     * this method returns "".</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The mechanism name.
     * @since   JSO-0.12
     */
    public String getMechanismName() {
        return _Mech;
    }
    /**
     * <p>
     * Sets the mechanism name to use for authentication.</p>
     *
     * <p>
     * The value of <tt>mech</tt> may be <tt>null</tt> or "", in which case
     * it is considered to be "".</p>
     *
     * @param mech The new mechanism name.
     * @since JSO-0.12
     */
    public void setMechanismName(String mech) {
        _Mech = ((mech != null) ? mech : "");
    }

    /**
     * <p>
     * Retrieves the server name to authenticate to. By default, this
     * method returns "".</p>
     *
     * <p>
     * The value returend by this method is never <tt>null</tt>.</p>
     *
     * @return The authentication server name.
     * @since   JSO-0.12
     */
    public String getServer() {
        return _Service;
    }
    /**
     * <p>
     * Sets the server name to authenticate to.</p>
     *
     * <p>
     * The value of <tt>srv</tt> may be <tt>null</tt> or "", in which case
     * it is considered to be "".</p>
     *
     * @param srv The new authentication server name.
     * @since   JSO-0.12
     */
    public void setServer(String srv) {
        _Service = ((srv != null) ? srv : "");
    }

    /**
     * <p>
     * Retrieves the SASL properties. By default, all known properties
     * are included in the map, and initialized to {@link
     * SASLMechanismManager#DEFAULT_PROPERTIES}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each
     * element in the returned map is a mapping of a string key to a
     * string value.</p>
     *
     * @return The properties to authenticate by.
     * @since   JSO-0.12
     */
    public Map getProperties() {
        return _Props;
    }
    /**
     * <p>
     * Sets the SASL properties. This method first clears an internal
     * map, puts all the entries in {@link
     * SASLMechanismManager#DEFAULT_PROPERTIES}, then puts all entries
     * contained in <tt>props</tt>.</p>
     *
     * <p>
     * The value of <tt>props</tt> may be <tt>null</tt> in which case 
     *
     * @param props The properties to authenticate by.
     * @since   JSO-0.12
     */
    public void setProperties(Map props) {
        _Props.clear();
        _Props.putAll(SASLMechanismManager.DEFAULT_PROPERTIES);
        if (props != null)
            _Props.putAll(props);
    }

    /**
     * <p>
     * Retrieves the <tt>CallbackHandler</tt> to use for authentication.
     * By default, this method returns <tt>null</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The authentication <tt>CallbackHandler</tt>.
     * @since   JSO-0.12
     */
    public CallbackHandler getCallbackHandler() {
        return _CBH;
    }
    /**
     * <p>
     * Sets the <tt>CallbackHandler</tt> to use for authentication.</p>
     *
     * <p>
     * The value of <tt>cbh</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param cbh The new authentication <tt>CallbackHandler</tt>.
     * @since   JSO-0.12
     */
    public void setCallbackHandler(CallbackHandler cbh) {
        _CBH = cbh;
    }

    /**
     * <p>
     * Retrieves the <tt>StreamDataFactory</tt> to use. By default, this
     * value is the instance obtained using {@link
     * JSOImplementation#getDataFactory} from the "global" instance of
     * <tt>JSOImplementation</tt> (e.g. {@link
     * JSOImplementation#getInstance})</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamDataFactory</tt> to use.
     * @since   JSO-0.12
     */
    public StreamDataFactory getDataFactory() {
        if (_SDF == null)
            _SDF = JSOImplementation.getInstance().getDataFactory();
        return _SDF;
    }
    /**
     * <p>
     * Sets the <tt>StreamDataFactory</tt> to use.</p>
     *
     * <p>
     * The value of <tt>sdf</tt> may be <tt>null</tt>, in which case the
     * default <tt>StreamDataFactory</tt> will be returned by {@link
     * #getDataFactory}.</p>
     *
     * @param sdf The new <tt>StreamDataFactory</tt>.
     * @since   JSO-0.12
     */
    public void setDataFactory(StreamDataFactory sdf) {
        _SDF = sdf;
    }

    /**
     * <p>
     * Determines if this <tt>SASLServerInfo</tt> contains enough
     * information to create a server <tt>SASLMechanism</tt>.</p>
     *
     * @return <tt>true</tt> if there is a valid mechanism name and if
     * there is a valid server name
     * @since   JSO-0.12
     */
    public boolean isValid() {
        return (Utilities.isValidString(getMechanismName()) && Utilities.isValidString(getServer()));
    }
}