/*
 * ContextDecorator.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.nio.charset.Charset;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.StreamObjectVisitor;
import org.jabberstudio.jso.StreamText;
import org.jabberstudio.jso.io.StreamBuilder;

/**
 * <p>
 * Class that implements the "Decorator" pattern over the <tt>StreamContext</tt>
 * interface.</p>
 *
 * <p>
 * Subclasses must implement {@link #getDecoratedContext}. All other methods
 * are implemented by calling the appropriate method on the
 * <tt>StreamContext</tt> returned by the above method.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.10
 */
public abstract class ContextDecorator implements StreamContext {
    //Methods
    /**
     * <p>
     * Retrieves the decorated <tt>StreamContext</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The decorated <tt>StreamContext</tt>.
     * @since   JSO-0.10
     */
    protected abstract StreamContext getDecoratedContext();
    
    public StreamElement getParent() {
        return getDecoratedContext().getParent();
    }
    public void setParent(StreamElement p) throws IllegalStateException {
        getDecoratedContext().setParent(p);
    }
    
    public StreamContext getContext() {
        return getDecoratedContext().getContext();
    }
    
    public PacketRouter getRouter() {
        return getDecoratedContext().getRouter();
    }
    
    public Stream.Status getCurrentStatus() {
        return getDecoratedContext().getCurrentStatus();
    }
    
    public String getCharsetName() {
        return getDecoratedContext().getCharsetName();
    }
    public void setCharsetName(String name) throws IllegalArgumentException {
        getDecoratedContext().setCharsetName(name);
    }
    public Charset obtainCharset() {
        return getDecoratedContext().obtainCharset();
    }

    public StreamDataFactory getDataFactory() {
        return getDecoratedContext().getDataFactory();
    }
    
    public boolean isInbound() {
        return getDecoratedContext().isInbound();
    }
    public boolean isOutbound() {
        return getDecoratedContext().isOutbound();
    }

    public String getID() {
        return getDecoratedContext().getID();
    }
    public void setID(String id) {
        getDecoratedContext().setID(id);
    }

    public String getDeclaredLanguage() {
        return getDecoratedContext().getDeclaredLanguage();
    }
    public void setDeclaredLanguage(String lang) throws UnsupportedOperationException {
        getDecoratedContext().setDeclaredLanguage(lang);
    }
    public String getLanguage() {
        return getDecoratedContext().getLanguage();
    }
    
    public Locale getDeclaredLocale() {
        return getDecoratedContext().getDeclaredLocale();
    }
    public void setDeclaredLocale(Locale lang) throws UnsupportedOperationException {
        getDecoratedContext().setDeclaredLocale(lang);
    }
    public Locale getLocale() {
        return getDecoratedContext().getLocale();
    }

    public JID getFrom() {
        return getDecoratedContext().getFrom();
    }
    public void setFrom(JID from) {
        getDecoratedContext().setFrom(from);
    }
    
    public JID getTo() {
        return getDecoratedContext().getTo();
    }
    public void setTo(JID to) {
        getDecoratedContext().setTo(to);
    }
    
    public String getVersion() {
        return getDecoratedContext().getVersion();
    }
    public void setVersion(String ver) {
        getDecoratedContext().setVersion(ver);
    }
    
    public NSI getNSI() {
        return getDecoratedContext().getNSI();
    }

    public String getLocalName() {
        return getDecoratedContext().getLocalName();
    }
    public String getNamespacePrefix() {
        return getDecoratedContext().getNamespacePrefix();
    }
    public String getNamespaceURI() {
        return getDecoratedContext().getNamespaceURI();
    }
    public String getQualifiedName() {
        return getDecoratedContext().getQualifiedName();
    }
    
    public Set getDeclaredNamespaces() {
        return getDecoratedContext().getDeclaredNamespaces();
    }
    public StreamNamespace getNamespaceByURI(String uri) {
        return getDecoratedContext().getNamespaceByURI(uri);
    }
    public StreamNamespace getNamespaceByPrefix(String prefix) {
        return getDecoratedContext().getNamespaceByPrefix(prefix);
    }
    public String getNamespacePrefix(String uri) throws IllegalArgumentException {
        return getDecoratedContext().getNamespacePrefix(uri);
    }
    public String getNamespaceURI(String prefix) throws IllegalArgumentException {
        return getDecoratedContext().getNamespaceURI(prefix);
    }
    public void addNamespace(String prefix, String uri) throws UnsupportedOperationException {
        getDecoratedContext().addNamespace(prefix, uri);
    }
    public void removeNamespace(String prefix, String uri) throws UnsupportedOperationException {
        getDecoratedContext().removeNamespace(prefix, uri);
    }

    public Map getAttributes() {
        return getDecoratedContext().getAttributes();
    }
    public String getAttributeValue(NSI name) throws IllegalArgumentException {
        return getDecoratedContext().getAttributeValue(name);
    }
    public String getAttributeValue(String name) throws IllegalArgumentException {
        return getDecoratedContext().getAttributeValue(name);
    }
    public String getAttributeValue(String name, String ns) throws IllegalArgumentException {
        return getDecoratedContext().getAttributeValue(name, ns);
    }
    
    public void setAttributeValue(String name, String val) throws IllegalArgumentException, UnsupportedOperationException {
        getDecoratedContext().setAttributeValue(name, val);
    }
    public void setAttributeValue(NSI name, String val) throws IllegalArgumentException, UnsupportedOperationException {
        getDecoratedContext().setAttributeValue(name, val);
    }
    public void setAttributeValue(String name, String ns, String val) throws IllegalArgumentException, UnsupportedOperationException {
        getDecoratedContext().setAttributeValue(name, ns, val);
    }
    
    public List listNodes() {
        return getDecoratedContext().listNodes();
    }

    public List listElements() {
        return getDecoratedContext().listElements();
    }
    public List listElements(Class iface) {
        return getDecoratedContext().listElements(iface);
    }
    public List listElements(NSI name) {
        return getDecoratedContext().listElements(name);
    }
    public List listElements(NSI name, Class iface) {
        return getDecoratedContext().listElements(name, iface);
    }
    public List listElements(String name) {
        return getDecoratedContext().listElements(name);
    }
    public List listElements(String name, Class iface) {
        return getDecoratedContext().listElements(name, iface);
    }
    public List listElements(String name, String ns) {
        return getDecoratedContext().listElements(name, ns);
    }
    public List listElements(String name, String ns, Class iface) {
        return getDecoratedContext().listElements(name, ns, iface);
    }
    
    public StreamElement getFirstElement() {
        return getDecoratedContext().getFirstElement();
    }
    public StreamElement getFirstElement(Class iface) {
        return getDecoratedContext().getFirstElement(iface);
    }
    public StreamElement getFirstElement(String name) {
        return getDecoratedContext().getFirstElement(name);
    }
    public StreamElement getFirstElement(String name, Class iface) {
        return getDecoratedContext().getFirstElement(name, iface);
    }
    public StreamElement getFirstElement(String name, String ns) {
        return getDecoratedContext().getFirstElement(name, ns);
    }
    public StreamElement getFirstElement(String name, String ns, Class iface) {
        return getDecoratedContext().getFirstElement(name, ns, iface);
    }
    public StreamElement getFirstElement(NSI name) {
        return getDecoratedContext().getFirstElement(name);
    }
    public StreamElement getFirstElement(NSI name, Class iface) {
        return getDecoratedContext().getFirstElement(name, iface);
    }
    
    public StreamElement addElement(String name) {
        return getDecoratedContext().addElement(name);
    }
    public StreamElement addElement(String name, Class iface) {
        return getDecoratedContext().addElement(name, iface);
    }
    public StreamElement addElement(String name, String uri) {
        return getDecoratedContext().addElement(name, uri);
    }
    public StreamElement addElement(String name, String uri, Class iface) {
        return getDecoratedContext().addElement(name, uri, iface);
    }
    public StreamElement addElement(NSI name) {
        return getDecoratedContext().addElement(name);
    }
    public StreamElement addElement(NSI name, Class iface) {
        return getDecoratedContext().addElement(name, iface);
    }
    
    public void clearElements() {
        getDecoratedContext().clearElements();
    }
    public void clearElements(Class iface) {
        getDecoratedContext().clearElements(iface);
    }
    public void clearElements(String name) {
        getDecoratedContext().clearElements(name);
    }
    public void clearElements(String name, Class iface) {
        getDecoratedContext().clearElements(name, iface);
    }
    public void clearElements(String name, String uri) {
        getDecoratedContext().clearElements(name, uri);
    }
    public void clearElements(String name, String uri, Class iface) {
        getDecoratedContext().clearElements(name, uri, iface);
    }
    public void clearElements(NSI name) {
        getDecoratedContext().clearElements(name);
    }
    public void clearElements(NSI name, Class iface) {
        getDecoratedContext().clearElements(name, iface);
    }

    public List listText() {
        return getDecoratedContext().listText();
    }

    public String normalizeText() {
        return getDecoratedContext().normalizeText();
    }
    public String normalizeTrimText() {
        return getDecoratedContext().normalizeTrimText();
    }
    
    public StreamText addText(String txt) {
        return getDecoratedContext().addText(txt);
    }
    
    public void clearText() {
        getDecoratedContext().clearText();
    }
    
    public int indexOf(StreamNode node) {
        return getDecoratedContext().indexOf(node);
    }
    public void add(StreamNode node) {
        getDecoratedContext().add(node);
    }
    public void insert(int idx, StreamNode node) throws IllegalArgumentException {
        getDecoratedContext().insert(idx, node);
    }
    public void remove(StreamNode node) {
        getDecoratedContext().remove(node);
    }
    public void clear() {
        getDecoratedContext().clear();
    }
    
    public void accept(StreamObjectVisitor v) {
        getDecoratedContext().accept(v);
    }
    
    public Object evaluateXPath(String expr) throws IllegalArgumentException {
        return getDecoratedContext().evaluateXPath(expr);
    }
    public Object evaluateXPath(String expr, Map nss) throws IllegalArgumentException {
        return getDecoratedContext().evaluateXPath(expr, nss);
    }
    
    public StreamObject select(String expr) throws IllegalArgumentException {
        return getDecoratedContext().select(expr);
    }
    public StreamObject select(String expr, Map nss) throws IllegalArgumentException {
        return getDecoratedContext().select(expr, nss);
    }
    
    public Number selectNumber(String expr) throws IllegalArgumentException {
        return getDecoratedContext().selectNumber(expr);
    }
    public Number selectNumber(String expr, Map nss) throws IllegalArgumentException {
        return getDecoratedContext().selectNumber(expr, nss);
    }
    
    public String selectString(String expr) throws IllegalArgumentException {
        return getDecoratedContext().selectString(expr);
    }
    public String selectString(String expr, Map nss) throws IllegalArgumentException {
        return getDecoratedContext().selectString(expr, nss);
    }
    
    public List selectAll(String expr) throws IllegalArgumentException {
        return getDecoratedContext().selectAll(expr);
    }
    public List selectAll(String expr, Map nss) throws IllegalArgumentException {
        return getDecoratedContext().selectAll(expr, nss);
    }
    
    public void detach() {
        getDecoratedContext().detach();
    }
    
    public StreamBuilder createBuilder() {
        return getDecoratedContext().createBuilder();
    }
    
    public void reset() {
        getDecoratedContext().reset();
    }
    public void reset(NSI name) throws IllegalArgumentException {
        getDecoratedContext().reset(name);
    }
    public void reset(StreamElement base) throws IllegalArgumentException {
        getDecoratedContext().reset(base);
    }
    public void reset(NSI name, StreamElement base) throws IllegalArgumentException {
        getDecoratedContext().reset(name, base);
    }
    public void reset(String xml) throws IllegalArgumentException {
        getDecoratedContext().reset(xml);
    }

    public Object clone() {
        return copy(null);
    }
    public StreamObject copy() {
        return copy(null);
    }
    public StreamObject copy(StreamElement p) {
        return getDecoratedContext().copy(p);
    }
    
    public String toString() {
        return getDecoratedContext().toString();
    }
}