/*
 * Monitor.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 * 
 * This file was modified by Sun Microsystems on Jnue 24th 2005.  The modifications
 * are:
 * 1. Creates the Timer as a daemon, so that the
 *    application can exit when it has done all of its work.
 * 2. Avoids the race condition between send() and timer
 *    scheduling.
 * 
 */

package org.jabberstudio.jso.util;


import java.util.Timer;
import java.util.TimerTask;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamError;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.event.MonitorEvent;
import org.jabberstudio.jso.event.MonitorListener;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.event.PacketListener;

/**
 * <p>
 * Abstract class for monitoring a packet router/dispatcher pair.</p>
 *
 * @author  linuxwolf
 */
public abstract class Monitor {
    //Inner-classes
    /**
     * <p>
     * Abstract class for performing a specific monitoring session. Instances
     * derived from this class are used for both synchronous and
     * asynchronous modes of monitoring.</p>
     *
     * @author  linuxwolf
     * @version $VERSION$
     * @since   JSO-0.12
     */
    protected abstract class MonitorTask extends TimerTask implements PacketListener {
        //Fields
        private boolean             _Proccing;
        private PacketRouter        _Router;
        private PacketDispatcher    _Dispatch;
        
        private long                _Start, _Timeout;
        private boolean             _Running;
        
        private MonitorEvent        _Event;
        private MonitorListener     _Listener;
        
        //Constructors
        /**
         * <p>
         * Creates a new <tt>MonitorTask</tt> with the given listener.</p>
         *
         * <p>
         * The value of <tt>l</tt> may be <tt>null</tt>.</p>
         *
         * @param l The <tt>MonitorListener</tt> for events.
         * @since   JSO-0.12
         */
        public MonitorTask(MonitorListener l) {
            _Proccing   = Monitor.this.isProcessRouter();
            _Router     = Monitor.this.getRouter();
            _Dispatch   = Monitor.this.getDispatcher();
            _Timeout    = Monitor.this.getTimeout();

            getDispatcher().addPacketListener(PacketEvent.RECEIVED, this);
            
            _Listener   = l;
            _Start      = System.currentTimeMillis();
            _Running    = true;
        }
        
        //Methods
        /**
         * <p>
         * Retrieves the flag determining if this <tt>PacketRouter</tt> calls
         * {@link PacketRouter#process} at each process interval.</p>
         *
         * @return <tt>true</tt> if <tt>PacketRouter.process()</tt> is called,
         * and <tt>false</tt> otherwise.
         * @since   JSO-0.12
         */
        public final boolean isProcessRouter() {
            return _Proccing;
        }
        
        /**
         * <p>
         * Retrieves the <tt>PacketRouter</tt> for this <tt>MonitorTask</tt>.
         * This value is used to send and process <tt>Packet</tt>s.</p>
         *
         * <p>
         * The value returned by this method is nevner <tt>null</tt>.</p>
         *
         * @return The <tt>PacketRouter</tt>.
         * @since   JSO-0.12
         */
        public final PacketRouter getRouter() {
            return _Router;
        }
        /**
         * <p>
         * Retrieves the <tt>PacketDispatcher</tt> for this
         * <tt>MonitorTask</tt>. This value is where the internal
         * <tt>PacketListener</tt> is registered and unregistered.</p>
         *
         * <p>
         * The value returned by this method is never <tt>null</tt>.</p>
         *
         * @return The <tt>PacketDispatcher</tt> to monitor.
         * @since   JSO-0.12
         */
        public final PacketDispatcher getDispatcher() {
            return _Dispatch;
        }
        /**
         * <p>
         * Retrieves the starting time for this <tt>MonitorTask</tt>. This
         * value is used for deteriming the timeout event.</p>
         *
         * @return The starting time, in milliseconds.
         * @since   JSO-0.12
         */
        public final long getStartingTime() {
            return _Start;
        }
        /**
         * <p>
         * Retrieves the timeout for this <tt>MonitorTask</tt>. This value is
         * the number of milliseconds to wait before triggering a
         * "monitorTimeout" event. A negative value indicates the
         * <tt>MonitorTask</tt> never times out.</p>
         *
         * <p>
         * The value returned by this method is never less than -1.</p>
         *
         * @return The timeout, in milliseconds.
         * @since   JSO-0.12
         */
        public final long getTimeout() {
            return _Timeout;
        }
        /**
         * <p>
         * Retrieves the <tt>MonitorEvent</tt> generated by this
         * <tt>MonitorTask</tt>.</p>
         *
         * <p>
         * The value returned by this method may be <tt>null</tt>, if it has
         * not yet been generated.</p>
         *
         * @return The generated <tt>MonitorEvent</tt>.
         * @since   JSO-0.12
         */
        public final MonitorEvent getEvent() {
            return _Event;
        }
        /**
         * <p>
         * Retrieves the <tt>MonitorListener</tt> to fire events on.</p>
         *
         * <p>
         * The value returned by this method may be <tt>null</tt>, if it was
         * not set.</p>
         *
         * @return The <tt>MonitorListener</tt> to fire on.
         * @since   JSO-0.12
         */
        public final MonitorListener getListener() {
            return _Listener;
        }

        /**
         * <p>
         * Determines if this <tt>MonitorTask</tt> is still running.</p>
         *
         * @return <tt>true</tt> if this <tt>MonitorTask</tt> is running.</p>
         */
        public final boolean isRunning() {
            return _Running;
        }
        /**
         * <p>
         * Stops this <tt>MonitorTask</tt>. This method ensures {@link
         * #isRunning} returns <tt>false</tt>, that {@link #getEvent} does
         * not return <tt>null</tt>, and that the internal
         * <tt>PacketListener</tt> is unregistered.</p>
         *
         * <p>
         * The value of <tt>evt</tt> cannot be <tt>null</tt>, or an
         * <tt>IllegalArgumentException</tt> is thrown.</p>
         *
         * @param evt The <tt>MonitorEvent</tt> generated.
         * @throws IllegalArgumentException If the parameter is invalid.
         * @since   JSO-0.12
         */
        protected synchronized final void stop(MonitorEvent evt) throws IllegalArgumentException {
            if (evt == null)
                throw new IllegalArgumentException("MonitorEvent cannot be null");
            
            _Running = false;
            _Event = evt;
            
            cancel();
            getDispatcher().removePacketListener(PacketEvent.RECEIVED, this);
        }

        /**
         * <p>
         * Sends the given <tt>Packet</tt> using the known
         * <tt>PacketRouter</tt>. This method may call {@link
         * #fireMonitorFailed} if sending fails.</p>
         *
         * <p>
         * The value of <tt>p</tt> may be <tt>null</tt>.</p>
         *
         * @param p The <tt>Packet</tt> to send.</p>
         * @since   JSO-0.12
         */
        protected void send(Packet p) {
            try {
                getRouter().send(p);
            } catch (StreamException se) {
                try {
                    obtainLogger().error("could not send monitor packet", se);
                    fireMonitorFailed(se);
                } catch (Throwable t) {
                    obtainLogger().warn("error when firing monitor failed event", t);
                }
            }
        }
        
        /**
         * <p>
         * Method called when a <tt>Packet</tt> is transferred. This method
         * matches the transferred <tt>Packet</tt> using {@link #matched}, and
         * calls {@link #fireMonitorFound} if successful.</p>
         *
         * <p>
         * The value of <tt>evt</tt> should not be <tt>null</tt>.</p>
         *
         * @param evt The <tt>PacketEvent</tt>.
         * @since   JSO-0.12
         */
        public void packetTransferred(PacketEvent evt) {
            Logger      log = obtainLogger();
            Packet      input = evt.getData();

            if (log.isDebugEnabled())
                log.debug("attempting to match packet " + evt.getData());
            if (matched(input)) {
                try {
                    log.info("expected packet found");
                    fireMonitorFound(input);
                } catch (Throwable t) {
                    obtainLogger().warn("error when firing monitor found event", t);
                }
            }
        }

        /**
         * <p>
         * Executes the current process/match routine. This method is typically
         * called by the <tt>Timer</tt> to perform the monitor evaluation.</p>
         *
         * <p>
         * This method may call a {@link #fireMonitorTimeout} if the elapsed
         * time exceeds {@link #getTimeout}, or {@link #fireMonitorFailed} if
         * the registered <tt>Packetrouter</tt> could not be processed.</p>
         *
         * @since   JSO-0.12
         */
        public void run() {
            Logger              log = obtainLogger();
            PacketRouter        router = getRouter();
            long                start = getStartingTime(), timeout = getTimeout(), current;
            boolean             valid = true;
            
            try {
                //Make sure we're live
                if (!router.getInboundContext().getCurrentStatus().isConnected()) {
                    try {
                        fireMonitorFailed(new StreamException(router.getDataFactory().createStreamError(StreamError.REMOTE_CONNECTION_FAILED_CONDITION)));
                    } catch (Throwable t) {
                        log.warn("error when firing monitor failed event", t);
                    }
                }
                
                //Make sure we're processing here
                if (isProcessRouter())
                    router.process();
                
                //Make sure we're not timed out
                if (isRunning() && (timeout > -1) && ((System.currentTimeMillis() - start) > timeout)) {
                    //Let's try one more time
                    if (isProcessRouter())
                        router.process();
                    valid = !isRunning();
                }

                if (!valid) {
                    try {
                        log.info("monitor timed out");
                        fireMonitorTimeout();
                    } catch (Throwable t) {
                        log.warn("error when firing monitor timeout event", t);
                    }
                }
            } catch (StreamException se) {
                try {
                    log.error("monitor failed to process router");
                    fireMonitorFailed(se);
                } catch (Throwable t) {
                    log.warn("error when firing monitor failed event", t);
                }
            }
        }
        
        /**
         * <p>
         * Called to match the given input <tt>Packet</tt> against the
         * matching criteria.</p>
         *
         * <p>
         * The value of <tt>input</tt> should not be <tt>null</tt>.</p>
         *
         * @param input The <tt>Packet</tt> to match.
         * @return <tt>true</tt> if <tt>input</tt> meets the matching criteria.
         * @since   JSO-0.12
         */
        protected abstract boolean matched(Packet input);

        /**
         * <p>
         * Called when this <tt>MonitorTask</tt> finds a matching
         * <tt>Packet</tt>.</p>
         *
         * <p>
         * The value of <tt>found</tt> should not be <tt>null</tt>.</p>
         *
         * @param found The matched <tt>Packet</tt>.
         * @since   JSO-0.12
         */
        protected abstract void fireMonitorFound(Packet found);
        /**
         * <p>
         * Called when this <tt>MonitorTask</tt> has timed out.</p>
         *
         * @since   JSO-0.12
         */
        protected abstract void fireMonitorTimeout();
        /**
         * <p>
         * Called when this <tt>MonitorTask</tt> has failed to operate on the
         * <tt>PacketRouter</tt>.</p>
         *
         * <p>
         * The value of <tt>cause</tt> should not be <tt>null</tt>.</p>
         *
         * @param cause The <tt>StreamException</tt> causing the failure.
         * @since   JSO-0.12
         */
        protected abstract void fireMonitorFailed(StreamException cause);
    }
    
    //"Constants"
    /**
     * The default timeout value.
     *
     * @since   JSO-0.12
     */
    public static final int     DEFAULT_TIMEOUT = -1;
    /**
     * The default interval between calls to {@link PacketRouter#process}.
     *
     * @since   JSO-0.12
     */
    public static final int     DEFAULT_PROC_INTERVAL = 10;
    
    //Fields
    private PacketRouter        _Router;
    private PacketDispatcher    _Dispatch;
    private Timer               _Timer;
    
    private boolean             _ProcRouter = true;
    private long                _Timeout = DEFAULT_TIMEOUT;
    private long                _ProcInt = DEFAULT_PROC_INTERVAL;
    
    private boolean             _Running;
    
    private static Timer        _DefaultTimer;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>Monitor</tt> with all fields set to default
     * values.</p>
     *
     * @since   JSO-0.12
     */
    public Monitor() {
    }

    //Methods
    /**
     * <p>
     * Obtains the logger for this <tt>Monitor</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The logger for this <tt>Monitor</tt>.
     * @since   JSO-0.12
     */
    protected final Logger obtainLogger() {
        return Logger.getLogger(getClass());
    }

    /**
     * <p>
     * Retrieves the default <tt>Timer</tt> instance used by this
     * <tt>Monitor</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The default <tt>Timer</tt> instance.
     * @since   JSO-0.12
     */
    protected final static synchronized Timer getDefaultTimer() {
        if (_DefaultTimer == null)
            _DefaultTimer = new Timer(true);
        return _DefaultTimer;
    }

    /**
     * <p>
     * Retrieves the <tt>PacketRouter</tt> for this <tt>Monitor</tt>.
     * This value is used to send and process <tt>Packet</tt>s.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>, if it has not
     * yet been set.</p>
     *
     * @return The <tt>PacketRouter</tt>.
     * @since   JSO-0.12
     */
    public final synchronized PacketRouter getRouter() {
        return _Router;
    }
    /**
     * <p>
     * Sets the <tt>PacketRouter</tt> for this <tt>Monitor</tt>.</p>
     *
     * <p>
     * Note that any changes to this value only apply to subsequent calls to
     * {@link #send} and {@link #sendAndWatch} (and their variations). Any
     * currently-executing monitors will not be affected.</p>
     *
     * <p>
     * The value of <tt>r</tt> may be <tt>null</tt>. However, this will cause
     * {@link #send} and {@link #sendAndWatch} to throw
     * <tt>IllegalStateException</tt>s.</p>
     *
     * @param r The new <tt>PacketRouter</tt>.
     * @since   JSO-0.12
     */
    public final synchronized void setRouter(PacketRouter r) {
        _Router = r;
    }
    
    /**
     * <p>
     * Retrieves the <tt>PacketDispatcher</tt> for this <tt>Monitor</tt>.
     * This value is where the internal <tt>PacketListener</tt> is registered
     * and unregistered.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>, if it has not
     * yet been set.</p>
     *
     * @return The <tt>PacketDispatcher</tt> to monitor.
     * @since   JSO-0.12
     */
    public final synchronized PacketDispatcher getDispatcher() {
        return _Dispatch;
    }
    /**
     * <p>
     * Sets the <tt>PacketDispatcher</tt> for this <tt>Monitor</tt>.</p>
     *
     * <p>
     * Note that any changes to this value only apply to subsequent calls to
     * {@link #send} and {@link #sendAndWatch} (and their variations). Any
     * currently-executing monitors will not be affected.</p>
     *
     * <p>
     * The value of <tt>d</tt> may be <tt>null</tt>. However, this will cause
     * {@link #send} and {@link #sendAndWatch} to throw
     * <tt>IllegalStateException</tt>s.</p>
     *
     * @param d The new <tt>PacketDispatcher</tt>.
     * @since   JSO-0.12
     */
    public final synchronized void setDispatcher(PacketDispatcher d) {
        _Dispatch = d;
    }
    
    /**
     * <p>
     * Retrieves the <tt>Timer</tt> for this <tt>Monitor</tt>. This value
     * is used for calls to {@link #send}, to monitor in the background.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>, if it has not
     * yet been set.</p>
     *
     * @return The <tt>Timer</tt> to monitor with.
     * @since   JSO-0.12
     */
    public final synchronized Timer getTimer() {
        return _Timer;
    }
    /**
     * <p>
     * Sets the <tt>PacketRouter</tt> for this <tt>Monitor</tt>.</p>
     *
     * <p>
     * Note that any changes to this value only apply to subsequent calls to
     * {@link #send} and {@link #sendAndWatch} (and their variations). Any
     * currently-executing monitors will not be affected.</p>
     *
     * <p>
     * The value of <tt>t</tt> may be <tt>null</tt>, in which case in internal
     * shared <tt>Timer</tt> will be used.</p>
     *
     * @param t The new <tt>Timer</tt>.
     * @since   JSO-0.12
     */
    public final synchronized void setTimer(Timer t) {
        _Timer = t;
    }
    
    /**
     * <p>
     * Retrieves the timeout for this <tt>Monitor</tt>. This value is the
     * number of milliseconds to wait before triggering a "monitorTimeout"
     * event. A negative value indicates the <tt>Monitor</tt> never
     * times out.</p>
     *
     * <p>
     * The value returned by this method is never less than -1.</p>
     *
     * @return The timeout, in milliseconds.
     * @since   JSO-0.12
     */
    public final synchronized long getTimeout() {
        return _Timeout;
    }
    /**
     * <p>
     * Sets the timeout for this <tt>Monitor</tt>.</p>
     *
     * <p>
     * Note that any changes to this value only apply to subsequent calls to
     * {@link #send} and {@link #sendAndWatch} (and their variations). Any
     * currently-executing monitors will not be affected.</p>
     *
     * <p>
     * If the value of <tt>t</tt> is less than 0, this <tt>Monitor</tt>
     * never time out.</p>
     *
     * @param t The timeout interval, in milliseconds.
     * @since   JSO-0.12
     */
    public final synchronized void setTimeout(long t) {
        _Timeout = (t < -1) ? DEFAULT_TIMEOUT : t;
    }

    /**
     * <p>
     * Retrieves the flag determining if this <tt>Monitor</tt> processes the
     * <tt>PacketRouter</tt> at each process interval. If <tt>true</tt>, each
     * time the process interval elapses, this <tt>Monitor</tt> will call
     * {@link PacketRouter#process}.</p>
     *
     * @return <tt>true</tt> if <tt>PacketRouter.process()</tt> should be
     * called at each process interval, and <tt>false</tt> otherwise.</p>
     * @since   JSO-0.12
     */
    public final synchronized boolean isProcessRouter() {
        return _ProcRouter;
    }
    /**
     * <p>
     * Sets the flag determining if this <tt>Monitor</tt> processes the
     * <tt>PacketRouter</tt> at each process interval.</p>
     *
     * @param proc The new flag value.
     * @since   JSO-0.12
     */
    public final synchronized void setProcessRouter(boolean proc) {
        _ProcRouter = proc;
    }

    /**
     * <p>
     * Retrieves the process interval for this <tt>Monitor</tt>. This
     * value is the number of milliseconds to wait between calls to {@link
     * PacketRouter#process}.</p>
     *
     * <p>
     * The value returned by this method is never less than 0.</p>
     *
     * @return The process interval, in milliseconds.
     * @since   JSO-0.12
     */
    public final synchronized long getProcessInterval() {
        return _ProcInt;
    }
    /**
     * <p>
     * Sets the process interval for this <tt>Monitor</tt>.</p>
     *
     * <p>
     * Note that any changes to this value only apply to subsequent calls to
     * {@link #send} and {@link #sendAndWatch} (and their variations). Any
     * currently-executing monitors will not be affected.</p>
     *
     * <p>
     * If the value of <tt>i</tt> is less than 1, the default interval {@link
     * #DEFAULT_PROC_INTERVAL} is used.</p>
     *
     * @param i The process interval, in milliseconds.
     * @since   JSO-0.12
     */
    public final synchronized void setProcessInterval(long i) {
        _ProcInt = (i < 1) ? DEFAULT_PROC_INTERVAL : i;
    }

    /**
     * <p>
     * Sends the given <tt>Packet</tt>, and waits in the background for its
     * matching result. This method adds a <tt>TimerTask</tt> to the configured
     * <tt>Timer</tt>, which may call {@link PacketRouter#process} until one of
     * the following events occur:
     * <ul>
     * <li>The <tt>Packet</tt> matching the expected criteria is found.</li>
     * <li>The timeout period is exceeded.</li>
     * <li>There is a failure in the <tt>PacketRouter</tt>.</li>
     * </ul></p>
     *
     * <p>
     * The value of <tt>p</tt> may not be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>l</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the value returned by {@link #getDispatcher} is <tt>null</tt>, an
     * <tt>IllegalStateException</tt> is thrown.</p>
     *
     * <p>
     * If the value returned by {@link #getRouter} is <tt>null</tt>, an
     * <tt>IllegalStateException</tt> is thrown.</p>
     *
     * <p>
     * If the value returned by {@link #getTimer} is <tt>null</tt>, a default
     * <tt>Timer</tt> is used.</p>
     *
     * @param p The <tt>Packet</tt> to send.
     * @param l The <tt>MonitorListener</tt> to call when any condition is met.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws IllegalStateException If any required properties are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public void send(Packet p, MonitorListener l) throws IllegalArgumentException, IllegalStateException {
        long        interval;
        Timer       timer;
        MonitorTask match;
        
        //Validate listener
        if (l == null)
            throw new IllegalArgumentException("MonitorListener cannot be null");

        synchronized (this) {
            match = createMonitorTask(p, l);
            interval = getProcessInterval();
            timer = getTimer();
            if (timer == null)
                timer = getDefaultTimer();
        }

        //Send and watch
        timer.schedule(match, 0, interval);
        match.send(p);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt>, and waits for its matching result.
     * This method calls {@link PacketRouter#process} until one of the
     * following events occur:
     * <ul>
     * <li>The <tt>Packet</tt> matching the expected criteria is found.</li>
     * <li>The timeout period is exceeded.</li>
     * <li>There is a failure in the <tt>PacketRouter</tt>.</li>
     * </ul></p>
     *
     * <p>
     * The value of <tt>p</tt> may be <tt>null</tt>.</p>
     *
     * <p>
     * If the value returned by {@link #getDispatcher} is <tt>null</tt>, an
     * <tt>IllegalStateException</tt> is thrown.</p>
     *
     * <p>
     * If the value returned by {@link #getRouter} is <tt>null</tt>, an
     * <tt>IllegalStateException</tt> is thrown.</p>
     *
     * @param p The <tt>Packet</tt> to send and watch for.
     * @return The matching <tt>Packet</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws IllegalStateException If any required properties are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public Packet sendAndWatch(Packet p) throws IllegalArgumentException, IllegalStateException, StreamException {
        MonitorTask match;
        
        synchronized (this) {
            match = createMonitorTask(p, null);
        }
        match.send(p);
        
        return watch(match);
    }
    
    /**
     * <p>
     * Creates a <tt>MonitorTask</tt> using the given <tt>Packet</tt> and
     * <tt>MonitorListener</tt>. The object returned is used for both
     * synchronous and asynchronous monitoring.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> may be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>l</tt> may be <tt>null</tt>.</p>
     *
     * @param p The <tt>Packet</tt> to be sent.
     * @param l The <tt>MonitorListener</tt> to register.
     * @throws IllegalStateException If any required properties are invalid.
     * @see #sendAndWatch(Packet)
     * @see #send(Packet, MonitorListener)
     * @since   JSO-0.12
     */
    protected abstract MonitorTask createMonitorTask(Packet p, MonitorListener l)  throws IllegalStateException;
    /**
     * <p>
     * Watches on the given <tt>MonitorTask</tt> until some monitoring event
     * is triggered, blocking the current thread.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before a matching <tt>Packet</tt> was found.</p>
     *
     * <p>
     * The value of <tt>MonitorTask</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param task The task to monitor.
     * @return The matched <tt>Packet</tt>, or <tt>null</tt> if the timeout
     * is exceeded.
     * @throws StreamException If there are any errors processing the
     * configured <tt>PacketRouter</tt>.
     * @since   JSO-0.12
     */
    protected synchronized final Packet watch(MonitorTask task) throws StreamException {
        MonitorEvent    evt;

        //...and watch
        while (task.isRunning()) {
            task.run();
            
            try {
                Thread.sleep(getProcessInterval());
            } catch (InterruptedException ie) {
                obtainLogger().error("sleep thread interrupted", ie);
            }
        }
        
        //Get results and check
        evt = task.getEvent();
        if (evt.getReason() == MonitorEvent.FAILED)
            throw evt.getCause();
        
        return evt.getActualPacket();
    }
}
