/*
 * PacketException.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.util.List;

import org.jabberstudio.jso.JSOImplementation;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;

/**
 * <p>
 * Exception for throwing <tt>PacketError</tt> objects. The text of the
 * <tt>PacketError</tt> can be immediately retrieved using
 * <tt>#getErrorText</tt>. The code for the <tt>PacketError</tt> can be
 * immediately retrieved using {@link #getErrorCode}.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.6
 */
public class PacketException extends java.lang.Exception {
    //Fields
    private PacketError     _PE;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>PacketException</tt> with the given
     * <tt>PacketError</tt>. This constructor uses {@link
     * PacketError#getText} for the exception's message.</p>
     *
     * <p>
     * The value of <tt>pe</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param pe The <tt>PacketError</tt> being thrown.
     * @throws NullPointerException If <tt>pe</tt> is <tt>null</tt>.
     * @since   JSO-0.6
     */
    public PacketException(PacketError pe) throws NullPointerException {
        this(pe.getText(), pe);
    }
    /**
     * <p>
     * Creates a new <tt>PacketException</tt> with the given message and
     * <tt>PacketError</tt>.</p>
     *
     * <p>
     * The value of <tt>msg</tt> may be <tt>null</tt>, in which case
     * the exception's message is <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>pe</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param msg The message for this <tt>PacketException</tt>.
     * @param pe The <tt>PacketError</tt> being thrown.
     * @throws NullPointerException If <tt>pe</tt> is <tt>null</tt>.
     * @since   JSO-0.8
     */
    public PacketException(String msg, PacketError pe) throws NullPointerException {
        super(msg);
        
        if (pe == null)
            throw new NullPointerException("PacketError cannot be null");
        
        _PE = pe;
    }
    /**
     * <p>
     * Creates a new <tt>PacketException</tt> with the given error type and
     * condition. This constructor creates a <tt>PacketError</tt>
     * using the given type and condition.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>cond</tt> must be a valid defined condition for
     * <tt>PacketError</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param type The type for the contained <tt>PacketError</tt>.
     * @param cond The defined condition for the contained <tt>PacketError</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public PacketException(PacketError.Type type, String cond) throws IllegalArgumentException, NullPointerException {
        this(null, JSOImplementation.getInstance().getDataFactory().createPacketError(type, cond));
    }
    /**
     * <p>
     * Creates a new <tt>PacketException</tt> with the given message,
     * error type and condition. This constructor creates a <tt>PacketError</tt>
     * using the given type and condition.</p>
     *
     * <p>
     * The value of <tt>msg</tt> may be <tt>null</tt>, in which case
     * the exception's message is <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>cond</tt> must be a valid defined condition for
     * <tt>PacketError</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param msg The message for this <tt>PacketException</tt>.
     * @param type The type for the contained <tt>PacketError</tt>.
     * @param cond The defined condition for the contained <tt>PacketError</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public PacketException(String msg, PacketError.Type type, String cond) throws IllegalArgumentException {
        this(msg, JSOImplementation.getInstance().getDataFactory().createPacketError(type, cond));
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>PacketError</tt> for this
     * <tt>PacketException</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>PacketError</tt> being thrown.
     * @since   JSO-0.6
     */
    public PacketError getPacketError() {
        return _PE;
    }
    
    /**
     * <p>
     * Retrieves the type for the contained <tt>PacketError</tt>. This is
     * a convenience method to immediately retrieve the value.</p>
     *
     * @return The type for the <tt>PacketError</tt>
     * @since   JSO-0.12
     */
    public PacketError.Type getErrorType() {
        return getPacketError().getType();
    }
    /**
     * <p>
     * Retrieves the defined error condition for the contained
     * <tt>PacketError</tt>. This is a convenience method to immediately
     * retrieve the defined condition.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The defined error condition for the <tt>PacketError</tt>.
     * @since   JSO-0.12
     */
    public String getDefinedCondition() {
        return getPacketError().getDefinedCondition();
    }
    /**
     * <p>
     * Retrieves the application-specific condition for the contained
     * <tt>PacketError</tt>. This is a convenience method to immediately
     * retrieve the application-specific condition.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The application-specific condition for the
     * <tt>PacketError</tt>.
     * @since   JSO-0.12
     */
    public NSI getApplicationCondition() {
        return getPacketError().getApplicationCondition();
    }
    /**
     * <p>
     * Retrieves the list of error conditions for the contained
     * <tt>PacketError</tt>. This is a convenience method to immediately
     * retrieve the conditions.</p>
     *
     * <tt>
     * The value returned by this method is never <tt>null</tt>. Each element
     * of the returned list is an <tt>ErrorCondition</tt> instance.</p>
     *
     * @return The list of error conditions for the <tt>PacketError</tt>.
     * @since   JSO-0.12
     */
    public List listConditions() {
        return getPacketError().listConditions();
    }
    
    /**
     * <p>
     * Retrieves the code for the contained <tt>PacketError</tt>. This is
     * a convenience method to immediately retrieve the value.</p>
     *
     * @return The code for the <tt>PacketError</tt>
     * @since   JSO-0.8
     * @deprecated As of JSO-0.12, this method is removed. Use the error type
     * and condition(s) from {@link #getPacketError} instead.
     */
    public int getErrorCode() {
        return getPacketError().getCode();
    }
    /**
     * <p>
     * Retrieves the text for the contained <tt>PacketError</tt>. This is
     * a convenience method to immediately retrieve the value.</p>
     *
     * @return The text for the <tt>PacketError</tt>
     * @since   JSO-0.8
     */
    public String getErrorText() {
        return getPacketError().getText();
    }
}