/*
 * PacketMonitor.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;

import java.util.Timer;
import java.util.TimerTask;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.event.MonitorEvent;
import org.jabberstudio.jso.event.MonitorListener;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.event.PacketListener;
import org.jabberstudio.jso.event.PacketMonitorEvent;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Class to monitor a <tt>PacketRouter</tt> and <tt>PacketDispatcher</tt> for
 * a packet matching a given "template". This class implements both synchronous
 * and asynchronous modes. The asynchronous mode utilitzes the class
 * <tt>java.util.Timer</tt> to perform its monitoring.</p>
 *
 * <p>
 * The matching criteria for a <tt>PacketMonitor</tt> is based on the
 * <tt>Packet</tt> to be sent. A matching <tt>Packet</tt> is one that
 * has the same <tt>NSI</tt> as the sent <tt>Packet</tt>, and has the same
 * value for {@link org.jabberstudio.jso.StreamElement#getID} (if the sent
 * <tt>Packet</tt> includes a non-null and non-empty ID).</p>
 *
 * <p>
 * This class is thread-safe. A single instance may be used by multiple
 * threads. Changing any of the properties for a given <tt>PacketMonitor</tt>
 * does not impact any monitoring requests currently pending.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class PacketMonitor extends Monitor {
    //Inner-classes
    private class Matcher extends MonitorTask {
        //Fields
        private Packet              _Expect;

        //Constructors
        public Matcher(Packet expect, MonitorListener l) {
            super(l);
            
            Logger      log = obtainLogger();
            
            _Expect     = expect;
            if (log.isDebugEnabled())
                log.debug("packet matcher against " + expect);
        }
        
        //Methods
        public final Packet getExpected() {
            return _Expect;
        }
        
        protected boolean matched(Packet actual) {
            Logger  log = obtainLogger();
            Packet  expected = getExpected();
            String  id1, id2;
            NSI     name1, name2;
            boolean check = true;
            
            //Check NSI (ns-loosely)
            name1 = expected.getNSI();
            name2 = actual.getNSI();
            if (log.isDebugEnabled())
                log.debug("validating NSI (expected=" + name1 + ";actual=" + name2 + ")");
            check = (NSI.NS_LOOSE_COMPARATOR.compare(name1, name2) == 0);
            
            //Check IDs (if present)
            id1 = expected.getID();
            if (check && Utilities.isValidString(id1)) {
                id2 = actual.getID();
                if (log.isDebugEnabled())
                    log.debug("validating ID (expected=" + id1 + ";actual=" + id2 + ")");
                check = Utilities.equateStrings(id1, id2);
            }
            
            
            return check;
        }
        
        protected void fireMonitorFound(Packet actual) {
            MonitorEvent    evt = PacketMonitorEvent.createFoundEvent(PacketMonitor.this, getExpected(), actual);
            MonitorListener l = getListener();
            
            stop(evt);
            if (l != null)
                l.monitorFound(evt);
        }
        protected void fireMonitorTimeout() {
            MonitorEvent    evt = PacketMonitorEvent.createTimeoutEvent(PacketMonitor.this, getExpected());
            MonitorListener l = getListener();
            
            stop(evt);
            if (l != null)
                l.monitorTimeout(evt);
        }
        protected void fireMonitorFailed(StreamException cause) {
            MonitorEvent    evt = PacketMonitorEvent.createFailedEvent(PacketMonitor.this, getExpected(), cause);
            MonitorListener l = getListener();
            
            stop(evt);
            if (l != null) 
                l.monitorFailed(evt);
        }
    }
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>PacketMonitor</tt> with all fields set to default
     * values.</p>
     *
     * @since   JSO-0.12
     */
    public PacketMonitor() {
    }
    
    //Methods
    /**
     * <p>
     * Creates a <tt>MonitorTask</tt> using the given <tt>Packet</tt> and
     * <tt>MonitorListener</tt>. The object returned is used for both
     * synchronous and asynchronous monitoring.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>p</tt> may be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>l</tt> may be <tt>null</tt>.</p>
     *
     * @param p The <tt>Packet</tt> to be sent.
     * @param l The <tt>MonitorListener</tt> to register.
     * @throws IllegalStateException If any required properties are invalid.
     * @see #sendAndWatch(Packet)
     * @see #send(Packet, MonitorListener)
     * @since   JSO-0.12
     */
    protected MonitorTask createMonitorTask(Packet p, MonitorListener l) throws IllegalArgumentException, IllegalStateException {
        PacketRouter        router;
        PacketDispatcher    dispatch;
        Matcher             matcher;
        
        //Validate input
        if (p == null)
            throw new IllegalArgumentException("Expected Packet cannot be null");

        router = getRouter();
        dispatch = getDispatcher();

        //Validate router and dispatcher
        if (router == null)
            throw new IllegalStateException("PacketRouter is not valid");
        if (dispatch == null)
            throw new IllegalStateException("PacketDispatcher is not valid");

        return new Matcher(p, l);
    }


    /**
     * <p>
     * Sends the given <tt>Packet</tt> using the given <tt>Stream</tt>, and
     * waits for its matching result. This method creates a new
     * <tt>PacketMonitor</tt>, using <tt>s</tt> for the <tt>PacketRouter</tt>
     * and <tt>PacketDispatcher</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before any matches were found.</p>
     *
     * <p>
     * The value of <tt>s</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param s The <tt>Stream</tt> monitor.
     * @param p The <tt>Packet</tt> to send and watch for.
     * @return The <tt>Packet</tt> matching <tt>expected</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public static final Packet sendAndWatch(Stream s, Packet p) throws IllegalArgumentException, StreamException {
        return sendAndWatch(s, p, DEFAULT_TIMEOUT, DEFAULT_PROC_INTERVAL);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt> using the given <tt>Stream</tt>, and
     * waits for its matching result. This method creates a new
     * <tt>PacketMonitor</tt>, using <tt>s</tt> for the <tt>PacketRouter</tt>
     * and <tt>PacketDispatcher</tt>, and <tt>t</tt> for the timeout.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before any matches were found.</p>
     *
     * <p>
     * The value of <tt>s</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>t</tt> may be less than 0, in which case it is
     * considered to be -1.</p>
     *
     * @param s The <tt>Stream</tt> monitor.
     * @param p The <tt>Packet</tt> to send and watch for.
     * @param t The timeout period, in milliseconds.
     * @return The <tt>Packet</tt> matching <tt>expected</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public static final Packet sendAndWatch(Stream s, Packet p, long t) throws IllegalArgumentException, StreamException {
        return sendAndWatch(s, p, t, DEFAULT_PROC_INTERVAL);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt> using the given <tt>Stream</tt>, and
     * waits for its matching result. This method creates a new
     * <tt>PacketMonitor</tt>, using <tt>s</tt> for the <tt>PacketRouter</tt>
     * and <tt>PacketDispatcher</tt>, <tt>t</tt> for the timeout, and <tt>i</tt>
     * for the process interval.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before any matches were found.</p>
     *
     * <p>
     * The value of <tt>s</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>t</tt> may be less than 0, in which case it is
     * considered to be -1.</p>
     *
     * <p>
     * The value of <tt>i</tt> may be less than 1, in which case it is
     * considered to be {@link #DEFAULT_PROC_INTERVAL}.</p>
     *
     * @param s The <tt>Stream</tt> monitor.
     * @param p The <tt>Packet</tt> to send and watch for.
     * @param t The timeout period, in milliseconds.
     * @param i The process interval, in milliseconds.
     * @return The <tt>Packet</tt> matching <tt>expected</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public static final Packet sendAndWatch(Stream s, Packet p, long t, long i) throws IllegalArgumentException, StreamException {
        if (s == null)
            throw new IllegalArgumentException("Stream cannot be null");
        
        return sendAndWatch(s, s, p, t, i);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt> using the given <tt>PacketRouter</tt>,
     * and waits for its matching result on <tt>PacketDispatcher</tt>. This
     * method creates a new <tt>PacketMonitor</tt>, using <tt>r</tt> for the
     * <tt>PacketRouter</tt>, and <tt>d</tt> for the
     * <tt>PacketDispatcher</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before any matches were found.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param r The <tt>PacketRouter</tt> monitor.
     * @param d The <tt>PacketDispatcher</tt> monitor.
     * @param p The <tt>Packet</tt> to send and watch for.
     * @return The <tt>Packet</tt> matching <tt>expected</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public static final Packet sendAndWatch(PacketRouter r, PacketDispatcher d, Packet p) throws IllegalArgumentException, StreamException {
        return sendAndWatch(r, d, p, DEFAULT_TIMEOUT, DEFAULT_PROC_INTERVAL);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt> using the given <tt>PacketRouter</tt>,
     * and waits for its matching result on <tt>PacketDispatcher</tt>. This
     * method creates a new <tt>PacketMonitor</tt>, using <tt>r</tt> for the
     * <tt>PacketRouter</tt>, <tt>d</tt> for the <tt>PacketDispatcher</tt>, and
     * <tt>t</tt> for the timeout.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before any matches were found.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>t</tt> may be less than 0, in which case it is
     * considered to be -1.</p>
     *
     * @param r The <tt>PacketRouter</tt> monitor.
     * @param d The <tt>PacketDispatcher</tt> monitor.
     * @param p The <tt>Packet</tt> to send and watch for.
     * @param t The timeout period, in milliseconds.
     * @return The <tt>Packet</tt> matching <tt>expected</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public static final Packet sendAndWatch(PacketRouter r, PacketDispatcher d, Packet p, long t) throws IllegalArgumentException, StreamException {
        return sendAndWatch(r, d, p, t, DEFAULT_PROC_INTERVAL);
    }
    /**
     * <p>
     * Sends the given <tt>Packet</tt> using the given <tt>PacketRouter</tt>,
     * and waits for its matching result on <tt>PacketDispatcher</tt>. This
     * method creates a new <tt>PacketMonitor</tt>, using <tt>r</tt> for the
     * <tt>PacketRouter</tt>, <tt>d</tt> for the <tt>PacketDispatcher</tt>,
     * <tt>t</tt> for the timeout, and <tt>i</tt> for the process interval.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the timeout
     * was reached before any matches were found.</p>
     *
     * <p>
     * The value of <tt>r</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>d</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>p</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>t</tt> may be less than 0, in which case it is
     * considered to be -1.</p>
     *
     * <p>
     * The value of <tt>i</tt> may be less than 1, in which case it is
     * considered to be {@link #DEFAULT_PROC_INTERVAL}.</p>
     *
     * @param r The <tt>PacketRouter</tt> monitor.
     * @param d The <tt>PacketDispatcher</tt> monitor.
     * @param p The <tt>Packet</tt> to send and watch for.
     * @param t The timeout period, in milliseconds.
     * @param i The process interval, in milliseconds.
     * @return The <tt>Packet</tt> matching <tt>expected</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws StreamException If the <tt>PacketRouter</tt> fails.
     * @since   JSO-0.12
     */
    public static final Packet sendAndWatch(PacketRouter r, PacketDispatcher d, Packet p, long t, long i) throws IllegalArgumentException, StreamException {
        PacketMonitor   mon;

        if (r == null)
            throw new IllegalArgumentException("PacketRouter cannot be null");
        if (d == null)
            throw new IllegalArgumentException("PacketDispatcher cannot be null");
        
        mon = new PacketMonitor();
        mon.setRouter(r);
        mon.setDispatcher(d);
        mon.setTimeout(t);
        mon.setProcessInterval(i);
        
        return mon.sendAndWatch(p);
    }
}