/*
 * Service.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 * Class to assist in finding and instantiating SPI providers.</p>
 * 
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class Service {
    //"Constants"
    private static final Pattern    RE_CLASS = Pattern.compile("[a-zA-Z0-9_\\$\\.]+");
    
    //Fields
    private ClassLoader _Source;
    private Class       _Target;
    
    private Map         _Providers = new java.util.TreeMap();
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>Service</tt> object with the given target class and
     * source classloader.</p>
     *
     * <p>
     * The value of <tt>target</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>src</tt> may be <tt>null</tt>, in which case the
     * system classloader is used.</p>
     *
     * @param target The target class.
     * @param src The source classloader.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public Service(Class target, ClassLoader src) throws IllegalArgumentException {
        setTarget(target);
        setSource(src);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the target class for this <tt>Service</tt>. This is the class
     * each provider must implement to be found and loaded.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The target class.
     * @since   JSO-0.12
     */
    public Class getTarget() {
        return _Target;
    }
    private void setTarget(Class target) throws IllegalArgumentException {
        if (target == null)
            throw new IllegalArgumentException("Target class cannot be null");
        _Target = target;
    }
    
    /**
     * <p>
     * Retrieves the source classloader for this <tt>Service</tt>. This is the
     * classloader used to find and load the providers.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The source classloader.
     * @since   JSO-0.12
     */
    public ClassLoader getSource() {
        return _Source;
    }
    private void setSource(ClassLoader src) {
        _Source = (src == null) ? getClass().getClassLoader() : src;
    }

    
    /**
     * <p>
     * Retrieves the map of located providers.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each entry in
     * the returned map uses the fully-qualified class name for the key and the
     * <tt>Class</tt> instance for the value. The returned map cannot be
     * modified.</p>
     *
     * @return The map of providers.
     * @since   JSO-0.12
     */
    public synchronized Map getProviders() {
        if (_Providers.isEmpty()) {
            load(_Providers);
            _Providers = Collections.unmodifiableMap(_Providers);
        }
        return _Providers;
    }
    /**
     * <p>
     * Creates an instance of the given provider class name.</p>
     *
     * <p>
     * The value of <tt>name</tt> must map to a fully-qualified class name that
     * implements the target class, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param name The fully-qualified provider class to instantiate.
     * @return An instance of the provider class.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws IllegalStateException If the instance cannot be created.
     * @since   JSO-0.12
     */
    public synchronized Object obtainProvider(String name) throws IllegalArgumentException {
        Object  provInst = null;
        Class   provCls;
        
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("implementation name cannot be null");
        
        provCls = (Class)getProviders().get(name);
        if (provCls == null)
            throw new IllegalArgumentException("Provider " + name + " not available");
        try {
            provInst = provCls.newInstance();
        } catch(InstantiationException ie) {
            throw new IllegalStateException("Cannot instantiate provider " + name);
        } catch(IllegalAccessException iae) {
            throw new IllegalStateException("Cannot access provider " + name);
        }
        
        return provInst;
    }
    
    private final void load(Map provs) {
        ClassLoader     source = getSource();
        String          svc = "META-INF/services/" + getTarget().getName();
        Enumeration     svcs;
        URL             url;

        try {
            provs.clear();
            svcs = source.getResources(svc);
            while (svcs.hasMoreElements()) {
                url = (URL)svcs.nextElement();
                
                loadURL(url, provs);
            }
        } catch (IOException ioe) {
        }
    }
    private final void loadURL(URL url, Map provs) {
        BufferedReader  input;
        Matcher         match;
        String          line, test;
        Class           provCls;
        Object          provInst;
        
        try {
            input = new BufferedReader(new InputStreamReader(url.openStream()));
            
            while ((line = input.readLine()) != null) {
                if (line.startsWith("#"))
                    continue;
                match = RE_CLASS.matcher(line);
                if (match.find()) {
                    try {
                        test = match.group().trim();
                        provCls = getSource().loadClass(test);
                        provInst = provCls.newInstance();
                        provs.put(provCls.getName(), provCls);
                    } catch(ClassNotFoundException cnfe) {
                    } catch(InstantiationException ie) {
                    } catch(IllegalAccessException iae) {
                    }
                }
            }
        } catch (IOException ioe) {
        }
    }
}
