/*
 * StreamSourceProxy.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.io.IOException;

import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.io.StreamSource;

/**
 * <p>
 * Class to proxy an existing {@link StreamSource}. This is used to provide
 * additional support (such as security layer wrapping) on top of
 * another <tt>StreamSource</tt>.</p>
 *
 * <p>
 * All methods from <tt>StreamSource</tt> are implemented to call their peers
 * on the proxied <tt>StreamSource</tt>. Subclasses that override methods
 * should call the same method on the proxied <tt>StreamSource</tt> before
 * returning.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class StreamSourceProxy implements StreamSource {
    //Fields
    private StreamSource        _Proxied;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>StreamSourceProxy</tt> over the given
     * <tt>StreamSource</tt>.</p>
     *
     * <p>
     * The value of <tt>proxied</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param proxied The <tt>StreamSource</tt> to be proxied.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public StreamSourceProxy(StreamSource proxied) throws IllegalArgumentException {
        if (proxied == null)
            throw new IllegalArgumentException("Proxied StreamSource cannot be null");
        _Proxied = proxied;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the proxied <tt>StreamSource</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The proxied <tt>StreamSource</tt>.
     * @since   JSO-0.12
     */
    public StreamSource getProxiedStreamSource() {
        return _Proxied;
    }
    
    /**
     * <p>
     * Retrieves the "expected" hostname for this connection. This
     * implementation calls <tt>getHostname()</tt> on {@link
     * #getProxiedStreamSource}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>
     * or "".</p>
     *
     * @return The "remote" hostname.
     * @since   JSO-0.12
     */    
    public String getHostname() {
        return getProxiedStreamSource().getHostname();
    }
    
    /**
     * <p>
     * Connects this <code>StreamSource</code> for input and output. This
     * impementation calls <tt>connect(Stream)</tt> on {@link
     * #getProxiedStreamSource}.</p>
     *
     * <p>
     * The value of <tt>s</tt> must not be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * If there are any I/O errors in opening this <code>StreamSource</code>,
     * this method throws an <code>IOException</code>.  All other errors
     * result in some form of <code>RuntimeException</code> being thrown.</p>
     *
     * @param s The <tt>Stream</tt> to connect to.
     * @throws IOException If any I/O errors occur trying to open this
     * <code>StreamSource</code>.
     * @throws Exception If any other errors occur trying to open this
     * <tt>StreamSource</tt>.
     * @since   JSO-0.12
     */    
    public void connect(Stream s) throws IOException, Exception {
        getProxiedStreamSource().connect(s);
    }
    /**
     * <p>
     * Disconnects this <tt>StreamSource</tt> from input and output. This
     * implementation calls <tt>disconnect(Stream)</tt> on {@link
     * #getProxiedStreamSource}.</p>
     *
     * <p>
     * The value of <tt>s</tt> must not be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * If there are any I/O errors in closing this <code>StreamSource</code>,
     * this method throws an <code>IOException</code>.  All other errors
     * result in some form of <code>RuntimeException</code> being thrown.</p>
     *
     * @param s The <tt>Stream</tt> to disconnect from.
     * @throws IOException If any I/O errors occur trying to close this
     * <code>StreamSource</code>.
     * @throws Exception If any other errors occur trying to close this
     * <tt>StreamSource</tt>.
     * @since   JSO-0.12
     */
    public void disconnect(Stream s) throws IOException, Exception {
        getProxiedStreamSource().disconnect(s);
    }
    
    /**
     * <p>
     * Deterines if this <tt>StreamSource</tt> is ready to attempt a read.
     * This implementation calls <tt>ready()</tt> on {@link
     * #getProxiedStreamSource}.</p>
     *
     * @return <tt>true</tt> if this <tt>StreamSource</tt> can still be used.
     * @since JSO-0.12
     */
    public boolean ready() throws IOException {
        return getProxiedStreamSource().ready();
    }
    /**
     * <p>
     * Reads bytes from this <tt>StreamSource</tt> into the specified byte
     * buffer array. This implementation calls <tt>read(byte[], int, int)</tt>
     * on {@link #getProxiedStreamSource}.</p>
     *
     * <p>
     * This method returns the number of bytes read, or -1 if the end of the
     * input stream is reached.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>offset</tt> and <tt>length</tt> must represent a valid
     * range within <tt>buffer</tt>, or an <tt>IllegalArgumentException</tt>
     * is thrown. For <tt>offset</tt> the valid ranges are:</p>
     *
     * <table border='0' cellpadding='0' cellspacing='0'>
     * <tr><th>offset</th><td><code>0 <= offset < (buffer.length - length)</code></td></tr>
     * <tr><th>length</th><td><code>0 <= length < (buffer.length - offset)</code></td></tr>
     * </table>
     *
     * @param buffer The buffer to read to.
     * @param offset The offset into <tt>buffer</tt> to start at.
     * @param length The length within <tt>buffer</tt> to use.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws IOException If there are any I/O errors.
     * @since JSO-0.12
     */
    public int read(byte[] buffer, int offset, int length) throws IllegalArgumentException, IOException {
        return getProxiedStreamSource().read(buffer, offset, length);
    }
    /**
     * <p>
     * Writes bytes to this <tt>StreamSource</tt> from the specified byte
     * buffer array. This implementation calls <tt>write(byte[], int, int)</tt>
     * on {@link #getProxiedStreamSource}.</p>
     *
     * <p>
     * This method returns the number of bytes written.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>offset</tt> and <tt>length</tt> must represent a valid
     * range within <tt>buffer</tt>, or an <tt>IllegalArgumentException</tt>
     * is thrown. For <tt>offset</tt> the valid ranges are:</p>
     *
     * <table border='0' cellpadding='0' cellspacing='0'>
     * <tr><th>offset</th><td><code>0 <= offset < (buffer.length - length)</code></td></tr>
     * <tr><th>length</th><td><code>0 <= length < (buffer.length - offset)</code></td></tr>
     * </table>
     *
     * @param buffer The buffer to write from.
     * @param offset The offset into <tt>buffer</tt> to start at.
     * @param length The length within <tt>buffer</tt> to use.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws IOException If there are any I/O errors.
     * @since JSO-0.12
     */
    public int write(byte[] buffer, int offset, int length) throws IllegalArgumentException, IOException {
        return getProxiedStreamSource().write(buffer, offset, length);
    }
}