/*
 * MappingTable.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

import java.util.Arrays;

/**
 * <p>
 * Class for encapsulating a mapping rule.</p>
 *
 * <p>
 * Subclasses must implement {@link #map} to exchange the characters in the
 * provided <tt>StringBuffer</tt> to their mappings.</p>
 *
 * <p>
 * A default implementation that maps from a character table to a string
 * table is provided via {@link #create}.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12.3
 */
public abstract class MappingTable {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>MappingTable</tt>.</p>
     *
     * @since   JSO-0.12.3
     */
    protected MappingTable() {
    }

    //Methods
    /**
     * <p>
     * Maps the given input.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Typically,
     * <tt>in</tt> is returned.</p>
     *
     * @param   in The <tt>StringBuffer</tt> to map.
     * @return  The mapped <tt>StringBuffer</tt>, usually <tt>in</tt>.
     * @throws  NullPointerException If <tt>in</tt> is <tt>null</tt>.
     * @since   JSO-0.12.3
     */
    public abstract StringBuffer map(StringBuffer in) throws NullPointerException;


    /**
     * <p>
     * Creates a new <tt>MappingTable</tt> based on the given tables.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The returned
     * <tt>MappingTable</tt>'s <tt>map(StringBuffer)</tt> iterates through
     * the characters in the provided <tt>StringBuffer</tt>, exchanging them
     * if their corresponding value from <tt>to</tt>. If a character is not
     * found in <tt>from</tt>, it is not touched.</p>
     *
     * <p>
     * If <tt>from</tt> is <tt>null</tt>, an <tt>IllegalArgumentException</tt>
     * is thrown. If <tt>from</tt> is not the same length as <tt>to</tt>, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If <tt>to</tt> is <tt>null</tt>, an <tt>IllegalArgumentException</tt>
     * is thrown. If <tt>to</tt> is not the same length as <tt>to</tt>, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param   from The characters to map from.
     * @param   to The strings to map to.
     * @return  The <tt>MappingTable</tt> over <tt>from</tt> and <tt>to</tt>.
     * @throws  IllegalArgumentException If <tt>from</tt> or <tt>to</tt> is
     * not valid.
     * @since   JSO-0.12.3
     */
    public static MappingTable create(final char[] from, final String[] to) throws IllegalArgumentException {
        if ((from == null) || (to == null))
            throw new IllegalArgumentException("from and to arrays cannot be null");
        if (from.length != to.length)
            throw new IllegalArgumentException("from and to arrays must be the same length");

        return new MappingTable() {
            public StringBuffer map(StringBuffer in) throws NullPointerException {
                int             len = in.length();
                char[]          data = new char[len];
                char            find;
                String          token, replace;
                int             idx, pos;
                
                in.getChars(0, len, data, 0);
                in.setLength(0);
                for (idx = 0; idx < data.length; idx++) {
                    pos = Arrays.binarySearch(from, data[idx]);
                    
                    if (pos >= 0) {
                        if ((replace = to[pos]) != null)
                            in.append(replace);
                    } else {
                        in.append(data[idx]);
                    }
                }
                
                return in;
            }
        };
    }
}
