/*
 * NFKC.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Class that performs Normalization Form KC.</p>
 *
 * <p>
 * <b>NOTE:</b> This class is a derivative work from GNU Libidn.</p>
 *
 * @author  linuxwolf
 * @author  Oliver Hitz
 * @version $VERSION$
 * @since   JSO-0.12.3
 */
public class NFKC {
    //Constructors
    private NFKC() {
    }

    //Methods
    /**
     * <p>
     * Normalization the given string.</p>
     *
     * @param   in The string to normalize.
     * @return  The normalized string.
     * @since   JSO-0.12.3
     */
    public static String normalize(String in) {
        StringBuffer   work;
        String          out;

        if (Utilities.isValidString(in)) {
            //Decompose
            work = decompose(in);

            // Bring the stringbuffer into canonical order.
            order(work);

            // Do the canonical composition.
            out = compose(work);
        } else {
            out = in;
        }

        return out;
    }

    private static final StringBuffer decompose(CharSequence in) {
        StringBuffer   out = new StringBuffer();
	int len = in.length();

        for (int i = 0; i < len; i++) {
            char code = in.charAt(i);

            // In Unicode 3.0, Hangul was defined as the block from U+AC00
            // to U+D7A3, however, since Unicode 3.2 the block extends until
            // U+D7AF. The decomposeHangul function only decomposes until
            // U+D7A3. Should this be changed?
            if (code >= 0xAC00 && code <= 0xD7AF) {
                out.append(Hangul.decomposeHangul(code));
            } else {
                Decomposition.decompose(code, out);
            }
        }

        return out;
    }
    private static final StringBuffer order(StringBuffer in) {
        boolean     isOrdered = false;
	int         len = in.length()-1;

        while (!isOrdered) {
            int lastCC = CombiningClass.get(in.charAt(0));

            isOrdered = true;
            for (int i = 0; i < len; i++) {
                int nextCC = CombiningClass.get(in.charAt(i+1));
                if (nextCC != 0 && lastCC > nextCC) {
                    for (int j = i+1; j > 0; j--) {
                        if (CombiningClass.get(in.charAt(j-1)) <= nextCC) {
                            break;
                        }
                        char t = in.charAt(j);
                        in.setCharAt(j, in.charAt(j-1));
                        in.setCharAt(j-1, t);
                        isOrdered = false;
                    }
                    nextCC = lastCC;
                }
                lastCC = nextCC;
            }
        }

        return in;
    }
    private static final String compose(StringBuffer out) {
        int     lastCC = 0;
        int     lastStart = 0;
	int     len = out.length();

        for (int i = 0; i < len; i++) {
            int cc = CombiningClass.get(out.charAt(i));

            if ((i > 0) && ((lastCC == 0) || (lastCC != cc))) {
                // Try to combine characters
                char    a = out.charAt(lastStart);
                char    b = out.charAt(i);
                int     c;

                c = Hangul.composeHangul(a, b);
                c = ((c == -1) ? Composition.compose(a, b) : c);

                if (c != -1) {
                    out.setCharAt(lastStart, (char) c);
                    out.deleteCharAt(i);
                    i--;

                    lastCC = ((i == lastStart) ? 0 : CombiningClass.get(out.charAt(i-1)));
                    continue;
                }
            }

            if (cc == 0)
                lastStart = i;
            lastCC = cc;
        }

        return out.toString();
    }

    //Inner-classes
        /**
         * Entire hangul code copied from:
         * http://www.unicode.org/unicode/reports/tr15/
         *
         */
    private static class Hangul {
        //"Constants"
        private static final int SBase = 0xAC00;
        private static final int LBase = 0x1100;
        private static final int VBase = 0x1161;
        private static final int TBase = 0x11A7;
        private static final int LCount = 19;
        private static final int VCount = 21;
        private static final int TCount = 28;
        private static final int NCount = VCount * TCount;
        private static final int SCount = LCount * NCount;

        //Constructors

        //Methods
        /**
         * Decomposes a hangul character.
         *
         * @param s A character to decompose.
         * @return A string containing the hangul decomposition of the input
         * character. If no hangul decomposition can be found, a string
         * containing the character itself is returned.
         */
        private static final String decomposeHangul(char s) {
            int SIndex = s - SBase;
            if (SIndex < 0 || SIndex >= SCount) {
                return String.valueOf(s);
            }
            StringBuffer result = new StringBuffer();
            int L = LBase + SIndex / NCount;
            int V = VBase + (SIndex % NCount) / TCount;
            int T = TBase + SIndex % TCount;
            result.append((char)L);
            result.append((char)V);
            if (T != TBase) result.append((char)T);
            return result.toString();
        }

        /**
         * Composes two hangul characters.
         *
         * @param a First character.
         * @param b Second character.
         * @return Returns the composed character or -1 if the two
         * characters cannot be composed.
         */
        private static final int composeHangul(char a, char b) {
            // 1. check to see if two current characters are L and V
            int LIndex = a - LBase;
            if (0 <= LIndex && LIndex < LCount) {
                int VIndex = b - VBase;
                if (0 <= VIndex && VIndex < VCount) {
                    // make syllable of form LV
                    return SBase + (LIndex * VCount + VIndex) * TCount;
                }
            }

            // 2. check to see if two current characters are LV and T
            int SIndex = a - SBase;
            if (0 <= SIndex && SIndex < SCount && (SIndex % TCount) == 0) {
                int TIndex = b - TBase;
                if (0 <= TIndex && TIndex <= TCount) {
                    // make syllable of form LVT
                    return a+TIndex;
                }
            }
            return -1;
        }
    }
}
