/*
 * AuthQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.util.DigestHash;
import org.jabberstudio.jso.util.Enumerator;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Interface for representing "jabber:iq:auth" queries. This interface is
 * designed to more conveniently retrieve and store authentication information
 * over an {@link org.jabberstudio.jso.Extension}. In addition, the inner class
 * {@link AuthQuery.Method} abstracts the setup of the authentication
 * credentials (other than username and resource).</p>
 *
 * <p>
 * Note that any modifications to the data contained by an instance of
 * <code>AuthQuery</code> is not automatically reflected back to the
 * "server" or "component".  The <code>AuthQuery</code> (wrapped in an
 * appropriate <code>InfoQuery</code>) must still be sent for changes to have
 * any real effect.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public interface AuthQuery extends Extension {
    //Inner-classes
    /**
     * <p>
     * Class to represent and establish a "method" of authenticating (such as
     * "password", "digest", or "zero-knowledge") via the "jabber:iq:auth"
     * namespace extension.</p>
     *
     * @author  linuxwolf
     * @version $VERSION$
     * @since   JSO-0.4
     */
    public abstract static class Method extends Enumerator {
        //Constructors
        /**
         * <p>
         * Creates a new <code>Method</code> with the given name.  The value
         * of <code>name</code> is what is returned by {@link #toString}.</p>
         *
         * <p>
         * The value of <code>name</code> cannot be null or "", or an
         * <code>IllegalArgumentException</code> is thrown.</p>
         *
         * @param name The name of this <code>Method</code>.
         * @throws IllegalArgumentException If the parameter is invalid.
         * @since   JSO-0.4
         */
        public Method(String name) throws IllegalArgumentException {
            super(name);
        }
        
        //Methods
        /**
         *
         */
        protected void resetAuth(AuthQuery aq) {
            String      username, resource;
            
            username = aq.getUsername();
            resource = aq.getResource();
            
            aq.reset();
            aq.setUsername(username);
            aq.setResource(resource);
        }
        /**
         * <p>
         * Determines if the given <code>AuthQuery</code> supports this
         * <code>Method</code>.  This determination is usually done by examing
         * the available fields in <code>aq</code>.</p>
         *
         * <p>
         * The value of <code>aq</code> cannot be <code>null</code>, or an
         * <code>IllegalArgumentException</code> is thrown.</p>
         *
         * @param aq The <code>AuthQuery</code> to examine.
         * @return <code>true</code> if <code>aq</code> supports this
         * <code>Method</code>.
         * @since   JSO-0.4
         */
        public abstract boolean isSupported(AuthQuery aq) throws IllegalArgumentException;

        /**
         * <p>
         * Generates a map of parameters for setting up the authentication
         * against the given <code>AuthQuery</code>.</p>
         *
         * <p>
         * The returned map contains the auth paramter name as the key, and
         * the initial value for the parameter as the value.  This initial
         * may be <code>null</code>.  The value returned by this method is
         * never <code>null</code>.</p>
         *
         * <p>
         * The value of <code>aq</code> cannot be <code>null</code> or an
         * <code>IllegalArgumentException</code> is thrown.</p>
         *
         * @param aq the <code>AuthQuery</code> to setup parameters against.
         * @return The map of authentication parameters.
         * @throws IllegalArgumentException If the parameter is invalid.
         * @throws IllegalStateException If <code>aq</code> does not support
         * this <code>Method</code>.
         * @since   JSO-0.4
         */
        public abstract Map setupAuthParams(AuthQuery aq) throws IllegalArgumentException, IllegalStateException;
        /**
         * <p>
         * Sets up the <code>AuthQuery</code> for authentication using the
         * given parameters. This method performs most of the calculations
         * (if any are required) and sets fields in <code>aq</code> to their
         * appropriate values.</p>
         *
         * <p>
         * Note that this method modifies the given <code>AuthQuery</code>.
         * It is recommended to create a copy if the original data is
         * required.</p>
         *
         * <p>
         * The value of <code>aq</code> cannot be <code>null</code> or an
         * <code>IllegalArgumentException</code> is thrown.</p>
         *
         * <p>
         * The value of <code>params</code> cannot be <code>null</code> or an
         * <code>IllegalArgumentException</code> is thrown.  In addition,
         * <code>params</code> usually must contain values for those keys
         * generated by {@link #setupAuthParams}, or an
         * <code>IllegalArgumentException</code> may be thrown.</p>
         * 
         * @param aq The <code>AuthQuery</code> to setup.
         * @param params The parameters for authentication.
         * @throws IllegalArgumentException If any parameters are invalid.
         * @throws IllegalStateException If <code>aq</code> does not support
         * this <code>Method</code>.
         * @since   JSO-0.4
         */
        public abstract void setupAuth(AuthQuery aq, Map params) throws IllegalArgumentException, IllegalStateException;

        /**
         * <p>
         * Generates a map of parameters for setting up the authentication
         * against the given <code>AuthQuery</code>.</p>
         *
         * <p>
         * The returned map contains the auth paramter name as the key, and
         * the initial value for the parameter as the value.  This initial
         * may be <code>null</code>.  The value returned by this method is
         * never <code>null</code>.</p>
         *
         * <p>
         * The value of <code>aq</code> cannot be <code>null</code> or an
         * <code>IllegalArgumentException</code> is thrown.</p>
         *
         * @param aq the <code>AuthQuery</code> to setup parameters against.
         * @return The map of authentication parameters.
         * @throws IllegalArgumentException If the parameter is invalid.
         * @throws IllegalStateException If <code>aq</code> does not support
         * this <code>Method</code>.
         * @since   JSO-0.4
         */
        public abstract Map setupFieldParams(AuthQuery aq) throws IllegalArgumentException;
        /**
         * <p>
         * Sets up the <code>AuthQuery</code> to initialize authentication
         * using the given parameters. This method results in <code>aq</code>
         * supporting this <code>Method</code> (e.g. {@link #isSupported}
         * returns <code>true</code>).
         *
         * <p>
         * Note that this method modifies the given <code>AuthQuery</code>.
         * It is recommended to create a copy if the original data is
         * required.</p>
         *
         * <p>
         * The value of <code>aq</code> cannot be <code>null</code> or an
         * <code>IllegalArgumentException</code> is thrown.</p>
         *
         * <p>
         * The value of <code>params</code> cannot be <code>null</code> or an
         * <code>IllegalArgumentException</code> is thrown.  In addition,
         * <code>params</code> usually must contain values for those keys
         * generated by {@link #setupFieldParams}, or an
         * <code>IllegalArgumentException</code> may be thrown.</p>
         * 
         * @param aq The <code>AuthQuery</code> to setup.
         * @param params The parameters for authentication.
         * @throws IllegalArgumentException If any parameters are invalid.
         * @since   JSO-0.4
         */
        public abstract void setupFields(AuthQuery aq, Map params) throws IllegalArgumentException;
    }
    
    //"Constants"
    /**
     * Namespace governing <code>AuthQuery</code>
     * @since   JSO-0.4
     */
    public static final String  NAMESPACE = "jabber:iq:auth";
    /**
     * Qualified name for <tt>AuthQuery</tt> ({jabber:iq:auth}query).
     * @since   JSO-0.12
     */
    public static final NSI     NAME = new NSI("query", NAMESPACE);
    
    /** <code>Method</code> governing "none" authentiation.
     * @since   JSO-0.4
     */
    public static final Method  NONE = new Method("none") {
        public boolean isSupported(AuthQuery aq) throws IllegalArgumentException {
            //Validate authquery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            return true;
        }
        public Map setupAuthParams(AuthQuery aq) throws IllegalArgumentException, IllegalStateException {
            Map     params = new java.util.TreeMap();
            
            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());

            return params;
        }
        public void setupAuth(AuthQuery aq, Map params) throws IllegalArgumentException, IllegalStateException {
            Object      pwd;

            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());
            
            resetAuth(aq);
        }
        public Map setupFieldParams(AuthQuery aq) throws IllegalArgumentException {
            Map     params = new java.util.TreeMap();
            
            return params;
        }
        public void setupFields(AuthQuery aq, Map params) throws IllegalArgumentException {
            //Validate AuthQuery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
        }
    };
    /** <code>Method</code> governing "plain" (or "password") authentiation.
     * @since   JSO-0.4
     */
    public static final Method  PLAIN = new Method("plain") {
        public boolean isSupported(AuthQuery aq) throws IllegalArgumentException {
            Set     names;
            
            //Validate authquery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            names = aq.getFieldNames();
            
            return names.contains("password");
        }
        public Map setupAuthParams(AuthQuery aq) throws IllegalArgumentException, IllegalStateException {
            Map     params = new java.util.TreeMap();
            
            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());

            params.put("password", null);
            return params;
        }
        public void setupAuth(AuthQuery aq, Map params) throws IllegalArgumentException, IllegalStateException {
            Object      pwd;

            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());
            
            //Validate parameters
            if (params == null)
                throw new IllegalArgumentException("Parameters cannot be null");
            if (!params.containsKey("password"))
                throw new IllegalArgumentException("Paramters must contain a 'password'");

            //Setup password
            pwd = params.get("password");
            
            resetAuth(aq);
            aq.setPassword((pwd != null ? pwd.toString() : null));
        }
        public Map setupFieldParams(AuthQuery aq) throws IllegalArgumentException {
            Map     params = new java.util.TreeMap();
            
            params.put("digest", null);
            
            return params;
        }
        public void setupFields(AuthQuery aq, Map params) throws IllegalArgumentException {
            Object      password = null;
            
            //Validate AuthQuery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            
            //Setup password
            if ((params != null) && params.containsKey("password")) {
                password = params.get("password");
            }

            aq.setPassword((password != null) ? password.toString() : null);
        }
    };
    /** <code>Method</code> governing "digest" authentiation.
     * @since   JSO-0.4
     */
    public static final Method DIGEST = new Method("digest") {
        public boolean isSupported(AuthQuery aq) throws IllegalArgumentException {
            Set     names;
            
            //Validate authquery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            names = aq.getFieldNames();
            
            return names.contains("digest");
        }
        public Map setupAuthParams(AuthQuery aq) throws IllegalArgumentException, IllegalStateException {
            Map     params = new java.util.TreeMap();

            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());

            params.put("sessionid", null);
            params.put("password", null);
            return params;
        }
        public void setupAuth(AuthQuery aq, Map params) throws IllegalArgumentException, IllegalStateException {
            Object      sessionid, password;
            String      digest;

            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());
            
            //Validate parameters
            if (params == null)
                throw new IllegalArgumentException("Parameters cannot be null");
            if (!params.containsKey("sessionid"))
                throw new IllegalArgumentException("Paramters must contain a 'sessionid'");
            if (!params.containsKey("password"))
                throw new IllegalArgumentException("Paramters must contain a 'password'");
            
            //Setup digest
            sessionid = params.get("sessionid");
            password = params.get("password");
            digest = DigestHash.SHA1.hash((sessionid != null ? sessionid.toString() : "") + (password != null ? password.toString() : ""));
            
            resetAuth(aq);
            aq.setDigest(digest);
        }
        public Map setupFieldParams(AuthQuery aq) throws IllegalArgumentException {
            Map     params = new java.util.TreeMap();
            
            params.put("digest", null);
            
            return params;
        }
        public void setupFields(AuthQuery aq, Map params) throws IllegalArgumentException {
            Object      digest = null;
            
            //Validate AuthQuery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            
            //Setup password
            if ((params != null) && params.containsKey("digest")) {
                digest = params.get("digest");
            }
            
            aq.setDigest((digest != null) ? digest.toString() : null);
        }
    };
    /** <code>Method</code> governing "zero knowledge" authentiation.
     * @since   JSO-0.4
     * @deprecated As of JSO-0.12, this field is deprecated because it is not
     * a secure method.
     */
    public static final Method  ZERO_KNOWLEDGE = new Method("zero knowledge") {
        public boolean isSupported(AuthQuery aq) throws IllegalArgumentException {
            Set     names;
            
            //Validate authquery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            names = aq.getFieldNames();
            
            return names.contains("token") && names.contains("sequence");
        }
        public Map setupAuthParams(AuthQuery aq) throws IllegalArgumentException, IllegalStateException {
            Map     params = new java.util.TreeMap();

            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());

            params.put("password", null);
            return params;
        }
        public void setupAuth(AuthQuery aq, Map params) throws IllegalArgumentException, IllegalStateException {
            DigestHash  hasher = DigestHash.SHA1;
            Object      password;
            String      temp;
            String      hash;
            String      token;
            int         sequence = 0, idx;

            //Verify supported
            if (!isSupported(aq))
                throw new IllegalStateException("AuthQuery does not support this method:  " + toString());
            
            //Validate parameters
            if (params == null)
                throw new IllegalArgumentException("Parameters cannot be null");
            if (!params.containsKey("password"))
                throw new IllegalArgumentException("Paramters must contain a 'password'");
            
            //Setup all inputs
            password = params.get("password");
            token = aq.getField("token");
            temp = aq.getField("sequence");
            try { sequence = Integer.parseInt(temp); } catch (NumberFormatException nfe) {}
            
            //Generate initial hash
            hash = hasher.hash(password != null ? password.toString() : "");
            hash = hasher.hash(hash + token);
            
            //Recursively hash the hash
            for (idx = 0; idx < sequence; idx++) {
                hash = hasher.hash(hash);
            }

            //Setup AuthQuery
            resetAuth(aq);
            aq.setField("hash", hash);
        }
        public Map setupFieldParams(AuthQuery aq) throws IllegalArgumentException {
            Map     params = new java.util.TreeMap();
            
            params.put("token", null);
            params.put("sequence", null);
            
            return params;
        }
        public void setupFields(AuthQuery aq, Map params) throws IllegalArgumentException {
            Object      temp;
            String      token, sequence;
            
            //Validate AuthQuery
            if (aq == null)
                throw new IllegalArgumentException("AuthQuery cannot be null");
            
            //Validate Parameters
            if (params == null)
                throw new IllegalArgumentException("Parameters cannot be null");
            
            //Obtain and validate Token
            temp = params.get("token");
            token = (temp != null) ? temp.toString() : "";
            if (!Utilities.isValidString(token))
                throw new IllegalArgumentException("Token cannot be null or \"\"");
            
            //Object and validate Sequence
            temp = params.get("sequence");
            sequence = (temp != null) ? temp.toString() : "";
            if (!Utilities.isValidString(sequence))
                throw new IllegalArgumentException("Sequence cannot be null or \"\"");
            
            //Setup AuthQuery
            aq.setField("token", token);
            aq.setField("sequence", sequence);
        }
    };

    //Methods
    /**
     * <p>
     * Retrieves the username for this <tt>AuthQuery</tt>. This corresponds to
     * the field "username".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the username has not been set.</p>
     *
     * @return The username for this <tt>AuthQuery</tt>.
     * @since   JSO-0.4
     */
    public String getUsername();
    /**
     * <p>
     * Sets the username for this <tt>AuthQuery</tt>.</p>
     *
     * <p>
     * The value of <Code>usr</code> may be <tt>null</tt> or "". Note that
     * calling this method always results in the inclusion of the "username"
     * field. Call {@link #clearUsername} to remove this field.</p>
     *
     * @param usr The new username.
     * @since   JSO-0.4
     */
    public void setUsername(String usr);
    /**
     * <p>
     * Removes the username from this <tt>AuthQuery</tt>.</p>
     *
     * @since   JSO-0.4
     */
    public void clearUsername();

    /**
     * <p>
     * Retrieves the resource for this <tt>AuthQuery</tt>.. This corresponds to
     * the field "resource".</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the resource has not been set.</p>
     *
     * @return The resource for this <code>AuthQuery</code>.
     * @since   JSO-0.4
     */
    public String getResource();
    /**
     * <p>
     * Sets the resource for this <tt>AuthQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>res</tt> may be <tt>null</tt> or "".  Note that calling
     * this method always results in the inclusion of the "resource" field.
     * Call {@link #clearResource} to remove this field.</p>
     *
     * @param res The new resource.
     * @since   JSO-0.4
     */
    public void setResource(String res);
    /**
     * <p>
     * Removes the resource from this <tt>AuthQuery</tt>.</p>
     *
     * @since   JSO-0.4
     */
    public void clearResource();
    
    /**
     * <p>
     * Retrieves the password for this <tt>AuthQuery</tt>. This corresponds to
     * the field "password", and contains the plain-text password.</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the password has not been set.</p>
     *
     * @return The password for this <tt>AuthQuery</tt>.
     * @since   JSO-0.12
     */
    public String getPassword();
    /**
     * <p>
     * Sets the password for this <tt>AuthQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>pass</tt> may be <tt>null</tt> or "".  Note that calling
     * this method always results in the inclusion of the "password" field.
     * Call {@link #clearPassword} to remove this field.</p>
     *
     * @param pass The new password.
     * @since   JSO-0.12
     */
    public void setPassword(String pass);
    /**
     * <p>
     * Removes the password from this <tt>AuthQuery</tt>.</p>
     *
     * @since   JSO-0.4
     */
    public void clearPassword();
    
    /**
     * <p>
     * Retrieves the digest for this <tt>AuthQuery</tt>. This corresponds to
     * the field "digest", and contains the (SHA-1) digest.</p>
     *
     * <p>
     * This method returns <tt>null</tt> if the digest has not been set.</p>
     *
     * @return The digest for this <tt>AuthQuery</tt>.
     * @since   JSO-0.12
     */
    public String getDigest();
    /**
     * <p>
     * Sets the digest for this <tt>AuthQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>res</tt> may be <tt>null</tt> or "".  Note that calling
     * this method always results in the inclusion of the "digest" field. Call
     * {@link #clearDigest} to remove this field.</p>
     *
     * @param digest The new digest.
     * @since   JSO-0.12
     */
    public void setDigest(String digest);
    /**
     * <p>
     * Removes the digest from this <tt>AuthQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearDigest();
    
    /**
     * <p>
     * Retrieves the set of field names for this <code>AuthQuery</code>.
     * The returned set reflects only those fields currently present.</p>
     *
     * <p>
     * "Fields" are any contained elements in the same namespace as
     * <code>AuthQuery</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return The set of field names present.
     * @since   JSO-0.4
     */
    public Set getFieldNames();
    /**
     * <p>
     * Retrieves the value for the given field in this
     * <code>AuthQuery</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if field
     * is not present.</p>
     *
     * @param name The field name.
     * @return The field value.
     * @throws IllegalArgumentException if the parameter is invalid.
     * @since   JSO-0.4
     */
    public String getField(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Sets the value for the given field in this <code>AuthQuery</code>.
     * This method always results in the field being "set".  To remove a field,
     * use {@link #unsetField} instead.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param name The field name.
     * @throws IllegalArgumentException if the parameter is invalid.
     * @since   JSO-0.4
     */
    public void setField(String name, String value) throws IllegalArgumentException;
    /**
     * <p>
     * Removes the field from this <code>AuthQuery</code>.</p>
     *
     * <p>
     * The value of <code>name</code> cannot be <code>null</code> or "", or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param name The field name.
     * @throws IllegalArgumentException if the parameter is invalid.
     * @since   JSO-0.4
     */
    public void unsetField(String name) throws IllegalArgumentException;
}