/*
 * DiscoInfoQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.disco;


import java.util.List;
import java.util.Set;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;

/**
 * <p>
 * Interface for representing Service Discovery information queries, as
 * documented in JEP-0030: Service Discovery.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0030.html'>JEP-0030</a>
 */
public interface DiscoInfoQuery extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>DiscoInfoQuery</tt>
     * (http://jabber.org/protocol/disco#info).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE = DiscoUtilities.INFO_NAMESPACE;
    /**
     * The qualified name for <tt>DiscoInfoQuery</tt>
     * ({http://jabber.org/protocol/disco#info}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME = DiscoUtilities.INFO_NAME;
    
    //Methods
    /**
     * <p>
     * Retrieves the node for this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The node for this <tt>DiscoInfoQuery</tt>.
     * @since   JSO-0.12
     */
    public String getNode();
    /**
     * <p>
     * Sets the node for this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>node</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param node The new node.
     * @since   JSO-0.12
     */
    public void setNode(String node);
    
    /**
     * <p>
     * Retrieves the name for this <tt>DiscoInfoQuery</tt>. The name for an
     * entire <tt>DiscoInfoQuery</tt> matches the name for the first
     * <tt>Identity</tt> listed.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The name for this <tt>DiscoInfoQuery</tt>.
     * @since   JSO-0.12
     */
    public String getName();
    /**
     * <p>
     * Sets the name for this <tt>DiscoInfoQuery</tt>. This method modifies all
     * contained <tt>Identity</tt> objects to match this value. It may also
     * store this value internally, for use with <tt>Identity</tt> objects
     * added later.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * it is "reset" to that from the first contained <tt>Identity</tt>.</p>
     *
     * @param name The new name.
     * @since   JSO-0.12
     */
    public void setName(String name);

    /**
     * <p>
     * Retrieves the list of identities for this <tt>DiscoInfoQuery</tt>. This
     * method is equivalent to calling {@link #listIdentities(String, String)}
     * with both arguments <tt>null</tt> or "".</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non-<tt>null</tt> instance of
     * <tt>DiscoIdentity</tt>.</p>
     *
     * @return The list of identities.
     * @since   JSO-0.12
     */
    public List listIdentities();
    /**
     * <p>
     * Retrieves the list of identities for this <tt>DiscoInfoQuery</tt>,
     * matching the given category and/or type.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non-<tt>null</tt> instance of
     * <tt>DiscoIdentity</tt>.</p>
     *
     * <p>
     * The value of <tt>cat</tt> may be <tt>null</tt> or "", in which case
     * the category value is ignored.</p>
     *
     * <p>
     * The value of <tt>type</tt> may be <tt>null</tt> or "", in which case
     * the type value is ignored.</p>
     *
     * @param cat The category to match.
     * @param type The type to match.
     * @return The list of identities.
     * @since   JSO-0.12
     */
    public List listIdentities(String cat, String type);
    /**
     * <p>
     * Retrieves the identity for the given category and type in this
     * <tt>DiscoInfoQuery</tt>..</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a match cannot
     * be found.</p>
     *
     * <p>
     * The value of <tt>cat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param cat The category to find.
     * @param type The type to find.
     * @return The <tt>DiscoIdentity</tt> found for <tt>cat</tt> and
     * <tt>type</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoIdentity getIdentity(String cat, String type) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and adds an identity to this <tt>DiscoInfoQuery</tt>. The name
     * for this <tt>DiscoIdentity</tt> is that from {@link #getName}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If an
     * <tt>DiscoIdentity</tt> with the given category and type is already
     * present, that instance is returned unmodified.</p>
     *
     * <p>
     * The value of <tt>cat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param cat The category to add.
     * @param type The type to add.
     * @return The new <tt>DiscoIdentity</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoIdentity addIdentity(String cat, String type) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and adds an identity to this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If an
     * <tt>DiscoIdentity</tt> with the given category and type is already
     * present, that instance is returned unmodified.</p>
     *
     * <p>
     * The value of <tt>cat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case the
     * value from {@link #getName} is used.</p>
     *
     * @param cat The category to add.
     * @param type The type to add.
     * @param name The name to add.
     * @return The new <tt>DiscoIdentity</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoIdentity addIdentity(String cat, String type, String name) throws IllegalArgumentException;
    /**
     * <p>
     * Removes an identity from this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>cat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param cat The category to remove.
     * @param type The type to remove.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void removeIdentity(String cat, String type) throws IllegalArgumentException;
    /**
     * <p>
     * Removes all <tt>DiscoIdentity</tt> objects from this
     * <tt>DiscoInfoQuery</tt>. If the name is stored interally, it is not
     * removed.</p>
     *
     * @since   JSO-0.12
     */
    public void clearIdentities();

    /**
     * <p>
     * Retrieves the set of features for this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a string.</p>
     *
     * @return The list of features.
     * @since   JSO-0.12
     */
    public Set getFeatures();
    /**
     * <p>
     * Adds a feature to this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>feat</tt> may be <tt>null</tt> or "", in which case no
     * actions are performed. If <tt>feat</tt> is already present, this method
     * does nothing.</p>
     *
     * @param feat The feature to add.
     * @since   JSO-0.12
     */
    public void addFeature(String feat);
    /**
     * <p>
     * Removes a feature from this <tt>DiscoInfoQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>feat</tt> may be <tt>null</tt> or "", in which case no
     * actions are performed.</p>
     *
     * @param feat The feature to remove.
     * @since   JSO-0.12
     */
    public void removeFeature(String feat);
    /**
     * <p>
     * Removes all features from this <tt>DiscoInfoQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearFeatures();
}