/*
 * TimeQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.info;


import java.util.Date;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:iq:time" queries, as documented in
 * JEP-0090.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0090.html">JEP-0090: Entity Time</a>
 */
public interface TimeQuery extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>TimeQuery</tt> (jabber:iq:time).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE = "jabber:iq:time";
    /**
     * The qualified name for <tt>TimeQuery</tt> ({jabber:iq:time}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME = new NSI("query", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves or generates the date/time for this <tt>TimeQuery</tt>. This
     * is the value of the &lt;utc/&gt; element, parsed into a <tt>Date</tt>
     * object.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the date/time
     * has not yet been set.</p>
     *
     * @return The date/time.
     * @since   JSO-0.12
     */
    public Date getTime();
    /**
     * <p>
     * Sets the date/time for this <tt>TimeQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>time</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param time The new date/time.
     * @since   JSO-0.12
     */
    public void setTime(Date time);
    
    /**
     * <p>
     * Retrieves the timezone identifier for this <tt>TimeQuery</tt>. This
     * is the value of the &lt;tz/&gt; element, and is usually the three-letter
     * or descriptive name of the timezone. This value may or may not be a valid
     * timezone identitier as used by the <tt>java.util.TimeZone</tt> class.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be ""
     * if the value has not been set.</p>
     *
     * @return The timezone identifier.
     * @since   JSO-0.12
     */
    public String getTimezone();
    /**
     * <p>
     * Sets the timezone identifier for this <tt>TimeQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>tz</tt> may be <tt>null</tt> or "", in which case it is
     * considered to be "".</p>
     *
     * @param tz The new timezone identifier.
     * @since   JSO-0.12
     */
    public void setTimezone(String tz);
    
    /**
     * <p>
     * Retrieves the display for this <tt>TimeQuery</tt>. This is the value of
     * the &lt;display/&gt; element, and is usually the human-readable,
     * locale-specific format of {@link #getTime}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be ""
     * if the value has not been set.</p>
     *
     * @return The display.
     * @since   JSO-0.12
     */
    public String getDisplay();
    /**
     * <p>
     * Sets the display for this <tt>TimeQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>tz</tt> may be <tt>null</tt> or "", in which case it is
     * considered to be "".</p>
     *
     * @param display The new display.
     * @since   JSO-0.12
     */
    public void setDisplay(String display);
}