/*
 * JidFunction.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import java.util.List;

import org.jaxen.Context;
import org.jaxen.FunctionCallException;
import org.jaxen.Navigator;
import org.jaxen.function.StringFunction;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Class implementing the XPath function "jid([object])". This XPath function
 * converts the given argument into a <tt>JID</tt>.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class JidFunction extends AbstractFunction {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>JidFunction</tt> object.</p>
     *
     * @since   JSO-0.12
     */
    public JidFunction() {
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the "typical" function name for this <tt>JidFunction</tt>.</p>
     *
     * <p>
     * The value returned by this method is always "jid".</p>
     *
     * @return The "typical" function name.
     * @since   JSO-0.12
     */
    public String getFunctionName() {
        return "jid";
    }
    
    /**
     * <p>
     * Called during XPath evaluation to convert argument list into a
     * <tt>JID</tt>.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> should not be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>args</tt> should not be <tt>null</tt>. If <tt>args</tt>
     * is empty, the context node-set is used. If <tt>args</tt> contains more
     * than one value, a <tt>FunctionCallException</tt> is thrown.</p>
     *
     * @param ctx The Jaxen <tt>Context</tt> in use.
     * @param args The list of arguments to evaluate.
     * @return The <tt>JID</tt> representing the given argument list or
     * context node-set.
     * @throws FunctionCallException If the evaluation fails.
     * @since   JSO-0.12
     */
    public Object call(Context ctx, List args) throws FunctionCallException {
        Navigator   nav = ctx.getNavigator();
        Object      eval;
        int         size = args.size();
        
        if (size > 1)
            throw new FunctionCallException("jid() requires zero or one argument");

        eval = (size == 0) ? ctx.getNodeSet() : args.get(0);
        
        return evaluate(eval, nav);
    }
    
    /**
     * <p>
     * Converts the given argument object into a <tt>JID</tt>. This method
     * first attempts to convert the given argument into its string
     * representation before converting to a <tt>JID</tt>.</p>
     *
     * <p>
     * The value of <tt>arg</tt> should not be <tt>null</tt>, or a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>nav</tt> cannot be <tt>null</tt> or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param arg The argument to convert.
     * @param nav The Jaxen <tt>Navigator</tt> to use.
     * @return The <tt>JID</tt> representing <tt>arg</tt>.
     * @throws FunctionCallException If the argument cannot be converted to a
     * <tt>JID</tt>.
     * @throws NullPointerException If any required parameters are
     * <tt>null</tt>.
     * @since   JSO-0.12
     */
    public static JID evaluate(Object arg, Navigator nav) throws FunctionCallException {
        JID         jid = null;
        String      value;
        
        try {
            if (arg instanceof JID) {
                jid = (JID)arg;
            } else {
                value = StringFunction.evaluate(arg, nav);

                if (Utilities.isValidString(value))
                    jid = new JID(value);
            }
        } catch (IllegalArgumentException iae) {
            throw new FunctionCallException("Cannot evaluate to a JID.");
        }
        
        return jid;
    }
}