/*
 * MatchLocaleFunction.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.jaxen.Context;
import org.jaxen.FunctionCallException;
import org.jaxen.Navigator;

import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Class implementing the XPath function "locale-match(object[, object])". This
 * XPath function attempts to match the first Locale object to the second, first
 * converting both to appropriate <tt>Locale</tt> instances. If there is only
 * one argument to the function call, then the test is against the context
 * node-sets' locale(s).</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class MatchLocaleFunction extends AbstractFunction {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>MatchLocaleFunction</tt> object.</p>
     *
     * @since   JSO-0.12
     */
    public MatchLocaleFunction() {
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the "typical" function name for this
     * <tt>MatchLocaleFunction</tt>.</p>
     *
     * <p>
     * The value returned by this method is always "locale-match".</p>
     *
     * @return The "typical" function name.
     */
    public String getFunctionName() {
        return "locale-match";
    }
    
    /**
     * <p>
     * Called during XPath evaluation to match two arguments as Locales.</p>
     *
     * <p>
     * The value of <tt>ctx</tt> should not be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>args</tt> should not be <tt>null</tt>. If <tt>args</tt>
     * is empty or contains more than two arguements, a
     * <tt>FunctionCallException</tt> is thrown. If <tt>args</tt> only contains
     * one element, then the context node-set is used for the second
     * argument.</p>
     *
     * @param ctx The Jaxen <tt>Context</tt> in use.
     * @param args The list of arguments to evaluate.
     * @return A <tt>Boolean</tt> value determining if the two "Locales" match.
     * @throws FunctionCallException If the evaluation fails.
     * @since   JSO-0.12
     */
    public Object call(Context ctx, List args) throws FunctionCallException {
        Navigator   nav = ctx.getNavigator();
        Object      arg1 = null, arg2 = null;
        int         size = args.size();
        
        switch (size) {
            case 1:
                arg1 = args.get(0);
                arg2 = ctx.getNodeSet();
                break;
            case 2:
                arg1 = args.get(0);
                arg2 = args.get(1);
                break;
            default:
                throw new FunctionCallException(getFunctionName() +  " requires at one or two arguments");
        }
        
        return evaluate(arg1, arg2, nav);
    }
    
    /**
     * <p>
     * Matches the two arguments as Locales. Both arguments are first converted
     * to <tt>Locale</tt> instances. The first argument is used as the "source"
     * to match against, while the second argument is the target being
     * matched.</p>
     *
     * <p>
     * The value of <tt>arg1</tt> must evaluate to a valid <tt>Locale</tt>, or a
     * <tt>FunctionCallException</tt> is thrown.</p>
     *
     * <p>
     * If the value of <tt>arg2</tt> is a <tt>List</tt> of
     * <tt>StreamElement</tt> objects, then each <tt>StreamElement</tt>'s locale
     * (as determined {@link org.jabberstudio.jso.StreamNode#getLocale}) is
     * matched against <tt>arg1</tt>. Otherwise, the object is converted to a
     * Locale.</p>
     *
     * <p>
     * The value of <tt>nav</tt> cannot be <tt>null</tt> or an
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param arg1 The "source" Locale.
     * @param arg2 The "target" object(s).
     * @param nav The Jaxen <tt>Navigator</tt> to use.
     * @return <tt>true</tt> (as a <tt>Boolean</tt> object) if the "target"
     * matches the "source", and <tt>false</tt> (as a <tt>Boolean</tt> object)
     * otherwise.
     * @throws FunctionCallException If the argument cannot be converted to a
     * <tt>JID</tt>.
     * @throws NullPointerException If any required parameters are
     * <tt>null</tt>.
     * @since   JSO-0.12
     */
    public static Boolean evaluate(Object arg1, Object arg2, Navigator nav) throws FunctionCallException {
        Locale      source, target;
        Boolean     retval = null;
        
        try {
            source = LocaleFunction.evaluate(arg1, nav);
            if (source == null)
                throw new FunctionCallException("first argument is not a valid Locale");

            if          (nav.isElement(arg2)) {
                //Convert a single element into a list of elements
                arg2 = Collections.singletonList(arg2);
            }
            
            if          (arg2 instanceof List) {
                boolean     matched = true, result = true;
                Iterator    itr = ((List)arg2).iterator();
                
                while (matched && result && itr.hasNext()) {
                    Object  node = itr.next();
                    
                    if (result = nav.isElement(node)) {
                        target = ((StreamElement)node).getLocale();
                        result = Utilities.matchLocales(source, target);
                    } else {
                        matched = false;
                    }
                }

                //Set the result only if all nodes really were elements
                if (matched)
                    retval = Boolean.valueOf(result);
            }
            
            if (retval == null) {
                //Treat second argument as a locale instance
                target = LocaleFunction.evaluate(arg2, nav);

                if (target == null)
                    throw new FunctionCallException("second argument is not a valid Locale");
                retval = Boolean.valueOf(Utilities.matchLocales(source, target));
            }
        } catch (IllegalArgumentException iae) {
            throw new FunctionCallException("Cannot evaluate arguments");
        }
        
        return retval;
    }
}